/***************************************************************************
                          audio.cpp  -  description
                             -------------------
    begin                : Sun Sep 17 2000
    copyright            : (C) 1997-1999 by Sam Lantinga
                           (Original SDL_mixer example code)
    copyright            : (C) 2000 by Erik Unemyr
    email                : unemyr@hem.passagen.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
	Instructions for using this class:
	
	1. Include the Audio.h file in your source file (no shit).

	2. Add the desired audio commands in your source file.

			//Open the Audio Device
			Audio *audio = new Audio();

			//Set Music and Audio Volume
		 	audio->setAudioVolume(32); (Audio volume from 0-128)
			audio->setMusicVolume(32); (Music volume from 0-128)

			//Play .IT Music File
		 	audio->playMusicFile("file.it");

			Play .WAV Audio File
		 	audio->playAudioFile("file.wav");

	3. SDL and SDL_mixer must be linked while compiling.

****************************************************************************/
//External Header Declarations
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//Internal Header Declarations
#include "../common/FilePath.h"
#include "../common/Log.h"
#include "../common/general.h"
#include "Audio.h"

//Sound Constructor
Audio::Audio ()
{
	//Variable Declarations
	Uint16 audio_format = AUDIO_S16;
	Uint16 audio_rate = 22050;
	Uint16 audio_channels = 2;
	Uint16 audio_buffers = 4096;

	//Initialize the SDL Library
    if(SDL_WasInit(SDL_INIT_AUDIO)!=SDL_INIT_AUDIO)
      {
  	  if(SDL_InitSubSystem(SDL_INIT_AUDIO|SDL_INIT_NOPARACHUTE)<0)
	    {
		Log::Error("Error! %s\n",SDL_GetError());
		exit(255);
   	    }
	  }

	//Open the Audio Device
	if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) < 0)
	{
		Log::Error("Error! %s\n", SDL_GetError());
		exit(2);
	}

	//Set Variable Definitions
	music = NULL;
	wave = NULL;
	audio_open = 1;
}

//Sound Destructor
Audio::~Audio ()
{
	//Fade Out Music
	if( Mix_PlayingMusic() ) {
		Mix_FadeOutMusic(2000);
		SDL_Delay(2000);
	}

	//Stop Music
	if ( music ) {
		Mix_FreeMusic(music);
		music = NULL;
	}

	//Stop Audio
	if ( wave ) {
		setAudioVolume(0);
		Mix_FreeChunk(wave);
		wave = NULL;
	}

	//Close the Audio Device
	if ( audio_open ) {
		Mix_CloseAudio();
		audio_open = 0;
	}
}

//Play Specified Audio File
void Audio::playAudioFile (const char *file, BOOL Loop)
{
    assert(file!=NULL);
 
    char *BaseFilename=new char[strlen(file)+1];
    FilePath path( BaseFilename );
    path.getValidFilePath( BaseFilename );

	//Set the Buffer To Null
	wave = NULL;
	Uint16 channel;

	//Load the Requested Wave File
	wave = Mix_LoadWAV((const char *)BaseFilename);
	if ( wave == NULL )
	{
		Log::Error("Error! %s\n", SDL_GetError());
		exit(2);
	}
	
	//Finds a Free Channel
	for (channel=0; Mix_Playing(channel) == 1; channel++);
	
	//Too Many Occupied Channels, Ignore This Audio File (Number of Channels 					
	//Hardware Specific?)
	if (channel > MIX_CHANNELS)
	{	
		fprintf (stderr,"Too many\n");
		return;
	}

	//Play and then Exit
    if(Loop==TRUE)
      {
	  Mix_PlayChannel(channel, wave,-1);
      }
    else
      {
	  Mix_PlayChannel(channel, wave,0);
      }

	delete BaseFilename;
}

//Play Specified Sound File
void Audio::playMusicFile (const char *file, BOOL Loop)
{
    assert(file!=NULL);

    char *BaseFilename=new char[strlen(file)+1];
    FilePath path( BaseFilename );
    path.getValidFilePath( BaseFilename );

	//Fade Out Current Music, If Any
	if ( music )
	{
		Mix_FadeOutMusic(2000);
		SDL_Delay(2000);
	}

	//Set the Buffer To Null
	music = NULL;

	//Load the Requested Music File
	music = Mix_LoadMUS((const char *)BaseFilename);
	if ( music == NULL )
	{
		Log::Error("Error! %s\n", SDL_GetError());
		exit(2);
	}

	//Play and then Exit
    if(Loop==TRUE)
      {
	  Mix_FadeInMusic(music,-1,2000);
      }
    else
      {
  	  Mix_FadeInMusic(music,0,2000);
      }
	delete BaseFilename;
}

//Sets the Audio Volume (Value from 0-128)
void Audio::setAudioVolume (Uint32 audiovolume)
{
	//Variable Declarations
	Uint16 channel;

	//Sets the Audio Volume
	if (audiovolume <= MIX_MAX_VOLUME && audiovolume >= 0)
	{
		for (channel = 0; channel <= MIX_CHANNELS; channel++)
		{
			Mix_Volume(channel, audiovolume);
		}
	}
}

//Sets the Music Volume (Value from 0-128)
void Audio::setMusicVolume (Uint32 musicvolume)
{
	//Sets the Music Volume
	if (musicvolume <= MIX_MAX_VOLUME && musicvolume >= 0)
		Mix_VolumeMusic(musicvolume);
}




