/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This class start the client of the game, just by calling all the 
    objects that manage the game and then giving control to the Core.
***************************************************************************/

#include "../common/Core.h"
#include "../common/general.h"
#include "../common/Log.h"
#include "../common/NetworkInit.h"
#include "../common/Screen.h"
#include "../common/DataLoader.h"
#include "Renderer.h"

#include "parseCMDLine.h"

#include <signal.h>

int main(int Num,char **arg)
  {
  signal(SIGSEGV,SIG_DFL);

#ifdef macintosh
	freopen( "stderr.txt","w", stderr );
	freopen( "stdout.txt","w", stdout );
#endif

  NetworkInit *netInit;
  Core *core;

  if(SDL_Init(SDL_INIT_NOPARACHUTE)<0)
    {
    Log::Error("Can't init SDL library.\n");
    }
  atexit(SDL_Quit);

  Log::Debug("Begin Client %s",ARIANNE_HARDCODE_VERSION);
  Log::Debug("Begin initialization stage.");
  #ifdef THREADLESS
    Log::Debug("Using threadless client version.");
  #endif

  parseCMDLine(Num,arg);
  DataLoader::Init(g_configFile);

  netInit=new NetworkInit();

  #ifndef ARIANNE_TEXT_ONLY
  Screen::Init();
  Screen::SetSize(DataLoader::GetWidth(),DataLoader::GetHeight(),DataLoader::GetDepth(),DataLoader::GetFullscreen());
  #endif

  core=new Core(netInit->GetMessageManager(),netInit->GetNetworkManager());

  Log::Debug("Begin ingame stage.");

  netInit->Connect(DataLoader::GetHostName(),DataLoader::GetHostPort());
  netInit->Run();
  core->Run();

  Log::Debug("Begin exit stage.");

  netInit->Shutdown();
  Screen::Shutdown();

  /* Makes your compiler happy. */
  return 0;
  }
