/***************************************************************************
    copyright            : (C) 2000 by 
    email                : 
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* client/Font.cpp

Project : Arianne RPG
Created : 18 Aug 2000

Font handling routines.

*/

#include "Font.h"
#include "../common/Log.h"

BOOL BitmapFont::ParseFont(void)
{
    if (widths) {
	delete(widths);
	widths=NULL;
    }
    if (charpos) {
	delete (charpos);
	charpos=NULL;
    }
    if (font==NULL) return FALSE;
    height=font->h;
    toppoint=0;
    maxchar=256;
    int i,w,p;
    widths=new int[maxchar];
    charpos=new int[maxchar];
    // full ISO-8859-1 fixed width - for now
    w=font->w/maxchar;
    /* for variable width we must increment toppoint and scan the top bitmap line
    for end-of-cell markers */
    p=0;
    for (i=0;i<maxchar;i++) {
	widths[i]=w;
	charpos[i]=p;
	p+=w;
    }
    SDL_SetColorKey(font,SDL_SRCCOLORKEY,0);
    return TRUE;
}

BitmapFont::BitmapFont(void)
{
    font=NULL;
    widths=NULL;
    charpos=NULL;
    height=0;
}

BitmapFont::BitmapFont(const char *fontfile)
{
    font=NULL;
    widths=NULL;
    charpos=NULL;
    height=0;
    Load(fontfile);
}

BitmapFont::~BitmapFont(void)
{
    if (font) SDL_FreeSurface(font);
    if (widths) delete(widths);
    if (charpos) delete (charpos);
}

BOOL BitmapFont::Load(const char *fontfile)
{
    height=0;
    if (font) SDL_FreeSurface(font);

    SDL_Surface *tmp=SDL_LoadBMP(fontfile);
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",fontfile);
      }
    font=SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);

    return ParseFont();
}

int BitmapFont::GetWidth(const char *text)
{
    int textwidth=0;
    unsigned char c;
    if (widths && text)
	for (;c=*text++;)
	    if (c < maxchar) textwidth+=widths[c];
    return textwidth;
}

void BitmapFont::Draw(SDL_Surface *surface, Sint16 x, Sint16 y, const char *text, int flags)
{
    if (surface==NULL || text==NULL || *text==NUL || widths==NULL) return;
    switch (flags & MASK_X) {
	case RIGHT:
	    x-=GetWidth(text);
	    break;
	case CENTER:
	    x-=GetWidth(text)/2;
	    break;
    }
    switch (flags & MASK_Y) {
	case BOTTOM:
	    y-=GetHeight(text);
	    break;
	case CENTER:
	    y-=GetHeight(text)/2;
	    break;
    }
    SDL_Rect sr, dr;
    unsigned char c;
    for (;c=*text++;) if (c < maxchar) {
	sr.x=charpos[c];  sr.y=toppoint;
	sr.w=widths[c];   sr.h=height;
	dr.x=x;     dr.y=y;
	dr.w=sr.w;  dr.h=sr.h;
	SDL_BlitSurface(font,&sr,surface,&dr);
	x+=widths[c];
    }
}
