/* client/Font.h

Project : Arianne RPG
Created : 18 Aug 2000

Font handling routines.

*/

#ifndef __ARIANNE_RPG_FONT_H
#define __ARIANNE_RPG_FONT_H
#ifndef __cplusplus
#error C++ compiler required.
#endif

#include "../common/general.h"

// Abstract Font class
class AFont : public Disposable {
protected:
    enum { MASK_X=3, MASK_Y=12 };
public:
    // Generic drawing flags
    enum { CENTER=0, LEFT=1, RIGHT=2, TOP=4, BOTTOM=8 }; // *** Only Left-to-Right for now
    /** Compute width of a string */
    int virtual GetWidth(const char *text) = 0;

    /** Compute height of a string */
    int virtual GetHeight(const char *text) = 0;

    /** Draw the string on a SDL surface */
    void virtual Draw(SDL_Surface *surface, Sint16 x, Sint16 y, const char *text, int flags) = 0;
};

class BitmapFont : public AFont {
protected:
    // where we keep it
    SDL_Surface *font;
    // the number of characters
    int maxchar;
    // The height is fixed
    int height;
    // The top point to be displayed
    int toppoint;
    // An array of widths, 0 means not present
    int *widths;
    // An array of character x-positions
    int *charpos;
    // Scan for character widths
    BOOL ParseFont(void);
public:
    BitmapFont(void);
    BitmapFont(const char *fontfile);
    ~BitmapFont(void);
    /** Load a (new) font from a bitmapped file */
    BOOL Load(const char *fontfile);

    int GetWidth(const char *text);
    int GetHeight(const char *text) { return height; };
    void Draw(SDL_Surface *surface, Sint16 x, Sint16 y, const char *text, int flags);
};

#endif /* __ARIANNE_RPG_FONT_H */
