/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is class is the one that render the Map and the objects to the
    screen. 
    
    This is a basic 2D topview renderer.

    Yo use just do it in the OpenGL way.
    Renderer::Begin();
    .
    Renderer::Show();
    .
    Renderer::End();
***************************************************************************/
#include "Renderer.h"
#include "../common/Screen.h"
#include "Font.h"
#include "../common/Log.h"

GFXData *Renderer::Logo;
GFXData **Renderer::Tiles;
GFXData **Renderer::Terrains;
PLAYER_SCREEN_POSITION *Renderer::PlayerScreenPos;

int Renderer::LoadData()
  {
  Log::Debug("Renderer::LoadData()");

  int j,i;

  PlayerScreenPos=new PLAYER_SCREEN_POSITION;

  Logo=new GFXData();
  Logo->ToggleKeyColorOn();
  Logo->SetNumAnimations(1);
  Logo->SetNumFrames(0,1);
  Logo->LoadFrames("gfx/logo",0);

  Tiles=new GFXData*[NUM_TILES_CHARA];
  char *FileBaseName=new char[256];

  for(i=0;i<NUM_TILES_CHARA;i++)
    {
    Tiles[i]=new GFXData();
    Tiles[i]->SetNumAnimations(4);
    for(j=0;j<4;j++)
      {
      Tiles[i]->ToggleKeyColorOn();
      Tiles[i]->SetNumFrames(j,2);
      sprintf(FileBaseName,"gfx/chara/%i",i);
      Tiles[i]->LoadFrames(FileBaseName,j);
      }
    }

  Terrains=new GFXData*[NUM_TILES_TERRAIN];
  
  for(i=0;i<NUM_TILES_TERRAIN;i++)
    {
    Terrains[i]=new GFXData();
    Terrains[i]->SetNumAnimations(1);
    Terrains[i]->SetNumFrames(0,1);
    sprintf(FileBaseName,"gfx/terrain/%i",i);
    Terrains[i]->LoadFrames(FileBaseName,0);
    }
  delete[] FileBaseName;

  return 0;
  }

/* To be used in OpenGL way. */
void Renderer::Begin()
  {
  }

void Renderer::End()
  {
  SDL_PumpEvents();
  SDL_Flip(Screen::GetScreen());
  }

void Renderer::Update(Object *object,World *world,Uint32 actualframe,Uint32 lastframe)
  {
  assert(object!=NULL);
  assert(world!=NULL);
  }

void Renderer::Show(Object *object,World *world,Uint32 actualframe,Uint32 lastframe)
  {
  assert(object!=NULL);
  assert(world!=NULL);

  AList<Terrain> *terrains;
  AList<Object> *objects;
  Uint32 i;  
  int ScreenW,ScreenH;

  Screen::GetSize(&ScreenW,&ScreenH);

  Uint32 x,y,z;
  object->Pos.GetXYZ(&x,&y,&z);

  /* We determine where to start the rendering */
  Uint32 RenderXinit,RenderYinit;
  RenderXinit=(Uint32)(x>(ScreenW/6.4)/2)?x-(ScreenW/6.4)/2:0;
  RenderYinit=(Uint32)(y>(ScreenH/6.4)/2)?y-(ScreenH/6.4)/2:0;

  terrains=world->isInRange((x>2000)?x-2000:0,(y>2000)?y-2000:0,x+2000,y+2000);

  for(i=0;i<terrains->Size();i++)
    {
    Terrain *tmp=(*terrains)[i];
    Sint32 x,y,w,h;

    Uint16 ImageW,ImageH;
    x=tmp->x1-RenderXinit;
    y=tmp->y1-RenderYinit;
    w=tmp->sizex;
    h=tmp->sizey;

    if(x*6.4f<ScreenW && y*6.4f<ScreenH)
      {
      Terrains[(Uint8)tmp->TextureIndex]->GetSize(&ImageW,&ImageH);
      ImageW=(Uint16)(ImageW/6.4);
      ImageH=(Uint16)(ImageH/6.4);
 
      for(Uint32 j=0;j<w/ImageW;j++)
        {
        for(Uint32 k=0;k<h/ImageH;k++)
          {
          Terrains[(Uint8)tmp->TextureIndex]->Show((int)((x+ImageW*j)*6.4f),(int)((y+ImageH*k)*6.4f),0,actualframe,lastframe); 
          }
        }
      }
    }

  terrains->Clear();
  delete terrains;

  objects=world->isInRange(object->Pos,2000);

  for(i=0;i<objects->Size();i++)
    {
    Object *tmp=(*objects)[i];
    Sint32 x,y,z;
    tmp->Pos.GetXYZ((Uint32*)&x,(Uint32*)&y,(Uint32*)&z);
    x=x-RenderXinit;
    y=y-RenderYinit;
    
    /* We get the representation. */
    Uint32 texIndex=tmp->GetGFXResume().GetBasicType();

    /* Now we calculate the character orientation to select anim.*/
    Uint8 yaw,roll,pitch;
    Uint8 facing;
    tmp->Pos.GetYRP(&yaw,&roll,&pitch);
    facing=roll/64;

    /* We store where the player is located on screen */
    if(object->mID==tmp->mID)
      {
      PlayerScreenPos->Player=object;
      PlayerScreenPos->x=(Uint16)(x*6.4f);
      PlayerScreenPos->y=(Uint16)(y*6.4f);
      }

    Tiles[texIndex]->Show((int)(x*6.4f),(int)(y*6.4f),facing,actualframe,lastframe); 
    }

  objects->Clear();
  delete objects;

  Logo->Show(470,3,0,0);
  }

void Renderer::GetPlayerPos(Object *Player,Uint16 *x,Uint16 *y)
  {
  if(Player->mID==PlayerScreenPos->Player->mID)
    {
    x[0]=PlayerScreenPos->x;
    y[0]=PlayerScreenPos->y;
    }
  else
    {
    x[0]=0;
    y[0]=0;
    }
  }
