/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This an abstraction of a common AVector variable.
***************************************************************************/
#include "AVector.h"
#include "AUint32.h"
#include "Bzip2Support.h"
#include <string.h>

AVector::AVector()
  {
  vector=NULL;
  i=0;
  size=0;
  MaxSize=0;
  }

AVector::~AVector()
  {
  if(vector)
    {
    delete[] vector;
    vector=0;
    }
      
  i=0;
  size=0;
  MaxSize=0;
  }

Uint8 *AVector::Get()
  {
  return vector;
  }


Uint32 AVector::GetVectorSize()
  {
  return size;
  }


void AVector::SetVectorSize(Uint32 size)
  {
  this->size=size;
  MaxSize=size;

  if(vector)
    {
    delete[] vector;
    }

  vector=new Uint8[size];
  i=0;
  }

void AVector::SetIndex(Uint32 i)
  {
  this->i=i;
  }
  
void AVector::AddUint8(Uint8 *data,Uint32 size)
  {
  assert(data!=NULL);

  memcpy(&vector[i],data,size);
  i+=size;
  }

void AVector::GetUint8(Uint8 *data,Uint32 dsize)
  {
  assert(data!=NULL);
  assert(i+dsize<=size);

  memcpy(data,&vector[i],dsize);
  i+=dsize;
  }

int AVector::operator+(AVector* vec)
  {
  assert(vec!=NULL);

  if(MaxSize<=this->GetSize()+vec->GetSize())
    {
    /* We alloc extra memory to avoid wasting CPU on copying */
    MaxSize=this->GetSize()+vec->GetSize();
    MaxSize+=(MaxSize*EXTRA_MEM_USAGE)/100;

    Uint8 *tmp=new Uint8[MaxSize];
    i=0;
    memcpy(tmp,vector,GetSize());
    i+=GetSize();
    memcpy(&tmp[i],vec->vector,vec->GetSize());
  
    size=this->GetSize()+vec->GetSize();
    
    if(vector)
      {
      delete[] vector;
      }

    vector=tmp;
    }
  else
    {
    memcpy(&vector[size],vec->vector,vec->GetSize());
    size=this->GetSize()+vec->GetSize();
    }

  i=0;

  return 0;
  }

Uint8 AVector::operator[](int index)
  {
  assert((Uint32)index<=size);

  return vector[index];
  }

Uint32 AVector::GetSize()
  {
  /* As you can note, the size should be +4 bytes.
     But as the size is used after applying the 
     Tovector method, the size is correct.
     
     NOTE: this isn't always true.
     TODO: Look for case of true of the above statement.
     */

  return GetVectorSize();
  }

void AVector::ToVector(AVector *Vector)
  {
  assert(Vector!=NULL);

  AUint32 vsize=this->size;

  vsize.ToVector(Vector);
  (*Vector)+this;
  }

void AVector::FromVector(AVector *Vector)
  {
  assert(Vector!=NULL);

  if(vector)
    {
    delete[] vector;
    }

  AUint32 vsize;
  vsize.FromVector(Vector);

  size=(Uint32)vsize;
  MaxSize=size;
  vector=new Uint8[size];

  Vector->GetUint8(vector,size);
  i=0;
  }

void AVector::Reset()
  {
  if(vector)
    {
    delete[] vector;
    vector=0;
    }

  size=0;
  MaxSize=size;
  i=0;
  }

AVector *AVector::Compress(Uint8 Compress)
  {
  AVector *compress;

  if(Compress==1)
    {
    Bzip2Support bzip2;
    bzip2.Begin();
    compress=bzip2.Encode(this);
    bzip2.End();
    }
  else
    {
    compress=new AVector;
    (*compress)+this;
    }

  assert(compress!=NULL);

  return compress;
  }

AVector *AVector::Decompress(Uint8 Decompress)
  {
  AVector *decompress;

  if(Decompress==1)
    {
    Bzip2Support bzip2;
    bzip2.Begin();
    decompress=bzip2.Decode(this);
    bzip2.End();
    }
  else
    {
    decompress=new AVector;
    (*decompress)+this;
    }

  assert(decompress!=NULL);

  return decompress;
  }
