/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This class start is the one that run the game itself, it does all the 
    basic functions of the client, but it delegate power on other classes.

    It can be seen as translator between players desires and messages to 
    lower layers.
***************************************************************************/
#include "Core.h"
#include "Messages.h"
#include "TimedWait.h"
#include "DataLoader.h"
#include "SysInfo.h"
#include "Log.h"
#include "../client/Renderer.h"
#include "../client/Audio.h"
#include "Actions.h"
#include "Screen.h"

// Minami's addition
#ifdef macintosh
#include <Threads.h>
#endif
// end of addition

#include "GUI/MeterSample.h"
#include "GUI/AnimatingIconSample.h"
#include "GUI/ShortCutListener.h"

Core::Core(MessageManager *MesManager, NetworkManager *NetManager)
  {
  this->MesManager=MesManager;
  this->NetManager=NetManager;

  eventProcessor=new EventProcessor();
  InitGUI();

  #ifndef ARIANNE_TEXT_ONLY
  Renderer::LoadData();  
  #endif

  mes_c2s_action=new Message_C2S_ACTION(MesManager);
  mes_c2s_ask_map_piece=new Message_C2S_ASK_MAP_PIECE(MesManager);
  mes_c2s_ask_perception=new Message_C2S_ASK_PERCEPTION(MesManager);
  mes_c2s_info_about_game=new Message_C2S_INFO_ABOUT_GAME(MesManager);
  mes_c2s_load_character=new Message_C2S_LOAD_CHARACTER(MesManager);
  mes_c2s_login=new Message_C2S_LOGIN(MesManager);
  mes_c2s_logout=new Message_C2S_LOGOUT(MesManager);
  mes_c2s_synchronism=new Message_C2S_SYNCHRONISM(MesManager);
  mes_c2s_system_info=new Message_C2S_SYSTEM_INFO(MesManager);

  mes_s2c_action=new Message_S2C_ACTION(MesManager);
  mes_s2c_load_character=new Message_S2C_LOAD_CHARACTER(MesManager);
  mes_s2c_login=new Message_S2C_LOGIN(MesManager);
  mes_s2c_logout=new Message_S2C_LOGOUT(MesManager);
  mes_s2c_map_piece=new Message_S2C_MAP_PIECE(MesManager);
  mes_s2c_synchronism=new Message_S2C_SYNCHRONISM(MesManager);
  mes_s2c_perception=new Message_S2C_PERCEPTION(MesManager);

  world=new World();
  world->SetWorld("world/map/","Initial");
  }

Core::~Core()
  {
  delete mes_c2s_action;
  delete mes_c2s_ask_map_piece;
  delete mes_c2s_ask_perception;
  delete mes_c2s_info_about_game;
  delete mes_c2s_load_character;
  delete mes_c2s_login;
  delete mes_c2s_logout;
  delete mes_c2s_synchronism;
  delete mes_c2s_system_info;

  delete mes_s2c_action;
  delete mes_s2c_load_character;
  delete mes_s2c_login;
  delete mes_s2c_logout;
  delete mes_s2c_map_piece;
  delete mes_s2c_synchronism;
  delete mes_s2c_perception;

  delete world;
  }

/**
This method start the core of the game.
**/
void Core::Run()
  {
  mes_c2s_login->SetArianneVersionID(ARIANNE_HARDCODE_VERSION);
  mes_c2s_login->Put();
  #ifdef macintosh
    ::YieldToThread( NetManager->senderID );
  #endif
  Log::Debug("Sending Login message to server.");

  while(mes_s2c_login->isEmpty(0)==TRUE)
    {    
    #ifdef THREADLESS
    NetManager->loop();
    #endif

    #ifdef macintosh
    ::YieldToAnyThread();
    #endif
    }

  mes_s2c_login->Get();

  SysInfo::GetSystemInfo(mes_c2s_system_info);
  mes_c2s_system_info->Put();
  #ifdef macintosh
    ::YieldToThread( NetManager->senderID );
  #endif
  Log::Debug("Sending System info message to server.");

  Log::Debug("Sending load character petition to server.");
  mes_c2s_load_character->SetName(DataLoader::GetCharacterName());
  mes_c2s_load_character->SetPassword(DataLoader::GetCharacterPassword());
  mes_c2s_load_character->Put();
  #ifdef macintosh
    ::YieldToThread( NetManager->senderID );
  #endif

  while(mes_s2c_load_character->isEmpty()==TRUE)
    {    
    #ifdef THREADLESS
    NetManager->loop();
    #endif

    #ifdef macintosh
    ::YieldToAnyThread();
    #endif
    }
  mes_s2c_load_character->Get();
  if(mes_s2c_load_character->GetAccepted()==TRUE)
    {
    Player=mes_s2c_load_character->GetCharacter();
    }
  else
    {
    Log::Error("Wrong login");
    }

  mes_c2s_ask_map_piece->Setxinit(0);
  mes_c2s_ask_map_piece->Setyinit(0);
  mes_c2s_ask_map_piece->Setxend(400);
  mes_c2s_ask_map_piece->Setyend(400);
  mes_c2s_ask_map_piece->SetCRC(0);
  mes_c2s_ask_map_piece->Put();
  #ifdef macintosh
    ::YieldToThread( NetManager->senderID );
  #endif

  while(mes_s2c_map_piece->isEmpty(0)==TRUE)
    {    
    #ifdef THREADLESS
    NetManager->loop();
    #endif

    #ifdef macintosh
    ::YieldToAnyThread();
    #endif
    }

  mes_s2c_map_piece->Get();
  Log::Debug("*****MAP_PIECE S2C()*****");
  AList<Terrain> *list=mes_s2c_map_piece->Getterrains();

  for(int j=0;j<list->Size();j++)
    {
    world->UpdateTerrain((*list)[j]);
    }

  mes_c2s_ask_perception->SetuID(Player->mID);
  Perception per;
  per.SetType(PERCEPTION_VISION);
  mes_c2s_ask_perception->Setperception(per);

  mes_c2s_ask_perception->Put();
  #ifdef macintosh
    ::YieldToThread( NetManager->senderID );
  #endif


  PrepareGUI();
  ShowGUI();

  /* We are in, now start the process. */
  Uint32 chrono_start,chrono_end,frames;

  chrono_start=SDL_GetTicks();

  frames=Process();

  chrono_end=SDL_GetTicks();

  Log::Debug("Sending Logout message to server.");
  char *Stadistics=new char[256];
  float fps=(float)frames/((float)(chrono_end-chrono_start)/1000.0);
  sprintf(Stadistics,"Frames rendered: %i  -- FPS: %f",frames,fps);

  mes_c2s_logout->SetStadistics(Stadistics);
  mes_c2s_logout->Put(0);
  #ifdef macintosh
    ::YieldToThread( NetManager->senderID );
  #endif
  }

/**
This method process all the messages to/from the server.
All the client computing is done here.
**/
Uint32 Core::Process()
  {
  Uint32 actualframe,lastframe,lasttimePerception,lasttimeSynchronism,lasttimeAction,start;
  Uint32 frames=0;

  bool LoadedObject=false;

  actualframe=lastframe=lasttimePerception=lasttimeSynchronism=lasttimeAction=start=SDL_GetTicks();
  
  #ifndef ARIANNE_TEXT_ONLY
  while(pressExitButton->isPressed()==FALSE)
  #else
  for(;SDL_GetTicks()-start<TIME_LIMIT;)
  #endif
    {
    #ifdef THREADLESS
    NetManager->loop();
    #endif

    #ifdef macintosh
    ::YieldToAnyThread();
    #endif

    ManagePerceptions(&lasttimePerception,&LoadedObject);
    ManageActions(&lasttimeAction);

    if(LoadedObject==true)
      {
      #ifndef ARIANNE_TEXT_ONLY
      Renderer::Begin();
      eventProcessor->Process();

      sendTextMessage->Update(Player->mID,Player->Pos);

      Object *tmpPlayer=Player->Clone();
      world->UpdateObject(tmpPlayer);
      Player=world->GetObject(tmpPlayer);
      Player=Player->Clone();

      playerMove->Update(Player);

      #ifndef ARIANNE_NO_RENDERER
      Renderer::Show(Player,world,actualframe,lastframe);
      #endif

      appbase->requestUpdate( NULL );

      Renderer::End();

      #else
      #ifndef MAXIMUM_SPEED
      SDL_Delay(100);
      #endif
      #endif
      
      frames++;
      }

    lastframe=actualframe;
    actualframe=SDL_GetTicks();
    }

  return frames;
  }

void Core::ManagePerceptions(Uint32 *lasttimePerception, bool *LoadedObject)
  {
  assert(lasttimePerception!=NULL);
  assert(LoadedObject!=NULL);

  if(SDL_GetTicks()-lasttimePerception[0]>5000)
    {
    lasttimePerception[0]=SDL_GetTicks();
    mes_c2s_ask_perception->SetuID(Player->mID);
      
    Perception per;
    per.SetType(PERCEPTION_VISION);
    mes_c2s_ask_perception->Setperception(per);

    mes_c2s_ask_perception->Put();
    #ifdef macintosh
    ::YieldToThread( NetManager->senderID );
    #endif
    }

  if(mes_s2c_perception->isEmpty(0)==FALSE)
    {     
    LoadedObject[0]=true;
    mes_s2c_perception->Get();
    Log::Debug("*****PERCEPTION S2C()*****");
      
    AList<Object> *tmpList=mes_s2c_perception->GetObjects();

    world->ClearObjects();
    for(Uint32 i=0;i<tmpList->Size();i++)
      {
      world->AddObject((*tmpList)[i]);
      }

    Player=world->GetObject(Player);
    Player=Player->Clone();

    Renderer::Update(Player,world);
    }
  }

void Core::ManageActions(Uint32 *lasttimeAction)
  {
  if(mes_s2c_action->isEmpty(0)==FALSE)
    {
    Log::Debug("*****ACTION S2C()*****");     
    mes_s2c_action->Get();
    Action *action=mes_s2c_action->Getaction();

    switch(action->GetType())
      {
      case ARIANNE_CHAT:
        {
        Action_CHAT *tmpChatMessage=new Action_CHAT;
        tmpChatMessage->FromGeneric(action);

        Log::Debug("*** Recieving a CHAT Message (%s)",(const char *)(tmpChatMessage->GetText()));
        recvTextMessage->AddMessage(tmpChatMessage->GetSourceID(),tmpChatMessage->GetText());
        delete tmpChatMessage;
        }
        break;
      case ARIANNE_MOVE:
        {
        }
        break;
      }

    //delete action;
    }

 
  if(SDL_GetTicks()-lasttimeAction[0]>2000)
    {
    lasttimeAction[0]=SDL_GetTicks();

    Uint32 x,y,z,x1,y1,z1;
    Player->Pos.GetXYZ(&x,&y,&z);
    Player->Pos.GetXYZLast(&x1,&y1,&z1);
    if((x==x1)&&(y==y1)&&(z==z1))
      {
      return;
      }

    Player->Pos.MoveTo(x,y,z);

    Action_MOVE *tmpMove=new Action_MOVE;
    tmpMove->SetPos(Player->Pos);
    tmpMove->SetSourceID(Player->mID);
    tmpMove->SetXYZ(x,y,z);

    Action *tmpAction=tmpMove->ToGeneric();
  
    mes_c2s_action->Setaction(tmpAction);
    mes_c2s_action->Put();      

    delete tmpMove;
    }
  }

void Core::InitGUI()
  {
  // application base window --- event handler
  appbase = new AppBase( Screen::GetGameScreen() );
  appbase->setWidth( SDL_GetVideoSurface()->w );
  appbase->setHeight( SDL_GetVideoSurface()->h );
  
  eventProcessor->setEventHandler( appbase );

  TitleWindow = new AWindow( "gfx/Main.bmp" );
  TitleWindow->setVisible(true);
  }


void Core::PrepareGUI()
  {
  #ifndef ARIANNE_TEXT_ONLY
  
  // windows
  awindow = new AWindow( "gfx/test_window.bmp" );
  bwindow = new ShapedWindow( "gfx/test_window_2.bmp");
  cwindow = new AWindow( "gfx/test_window.bmp");
  exitWindow = new AWindow("gfx/gui/Button/button_exit.bmp");

  awindow->setVisible( false );
  bwindow->setVisible( false );
  cwindow->setVisible( false );
  exitWindow->setVisible( false );
  
  // add windows to application base window
  appbase->addWindow( bwindow, 150, 150 );
  appbase->addWindow( awindow, 40, 40 );
  appbase->addWindow( cwindow, 60, 60 );
  appbase->addWindow( exitWindow,550, 450);
  appbase->addWindow( TitleWindow,0,0);
  
  // LineEdit with a TAB group
  LineEdit* lineedit = new LineEdit( 273, 20 );
  lineedit->setFont( "gfx/Fonts/Fixed.7x5.ISO-8859-1.bmp");

  // TextBox, with buttons that work for scrolling
  TextBox* box = new TextBox( 253, 136 );
  box->setFont("gfx/Fonts/Fixed.7x5.ISO-8859-1.bmp");
  lineedit->setCommandType( ARIANNE_COM_TEXT_ADD );
//  lineedit->addListener( box );

  sendTextMessage=new SendTextMessage(MesManager,0);
  lineedit->addListener( sendTextMessage );
  recvTextMessage=new RecvTextMessage();
  recvTextMessage->addListener(box);

  AButton* scUpButton = new AButton(
  	"gfx/gui/Button/scrollup.bmp",
  	"gfx/gui/Button/scrollup_on.bmp" );
  AButton* scDownButton = new AButton(
  	"gfx/gui/Button/scrolldown.bmp",
  	"gfx/gui/Button/scrolldown_on.bmp" );
  scUpButton->setCommandType( ARIANNE_COM_SCROLL_UP );
  scDownButton->setCommandType( ARIANNE_COM_SCROLL_DOWN );
  scUpButton->addListener( box );
  scDownButton->addListener( box );

  // Table
  Table* atable = new Table();
  atable->setCellSurface( "gfx/gui/Table/cell_back.bmp");
  atable->setCellHiliteSurface( "gfx/gui/Table/cell_hilite.bmp");
  atable->setCellFocusSurface( "gfx/gui/Table/cell_focus.bmp");
  atable->setTableSize( 10, 10, false );
  atable->setSize( 200, 150 );
  TableCell* redcell = new TableCell( NULL );
  redcell->setSurface( "gfx/gui/Icon/potion_red.bmp");
  TableCell* greencell = new TableCell( NULL );
  greencell->setSurface( "gfx/gui/Icon/potion_green.bmp");
  TableCell* bluecell = new TableCell( NULL );
  bluecell->setSurface( "gfx/gui/Icon/potion_blue.bmp");
  atable->addCellAt( redcell, 1, 1 );
  atable->addCellAt( greencell, 4, 4 );
  atable->addCellAt( bluecell, 10, 10 );
  // Buttons
  AButton* upButton = new AButton("gfx/gui/Button/button_circle.bmp",
  	"gfx/gui/Button/button_circle_on.bmp");
  AButton* downButton = new AButton("gfx/gui/Button/button_circle.bmp",
  	"gfx/gui/Button/button_circle_on.bmp");
  AButton* rightButton = new AButton("gfx/gui/Button/button_circle.bmp",
  	"gfx/gui/Button/button_circle_on.bmp");
  AButton* leftButton = new AButton("gfx/gui/Button/button_circle.bmp",
  	"gfx/gui/Button/button_circle_on.bmp");
  AButton* exitButton = new AButton("gfx/gui/Button/button_exit.bmp",
  	"gfx/gui/Button/button_exit_on.bmp");
  
  pressExitButton=new PressButton;
  exitButton->setCommandType( ARIANNE_COM_BUTTON_PRESSED );
  exitButton->addListener(pressExitButton);
  
  upButton->addListener( atable );
  upButton->setCommandType( ARIANNE_COM_SCROLL_UP );
  downButton->addListener( atable );
  downButton->setCommandType( ARIANNE_COM_SCROLL_DOWN );
  rightButton->addListener( atable );
  rightButton->setCommandType( ARIANNE_COM_SCROLL_RIGHT );
  leftButton->addListener( atable );
  leftButton->setCommandType( ARIANNE_COM_SCROLL_LEFT );
  
 
  // AnimatingIconSample --- demonstrates animating icon, that moves, too.
  AnimatingIconSample* icon = new AnimatingIconSample( "gfx/gui/icon_fire_1.bmp");
  icon->addFrame( "gfx/gui/icon_fire_2.bmp");
  GUIAnimator* animator = new GUIAnimator();
  animator->addClient( icon );
  
  // Adding a keyboard shortcut listener
  ShortCutListener* keyListener = new ShortCutListener();
  keyListener->setAcceptKey( true );
  keyListener->setAcceptMouse( false );
  keyListener->setWindowA( awindow );
  keyListener->setWindowB( bwindow );
  keyListener->setWindowC( cwindow );

  playerMove=new PlayerMove(MesManager);
  
  appbase->addListener( keyListener );
  appbase->addListener( playerMove );
  appbase->addAnimator( animator );
  
  // add all these GUI parts to each window
  awindow->addWidget( lineedit, 10, 173 );
  awindow->addWidget( box, 10, 27 );
  awindow->addWidget( scUpButton, 267, 27 );
  awindow->addWidget( scDownButton, 267, 148 );

  bwindow->addWidget( icon, 20, 20 );
  cwindow->addWidget( upButton, 220, 30 );
  cwindow->addWidget( downButton, 220, 60 );
  cwindow->addWidget( rightButton, 220, 90 );
  cwindow->addWidget( leftButton, 220, 120 );
  cwindow->addWidget( atable, 10, 30 );
  exitWindow->addWidget(exitButton,0,0);

  // let the windows visible, and we are ready;)
  #endif
  }

void Core::ShowGUI()
  {
  awindow->setVisible( true );
  bwindow->setVisible( true );
  cwindow->setVisible( true );
  exitWindow->setVisible( true );
  TitleWindow->setVisible(false);
  }

