#ifndef ARIANNE_Core_H
#define ARIANNE_Core_H

#include "general.h"
#include "MessageManager.h"
#include "NetworkManager.h"
#include "Messages.h"
#include "World.h"
#include "Object.h"
#include "EventProcessor.h"

#include "GUI/GUISet.h"
#include "GUI/ShapedWindow.h"

#include "GUIConnect/SendTextMessage.h"
#include "GUIConnect/RecvTextMessage.h"
#include "GUIConnect/PressButton.h"
#include "GUIConnect/PlayerMove.h"


/**
This class is like the CPU of the game.

It manages all the game aspects:
  * Send messages to/from parts of the game.
  * Do collision detections.
  * Applies rules
  * Manage Objects.
  * Transform user input intoobject actions.

This class is complex so perhaps it should be divided into several simplier classes
**/ 
class Core
  {
  MessageManager *MesManager;
  NetworkManager *NetManager;

  /* These messages are here to avoid constant creation and removal of the 
     message in the methods. */
  Message_C2S_LOGIN *mes_c2s_login;
  Message_S2C_LOGIN *mes_s2c_login;

  Message_C2S_LOGOUT *mes_c2s_logout;
  Message_S2C_LOGOUT *mes_s2c_logout;

  Message_C2S_SYNCHRONISM *mes_c2s_synchronism;
  Message_S2C_SYNCHRONISM *mes_s2c_synchronism;

  Message_C2S_SYSTEM_INFO *mes_c2s_system_info;

  Message_C2S_INFO_ABOUT_GAME *mes_c2s_info_about_game;

  Message_C2S_LOAD_CHARACTER *mes_c2s_load_character;
  Message_S2C_LOAD_CHARACTER *mes_s2c_load_character;

  Message_C2S_ACTION *mes_c2s_action;
  Message_S2C_ACTION *mes_s2c_action;

  Message_C2S_ASK_PERCEPTION *mes_c2s_ask_perception;
  Message_S2C_PERCEPTION *mes_s2c_perception;

  Message_C2S_ASK_MAP_PIECE *mes_c2s_ask_map_piece;
  Message_S2C_MAP_PIECE *mes_s2c_map_piece;

  World *world;
  Object *Player;
  EventProcessor *eventProcessor;

  AppBase* appbase;

  AWindow *TitleWindow;
  AWindow* awindow;
  AWindow* cwindow;
  AWindow* exitWindow;
  ShapedWindow* bwindow;
  
  SendTextMessage *sendTextMessage;
  RecvTextMessage *recvTextMessage;
  PressButton *pressExitButton;
  PlayerMove *playerMove;

  public:
  Core(MessageManager *MesManager,NetworkManager *NetManager);
  ~Core();
  
  void Run();

  private:
  Uint32 Process();
  void ManagePerceptions(Uint32 *lasttimePerception, bool *LoadedObject);
  void ManageActions(Uint32 *lasttimeAction);

  void InitGUI();
  void PrepareGUI();
  void ShowGUI();
  };

#endif