/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is the CORE class for the server.
    It has some differences with the one in the client, but the basic
    idea behind is the same.
***************************************************************************/
#include "CoreServer.h"
#include "PlayerID.h"
#include "Log.h"
#include "World.h"
#include "CharacterManager.h"
#include "Actions.h"

#include "../server/ReportGenerator.h"


#ifdef macintosh
#include <Threads.h>
#ifndef SERVER_MAC_RES_H
#include "server_mac_res.h"
#endif
#endif

CoreServer::CoreServer(MessageManager *MesManager,NetworkServerManager *NetServerManager)
  {
  this->MesManager=MesManager;
  this->NetServerManager=NetServerManager;
  player2id=new Player2ID();

  mes_c2s_action=new Message_C2S_ACTION(MesManager);
  mes_c2s_ask_map_piece=new Message_C2S_ASK_MAP_PIECE(MesManager);
  mes_c2s_ask_perception=new Message_C2S_ASK_PERCEPTION(MesManager);
  mes_c2s_info_about_game=new Message_C2S_INFO_ABOUT_GAME(MesManager);
  mes_c2s_load_character=new Message_C2S_LOAD_CHARACTER(MesManager);
  mes_c2s_login=new Message_C2S_LOGIN(MesManager);
  mes_c2s_logout=new Message_C2S_LOGOUT(MesManager);
  mes_c2s_synchronism=new Message_C2S_SYNCHRONISM(MesManager);
  mes_c2s_system_info=new Message_C2S_SYSTEM_INFO(MesManager);

  mes_s2c_action=new Message_S2C_ACTION(MesManager);
  mes_s2c_load_character=new Message_S2C_LOAD_CHARACTER(MesManager);
  mes_s2c_login=new Message_S2C_LOGIN(MesManager);
  mes_s2c_logout=new Message_S2C_LOGOUT(MesManager);
  mes_s2c_map_piece=new Message_S2C_MAP_PIECE(MesManager);
  mes_s2c_synchronism=new Message_S2C_SYNCHRONISM(MesManager);
  mes_s2c_perception=new Message_S2C_PERCEPTION(MesManager);

  world=new World();
  world->LoadWorld("world/map/","Initial");
  world->GetSize(&WorldX1,&WorldY1,&WorldX2,&WorldY2);

  reportGenerator=new ReportGenerator();
  reportGenerator->SetWorld(world);
  reportGenerator->SetNetworkServerManager(NetServerManager);
  ErrorsOnZombieCharacters=0;
  }

CoreServer::~CoreServer()
  {
  delete mes_c2s_action;
  delete mes_c2s_ask_map_piece;
  delete mes_c2s_ask_perception;
  delete mes_c2s_info_about_game;
  delete mes_c2s_load_character;
  delete mes_c2s_login;
  delete mes_c2s_logout;
  delete mes_c2s_synchronism;
  delete mes_c2s_system_info;

  delete mes_s2c_action;
  delete mes_s2c_load_character;
  delete mes_s2c_login;
  delete mes_s2c_logout;
  delete mes_s2c_map_piece;
  delete mes_s2c_synchronism;
  delete mes_s2c_perception;

  delete reportGenerator;
  }

void CoreServer::Run()
  {
  Uint32 actualtime=SDL_GetTicks();
  Uint32 lasttime=actualtime;
  Uint32 lastReportTime=actualtime;
  
  #ifdef macintosh
  gDone = false;
  EventRecord theEvent;
  while( ! gDone )
  #else
  while(1)
  #endif
    {
    NumPlayers=0;
    for(int i=0;i<MAX_CONNECTIONS_PER_SERVER;i++)
      {
      #ifdef THREADLESS
      NetServerManager->loop();
      #endif

      CheckLogin(i);
      if(player2id->Player(i)!=-1)
        {
        CheckCharacterLoad(i);
        CheckActions(i);
        CheckPerceptions(i);
        CheckMaps(i);
        CheckOther(i);
        CheckLogout(i);
        
        if((Sint32)(actualtime-player2id->GetLastTime(i))>TIMEOUT_TIME)
          {
          /* The player didn't send anything since X seconds ago. */
          KillCharacterOnConnection(i);
          }
        
        NumPlayers++;
        }
      }

    /* Printing the report... */
    if(actualtime-lastReportTime>REPORT_TIME)
      {
      reportGenerator->IncTime(((float)(actualtime-lastReportTime))/1000.0);
      reportGenerator->SetErrorsOnZombieCharacters(ErrorsOnZombieCharacters);
      reportGenerator->SetNumPlayers(NumPlayers);
      reportGenerator->Show();

      lastReportTime=actualtime;
      }

    /* Updating timers ...*/
    lasttime=actualtime;
    actualtime=SDL_GetTicks();

    /* Giving time to other processes. */
    #ifndef MAXIMUM_SPEED
    SDL_Delay(SLEEP_TIME);
    #endif

	#ifdef macintosh
	if( ::WaitNextEvent( everyEvent, &theEvent, nil, nil ))
  	  handleEvent( &theEvent );
    ::YieldToAnyThread();
    #endif
    }
  }

void CoreServer::CheckLogin(int i)
  {
  while(mes_c2s_login->isEmpty(i)==FALSE)
    {
    player2id->UpdateTime(i);
    mes_c2s_login->Get(i);
    Log::Debug("*****LOGIN C2S(%s) from %i*****",(const char *)(mes_c2s_login->GetArianneVersionID()),i);

    mes_s2c_login->SetServerToLog("localhost");
    mes_s2c_login->Put(i);
 
    player2id->Add(i,-2);
    }
  }

void CoreServer::CheckLogout(int i)
  {
  while(mes_c2s_logout->isEmpty(i)==FALSE)
    {
    player2id->UpdateTime(i);
    mes_c2s_logout->Get(i);
    Log::Debug("*****LOGOUT C2S(%s) from %i*****",(const char *)(mes_c2s_logout->GetStadistics()),i);

    Object *obj=world->GetObject(player2id->Player(i));

    Log::Debug("Deleting a character --UID-- %i",obj->mID);
    world->RemoveObject(obj);
    player2id->Del(i);

    mes_s2c_logout->SetByeString("Have a Good day.");
    mes_s2c_logout->Put(i);
    }
  }

void CoreServer::KillCharacterOnConnection(int i)
  {
  Object *obj=world->GetObject(player2id->Player(i));
  ErrorsOnZombieCharacters++;

  if(obj!=NULL)
    {
    Log::Warning("Deleting a character --UID-- %i (Timeout without answer)\n",obj->mID);
    world->RemoveObject(obj);
    player2id->Del(i);

    mes_s2c_logout->SetByeString("Have a Good day.");
    mes_s2c_logout->Put();
    }
  }

void CoreServer::CheckActions(int i)
  {
  while(mes_c2s_action->isEmpty(i)==FALSE)
    {
    player2id->UpdateTime(i);
    mes_c2s_action->Get(i);
    Action *action=mes_c2s_action->Getaction();

    switch(action->TypeAction)
      {
      case ARIANNE_CHAT:
        {
        Log::Debug("*****ACTION CHAT C2S from %i*****",i);
        Object *Character=world->GetObject(player2id->Player(i));  
        AList<Object> *listObjects=world->isInRange(Character->Pos,2000);

        for(int j=0;j<listObjects->Size();j++)
          {
          if(player2id->ID((*listObjects)[j]->mID)!=-1)
            {
            mes_s2c_action->Setaction(action);
            mes_s2c_action->Put(player2id->ID((*listObjects)[j]->mID));
            }
          }

        listObjects->Clear();
        delete listObjects;
        }
        break;
      case ARIANNE_MOVE:
        {
        Log::Debug("*****ACTION MOVE C2S from %i*****",i);
        Action_MOVE *actionMove=new Action_MOVE();
        actionMove->FromGeneric(action);

        Object *obj=world->GetObject(player2id->Player(i));
  
        Uint32 x,y,z;
        actionMove->GetXYZ(&x,&y,&z);
        if((x>=WorldX1)&&(x<WorldX2) && (y>=WorldY1)&&(y<WorldY2))
        obj->Pos.MoveTo(x,y,z);
        }
        break;
      }
    }
  }

void CoreServer::CheckOther(int i)
  {
  while(mes_c2s_system_info->isEmpty(i)==FALSE)
    {
    player2id->UpdateTime(i);
    mes_c2s_system_info->Get(i);
    Log::Debug("*****SYSTEM_INFO C2S(%s) from %i*****",(const char *)(mes_c2s_system_info->GetOs()),i);
    reportGenerator->AddSystemInfoStats(mes_c2s_system_info->GetOs(),
            mes_c2s_system_info->GetCpu(),
            mes_c2s_system_info->GetMemory());
    }

  while(mes_c2s_synchronism->isEmpty(i)==FALSE)
    {
    player2id->UpdateTime(i);
    Log::Debug("*****SYNCHRONISM C2S from %i*****",i);
    mes_c2s_synchronism->Get(i);
    mes_s2c_synchronism->Put(i);
    }
  }

void CoreServer::CheckPerceptions(int i)
  {
  while(mes_c2s_ask_perception->isEmpty(i)==FALSE)
    {
    player2id->UpdateTime(i);
    mes_c2s_ask_perception->Get(i);
    Log::Debug("*****ASK_PERCEPTION C2S from %i*****",i);
    if(player2id->Player(i)!=-1)
      {
      Object *Character=world->GetObject(player2id->Player(i));

      Uint32 lasttime=SDL_GetTicks();
      mes_s2c_perception->SetObjects(world->isInRange(Character->Pos,2000));
      Log::Debug("Time on getting Perception: %f",(float)(SDL_GetTicks()-lasttime)/1000.0);
      Perception per;
      per.SetType(PERCEPTION_VISION);
      mes_s2c_perception->Setperception(per);
      mes_s2c_perception->SetuID(player2id->Player(i));
      mes_s2c_perception->Put(i);
      }
    else
      {
      Log::Warning("Unknown player on slot %i",i);
      }
    }
  }

void CoreServer::CheckCharacterLoad(int i)
  {
  while(mes_c2s_load_character->isEmpty(i)==FALSE)
    {
    player2id->UpdateTime(i);
    mes_c2s_load_character->Get(i);
    Log::Debug("*****LOAD_CHARACTER C2S from %i*****",i);

    Object *Character=CharacterManager::LookFor(mes_c2s_load_character->GetName(),
                                                mes_c2s_load_character->GetPassword());    

    ABool accept(TRUE);
    mes_s2c_load_character->SetAccepted(accept);
    mes_s2c_load_character->SetCharacter(Character);
    mes_s2c_load_character->Put(i);

    player2id->Add(i,Character->mID);

    /* We track the number of players since start */
    reportGenerator->IncPlayersSinceStart();

    Log::Debug("Creating new character --UID-- %i",Character->mID);
    world->AddObject(Character);
    }
  }

void CoreServer::CheckMaps(int i)
  {
  while(mes_c2s_ask_map_piece->isEmpty(i)==FALSE)
    {
    player2id->UpdateTime(i);
    mes_c2s_ask_map_piece->Get(i);
    Log::Debug("*****ASK_MAP_PIECE C2S from %i*****",i);
    Uint32 x1,y1,x2,y2,CRC;

    x1=mes_c2s_ask_map_piece->Getxinit();
    y1=mes_c2s_ask_map_piece->Getyinit();
    x2=mes_c2s_ask_map_piece->Getxend();
    y2=mes_c2s_ask_map_piece->Getyend();
    CRC=mes_c2s_ask_map_piece->GetCRC();

    Uint32 lasttime=SDL_GetTicks();
    mes_s2c_map_piece->Setterrains(world->isInRange(x1,y1,x2,y2));
    Log::Debug("Time on getting Map: %f",(float)(SDL_GetTicks()-lasttime)/1000.0);

    mes_s2c_map_piece->Put(i);
    }
  }

Uint32 CoreServer::GetErrorsOnZombieCharacters()
  {
  return ErrorsOnZombieCharacters;
  }

Uint32 CoreServer::GetNumPlayers()
  {
  return NumPlayers;
  }


#ifdef macintosh
void CoreServer::handleEvent( EventRecord* e )
{
	switch( e->what )
	{
		case mouseDown:
			WindowPtr* window;
			short thePart = FindWindow( e->where, window );
			if( thePart == inMenuBar )
			{
				long menuChoice = MenuSelect( e->where );
				short theMenu, theItem;
				theMenu = HiWord( menuChoice );
				theItem = LoWord( menuChoice );
				
				switch( theMenu )
				{
					case FILE_MENU:
						gDone = true;
						break;
					
					case APPLE_MENU:
						MenuHandle appleMenu = GetMenuHandle( APPLE_MENU );
						if( appleMenu )
						{
							Str255 itemName;
							GetMenuItemText( appleMenu, theItem, itemName );
							OpenDeskAcc( itemName );
						}
						break;
				}
				
				HiliteMenu(0);
			}
	}
}
#endif
