#ifndef ARIANNE_CoreServer_H
#define ARIANNE_CoreServer_H

#include "general.h"
#include "MessageManager.h"
#include "NetworkServerManager.h"
#include "Messages.h"
#include "PlayerID.h"
#include "World.h"

#include "../server/ReportGenerator.h"

#ifdef macintosh
#include <Events.h>
#include <MacWindows.h>
#include <ToolUtils.h>

#define	APPLE_MENU	1000
#define FILE_MENU	1001
#endif

/**
This class is like the CPU of the game.

It manages all the game aspects:
  * Send messages to/from parts of the game.
  * Do collision detections.
  * Applies rules
  * Manage Objects.
  * Transform user input intoobject actions.

This class is complex so perhaps it should be divided into several simplier classes
**/ 
class CoreServer
  {
  MessageManager *MesManager;
  NetworkServerManager *NetServerManager;
  Player2ID *player2id;

  World *world;
  Uint32 WorldX1,WorldY1,WorldX2,WorldY2;

  Uint32 ErrorsOnZombieCharacters;
  Uint32 NumPlayers;

  /* These messages are here to avoid constant creation and removal of the 
     message in the methods. */
  Message_C2S_LOGIN *mes_c2s_login;
  Message_S2C_LOGIN *mes_s2c_login;

  Message_C2S_LOGOUT *mes_c2s_logout;
  Message_S2C_LOGOUT *mes_s2c_logout;

  Message_C2S_SYNCHRONISM *mes_c2s_synchronism;
  Message_S2C_SYNCHRONISM *mes_s2c_synchronism;

  Message_C2S_SYSTEM_INFO *mes_c2s_system_info;

  Message_C2S_INFO_ABOUT_GAME *mes_c2s_info_about_game;

  Message_C2S_LOAD_CHARACTER *mes_c2s_load_character;
  Message_S2C_LOAD_CHARACTER *mes_s2c_load_character;

  Message_C2S_ACTION *mes_c2s_action;
  Message_S2C_ACTION *mes_s2c_action;

  Message_C2S_ASK_PERCEPTION *mes_c2s_ask_perception;
  Message_S2C_PERCEPTION *mes_s2c_perception;

  Message_C2S_ASK_MAP_PIECE *mes_c2s_ask_map_piece;
  Message_S2C_MAP_PIECE *mes_s2c_map_piece;

  ReportGenerator *reportGenerator;

  public:
  CoreServer(MessageManager *MesManager,NetworkServerManager *NetServerManager);
  ~CoreServer();
  void Run();
  void CheckLogin(int i);
  void CheckLogout(int i);
  void CheckActions(int i);
  void CheckOther(int i);
  void CheckPerceptions(int i);
  void CheckCharacterLoad(int i);
  void CheckMaps(int i);
  void KillCharacterOnConnection(int i);

  Uint32 GetErrorsOnZombieCharacters();
  Uint32 GetNumPlayers();
  
  // Minami's addition
  #ifdef macintosh
  void handleEvent( EventRecord* e );
  BOOL gDone;
  #endif
  };
  
#endif