/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This a Processor of the events generated by the app or the OS.
***************************************************************************/
#include "EventProcessor.h"

EventProcessor::EventProcessor()
  {
  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
  SDL_EnableUNICODE(1);

  KeyboardStatus=new Uint8[SDLK_LAST];
  for(int i=0;i<SDLK_LAST;i++)  
    {
    KeyboardStatus[i]=FALSE;
    }

  KeyBuffer=new Uint16[256];
  iKeyBufferEnd=iKeyBufferInit=0;
  MouseX=MouseY=Button=0;
  commander = NULL;
  }

EventProcessor::~EventProcessor()
  {
  delete[] KeyboardStatus;
  delete[] KeyBuffer;
  }

BOOL EventProcessor::Process()
  {
  SDL_Event event;
  int counter=0;

  while(SDL_PollEvent(&event)==1)
    {
    if( commander )
      {
      if(!commander->handleCommand(&event))
        {
        switch( event.type ) 
  	      {
	      case SDL_KEYDOWN:
	        keyDown(&event);
  	        break;
	      case SDL_KEYUP:
	        keyUp(&event);
	        break;
	      case SDL_MOUSEMOTION:
	        mouseMotion(&event);
  	        break;
	      case SDL_MOUSEBUTTONDOWN:
	        mouseButtonDown(&event);
  	        break;
          case SDL_MOUSEBUTTONUP:
            mouseButtonUp(&event);
 	        break;
          case SDL_QUIT:
            break;
    	    default:
   	        break;
	      }

        } 
      commander->animate();
      }
    /* We add a limit to the number of events processed. */
    counter++;
    if(counter>5)
      {
      return TRUE;
      }
    }
  if( commander )
  	commander->animate();
  return FALSE;
  }

BOOL EventProcessor::isKeyPressed(Uint16 key)
  {
  return KeyboardStatus[key];
  }

Uint16 EventProcessor::GetUnicodeKey()
  {
  if(iKeyBufferInit!=iKeyBufferEnd)
    {
    return KeyBuffer[iKeyBufferInit];
    }

  return 0;
  }

void EventProcessor::ClearUnicodeBuffer()
  {
  iKeyBufferInit=iKeyBufferEnd=0;
  for(int i=0;i<256;i++)
    {
    KeyBuffer[i]=0;
    }
  }

void EventProcessor::GetMouseState(Uint16 *x,Uint16 *y, Uint8 *button)
  {
  Sint32 sdlx,sdly;
  button[0]=SDL_GetMouseState(&sdlx,&sdly);
  x[0]=sdlx;
  y[0]=sdly;
  }


void EventProcessor::keyUp(SDL_Event *event)
  {
  KeyboardStatus[event->key.keysym.sym]=FALSE;
  }

void EventProcessor::keyDown(SDL_Event *event)
  {
  KeyboardStatus[event->key.keysym.sym]=TRUE;

  if(event->key.keysym.unicode!=0)
    {
    /* We test if the array is full */
    if((iKeyBufferEnd+255)%256!=iKeyBufferInit)
      {
      KeyBuffer[iKeyBufferEnd]=event->key.keysym.unicode;
      iKeyBufferEnd=(iKeyBufferEnd+1)%256;
      }
    }
  }

void EventProcessor::mouseMotion(SDL_Event *event)
  {
  MouseX = event->motion.x;
  MouseY = event->motion.y;
  }

void EventProcessor::mouseButtonDown(SDL_Event *event)
  {
  Button=event->button.button;
  }

void EventProcessor::mouseButtonUp(SDL_Event *event)
  {
  Button=event->button.button;
  }

void EventProcessor::setEventHandler( AppBase* commander )
{
	this->commander = commander;
}
