/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This class modifies an string so that the format is OS-compatible.
***************************************************************************/
#include "FilePath.h"
#include "Log.h"

const char FilePath::PERIOD = '.';
const char FilePath::MAC_SEPARATOR = ':';
const char FilePath::UNIX_SEPARATOR = '/';

FilePath::FilePath( AString& original)
{
	assign( original );
	pathLength = 256;
}

FilePath::FilePath( const char* initValue )
{
    assert(initValue!=NULL);
	assign( initValue );
	pathLength = 256;
}

FilePath::~FilePath()
{
}

void
FilePath::setPathLength( int len )
{
	pathLength = len;
}

int
FilePath::getPathLength()
{
	return pathLength;
}

void
FilePath::setPath( const char* newPath )
{
	assign( newPath );
}

char*
FilePath::getOriginalPathStr()
{
	char* original = new char[ strlen( value ) ];
	strcpy( original, value );
	return original;
}

void
FilePath::getValidFilePath( char* path )
{
	#ifdef	macintosh
	
		char* newPath = new char[ pathLength ];
		int i,j;
		
		i = j = 0;
		
		while( value[i] != 0 )
		{
			// In case "../"
			if( value[i] == value[i+1] == PERIOD && value[ i+2 ] == UNIX_SEPARATOR )
			{
				newPath[j] = newPath[j+1] = MAC_SEPARATOR;
				j += 2;
				i += 3;
			}
			// In case "./"
			else if( value[i] == PERIOD && value[i+1] == UNIX_SEPARATOR )
			{
				newPath[j] = MAC_SEPARATOR;
				j++;
				i += 2;
			}
			// In case "/"
			else if( value[i] == UNIX_SEPARATOR )
			{
				newPath[j] = MAC_SEPARATOR;
				j++;
				i++;
			}
			// Just copy
			else
			{
				newPath[j] = value[i];
				j++;
				i++;
			}
		}
		newPath[j] = 0;
		if( strlen( newPath ) > pathLength )
		{
			Log::Debug( "File path is too long!" );
			Log::Debug( newPath );
			Log::Error( "File path is too long!" );
		}
		
		int k = 0;
		if( newPath[k] != MAC_SEPARATOR )
		{
			path[k] = MAC_SEPARATOR;
			k++;
		}
		i = 0;
		while( i < j )
		{
			path[k] = newPath[i];
			i++;
			k++;
		}
		path[k] = 0;
		
		delete newPath;
		
	#else
		// on other platform than MacOS, just return copy of FilePath.value
		if( strlen( value ) > (Uint16)pathLength )
		{
			Log::Debug( "File path is too long!");
			Log::Debug( value );
			Log::Error( "File path is too long!");
		}
		
		strcpy( path, value );		
	#endif
	
	
}
char*
FilePath::getValidFilePath()
{
	#ifdef	macintosh
	
		char* newPath = new char[ pathLength ];
		int i,j;
		
		i = j = 0;
		
		while( value[i] != 0 )
		{
			// In case "../"
			if( value[i] == value[i+1] == PERIOD && value[ i+2 ] == UNIX_SEPARATOR )
			{
				newPath[j] = newPath[j+1] = MAC_SEPARATOR;
				j += 2;
				i += 3;
			}
			// In case "./"
			else if( value[i] == PERIOD && value[i+1] == UNIX_SEPARATOR )
			{
				newPath[j] = MAC_SEPARATOR;
				j++;
				i += 2;
			}
			// In case "/"
			else if( value[i] == UNIX_SEPARATOR )
			{
				newPath[j] = MAC_SEPARATOR;
				j++;
				i++;
			}
			// Just copy
			else
			{
				newPath[j] = value[i];
				j++;
				i++;
			}
		}
		
		newPath[j] = 0;
		if( strlen( newPath ) > pathLength )
		{
			Log::Debug( "File path is too long!" );
			Log::Debug( newPath );
			Log::Error( "File path is too long!" );
		}
		
		char* path = new char[ pathLength ];
		
		int k = 0;
		if( newPath[k] != MAC_SEPARATOR )
		{
			path[k] = MAC_SEPARATOR;
			k++;
		}
		i = 0;
		while( i < j )
		{
			path[k] = newPath[i];
			i++;
			k++;
		}
		path[k] = 0;
		
		delete newPath;
		
		return path;
	
	#else
		// on other platform than MacOS, just return copy of FilePath.value
		if( strlen( value ) > (Uint16)pathLength )
		{
			Log::Debug( "File path is too long!");
			Log::Debug( value );
			Log::Error( "File path is too long!");
		}
		
		char* path = new char[ pathLength ];
		strcpy( path, value );		
		return path;

	#endif
	
}

void
FilePath::getValidFilePath( AString& path )
{
//	FilePath temp( path );
	char* tempCPtr = new char[ getPathLength()];
//	temp.getValidFilePath( tempCPtr );
    getValidFilePath( tempCPtr );
	path = tempCPtr;
	delete tempCPtr;
}
