/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This class represent a SPRITE.

    It also manages the animation and transparency related themes.
***************************************************************************/
#include "GFXData.h"
#include "FilePath.h"
#include "Log.h"
#include "Screen.h"

#include <cstdio>

GFXData::GFXData()
  {
  ActualFrame=0;
  isKeyColor=FALSE;
  ToChange=0;
  Image=0;
  }

GFXData::~GFXData()
  {
  for(int j=0;j<NumAnimations;j++)
    {
    for(int i=0;i<NumFrames[j];i++)
      {
      SDL_FreeSurface((Image[j])[i]);
      }
    delete[] Image[j];
    }

  delete[] Image;
  }

void GFXData::GetSize(Uint16 *w,Uint16 *h)
  {
  assert(w!=NULL);
  assert(h!=NULL);

  w[0]=(Image[0])[0]->w;
  h[0]=(Image[0])[0]->h;
  }

void GFXData::SetNumAnimations(int NumAnims)
  {
  assert(NumAnims>=0);

  NumAnimations=NumAnims;
  Image=new SDL_Surface**[NumAnims];
  NumFrames=new Uint8[NumAnims];
  }

void GFXData::ToggleKeyColorOn()
  {
  isKeyColor=TRUE;
  }

void GFXData::SetNumFrames(int NumAnim,int NF)
  {
  assert(NumAnim<NumAnimations);
  assert(NF>0);

  this->NumFrames[NumAnim]=NF;
  Image[NumAnim]=new SDL_Surface*[NF];
  }

void GFXData::LoadFrames(AString BaseFilename,int Anim)
  {
  FilePath path( BaseFilename );
  path.getValidFilePath( BaseFilename );

  for(int i=0;i<NumFrames[Anim];i++)
    {
    // append animation,frame and extension to the filename
    AString CompleteFilename = AString::Format("%s_%i_%i.bmp",(char const*)BaseFilename,Anim,i);

    // load the bitmap
    SDL_Surface *tmp=SDL_LoadBMP(CompleteFilename);
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)CompleteFilename);
      }

    if(isKeyColor==TRUE)
      {
      SDL_SetColorKey(tmp,SDL_SRCCOLORKEY,SDL_MapRGB(tmp->format,0,0,0));
      }
      
    (Image[Anim])[i]=SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
    }
  }

void GFXData::Show(int x,int y,int Animation,Uint32 actualtime,Uint32 lasttime,SDL_Surface *ShowScreen)
  {
  SDL_Rect src,dst;

  ActualFrame=(ActualFrame)%NumFrames[Animation];

  src.x=0;
  src.y=0;
  src.w=(Image[Animation])[ActualFrame]->w;
  src.h=(Image[Animation])[ActualFrame]->h;

  dst.x=x;
  dst.y=y;
  dst.w=(Image[Animation])[ActualFrame]->w;
  dst.h=(Image[Animation])[ActualFrame]->h;

  SDL_BlitSurface((Image[Animation])[ActualFrame],&src,ShowScreen,&dst);

  ToChange+=actualtime-lasttime;
  if(ToChange>ANIMATION_MSECONDS_PER_FRAME)
    {
    ToChange=0;
    ActualFrame=(ActualFrame+1)%NumFrames[Animation];
    }
  }

void GFXData::Show(int x,int y,int Animation,int key,SDL_Surface *ShowScreen)
  {
  SDL_Rect src,dst;
  src.x=0;
  src.y=0;
  src.w=(Image[Animation])[key]->w;
  src.h=(Image[Animation])[key]->h;

  dst.x=x;
  dst.y=y;
  dst.w=(Image[Animation])[key]->w;
  dst.h=(Image[Animation])[key]->h;

  SDL_BlitSurface((Image[Animation])[key],&src,ShowScreen,&dst);
  }
