/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "../general.h"
#include "../Screen.h"
#include "AButton.h"
#include "AppBase.h"
#include "../FilePath.h"
#include "../Log.h"

AButton::AButton( SDL_Surface* sur, SDL_Surface* on_sur )
	: AWidget( surface, NULL )
  {
  on_surface = on_sur;
  acceptMouse = true;					// Windows default to accept mouse events.
  pressed = false;
  }

AButton::AButton( AString filepath, AString on_filepath )
	: AWidget( filepath, NULL )
  {
  if( surface )
	SDL_SetColorKey( surface, SDL_SRCCOLORKEY,SDL_MapRGB( surface->format, 0, 0, 0 ));

  FilePath on_path( on_filepath );
  on_path.getValidFilePath( on_filepath );
  SDL_Surface *tmp = SDL_LoadBMP( on_filepath );
  if(tmp==NULL)
    {
    Log::Error("Can't open file: %s",(char const*)on_filepath);
    }
  if( tmp )
	SDL_SetColorKey( tmp, SDL_SRCCOLORKEY,SDL_MapRGB( tmp->format, 0, 0, 0 ));

  on_surface=SDL_DisplayFormat(tmp);
  SDL_FreeSurface(tmp);


  acceptMouse = true;
  pressed = false;
  }


AButton::~AButton()
  {
  }

// virtual functions

// Inherited from GUIPart

void AButton::draw( SDL_Rect* localClip, SDL_Rect* globalClip )
  {
  if( pressed && on_surface )
    SDL_BlitSurface( on_surface, localClip, Screen::GetScreen(), globalClip );
   else
   	SDL_BlitSurface( surface, localClip, Screen::GetScreen(), globalClip );
  }

// Inherited from ACommander class
BOOL AButton::handleMouseDown( SDL_Event* e )
  {
	  if( doesAcceptMouse() && ptInPart( e->button.x, e->button.y ))
	  {
	    pressed = true;
        requestUpdate( &partRect );
	  	return true;
	  }
	  else
	  	return false;
  }


BOOL AButton::handleMouseUp( SDL_Event* e )
  {
  BOOL tmp=false;

  if(pressed && ptInPart( e->button.x, e->button.y))
    {
    if(listeners)
      {
      for(Uint32 i=0;i<listeners->Size();i++)
        {
        if( getListener(i) )
          {
          pressed = false;
          requestUpdate( &partRect );
      	  tmp|=getListener(i)->execCommand( new GenericCommand( NULL, getCommandType()) );
          }
        }
      }
	}

  return tmp;
  }


BOOL AButton::handleMouseMove( SDL_Event* e )
  {
  if( ! ptInPart( e->button.x, e->button.y ))
    {
    if( pressed )
      {
      pressed = false;
      requestUpdate( &partRect );
      }
    }

  return false;
  }
