/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "../general.h"
#include "ACommander.h"
#include "UTIL/CommanderGroup.h"
#include "GUICommandSet.h"

ACommander::ACommander()
{
	acceptMouse = acceptKey = focus = false;
	listeners = NULL;
	commander = NULL;
	mouseListener = NULL;
	keyListener = NULL;
	group = NULL;
	commandType = ARIANNE_COM_NO_COMMAND;
}

ACommander::~ACommander()
{
}

int
ACommander::getCommanderIndex()
{
	return comIndex;
}

void
ACommander::setCommanderIndex( int index )
{
	comIndex = index;
}

void
ACommander::updateCommanderIndex()
{
	if( listeners )
	{
		ACommander* commander;
		Uint32 i = 0;
		while( i < listeners->Size())
		{
			commander = (ACommander*)(*listeners)[i];
			commander->setCommanderIndex( i + 1 );
			i++;
		}
	}
}

void
ACommander::setCommander( ACommander* parent )
{
	commander = parent;
}

void
ACommander::setCommander( ACommander* parent, int id )
{
	commander = parent;
	comIndex = id;
}

int
ACommander::addListener( ACommander* listener )
{
	if( ! listeners )
		listeners = new AList<ACommander>();
	
	if( listeners )
		listeners->Add( listener, listeners->Size());
	
	listener->setCommander( this, listeners->Size() );
    return listeners->Size();
}

void
ACommander::removeListener( int index )
{
	if( listeners )
		listeners->Remove( index - 1 );
}

void
ACommander::removeListener( ACommander* listener )
{
	// Can we do this ?
    /* YES, Just you need to browse all the values until yoou find the equal one */
}

ACommander*
ACommander::getListener( int index)
  {
  if(listeners)
    return (*listeners)[index];

  return 0;
  }

int
ACommander::getSizeListener()
  {
  return listeners->Size();
  }

/*
We encourage the use of the list of listeners.

void
ACommander::setListener( ACommander* listener )
{
	commandListener = listener;
}

ACommander*
ACommander::getListener()
{
	return commandListener;
}
*/

void
ACommander::requestMouseFocus( ACommander* listener, BOOL request )
{
	if( commander )
		commander->requestMouseFocus( listener, request );
	else
	{
		if( request )
		{
			if( mouseListener )
				mouseListener->setFocus( false );		// remove old focus
			mouseListener = listener;
			if( mouseListener )
				mouseListener->setFocus( true );
		}
		else
		{
			if( mouseListener )
				mouseListener->setFocus(false );
			mouseListener = NULL;
		}
	}
}

void
ACommander::requestKeyFocus( ACommander* listener, BOOL request )
{
	if( commander )
		commander->requestKeyFocus( listener, request );
	else
	{
		if( request )
		{
			if( keyListener )
				keyListener->setFocus( false );
			keyListener = listener;
			if( keyListener )
				keyListener->setFocus( true );
		}
		else
		{
			if( keyListener )
				keyListener->setFocus( false );
			keyListener = NULL;
		}
	}
}

void
ACommander::setGroup( CommanderGroup* group )
{
	this->group = group;
}

void
ACommander::setFocus( BOOL focus )
{
	if( focus && ! this->focus && group )
		group->requestFocus( this );
	this->focus = focus;
}

BOOL
ACommander::isFocus()
{
	return focus;
}

void
ACommander::setAcceptMouse( BOOL accept )
{
	acceptMouse = accept;
}

void
ACommander::setDefaultAcceptMouse( BOOL accept )
{
	acceptMouseStatus = accept;
}

void
ACommander::setAcceptKey( BOOL accept )
{
	acceptKey = accept;
}

void
ACommander::setDefaultAcceptKey( BOOL accept )
{
	acceptKeyStatus = accept;
}

BOOL
ACommander::doesAcceptMouse()
{
	return acceptMouse;
}

BOOL
ACommander::doesAcceptKey()
{
	return acceptKey;
}

void
ACommander::restoreAcceptStatus()
{
	acceptMouse = acceptMouseStatus;
	acceptKey = acceptKeyStatus;
	if( listeners )
	{
		for( Uint32 i = 0; i < listeners->Size(); i++ )
		{
			ACommander* listener = (ACommander*)(*listeners)[i];
			if( listener )
				listener->restoreAcceptStatus();
		}
	}
}

void
ACommander::setCommandType( int type )
{
	commandType = type;
}

int
ACommander::getCommandType()
{
	return commandType;
}

BOOL
ACommander::execCommand( GenericCommand* com )
{
	// Prototype;
	// BOOL done = false;
	// switch( com->type )
	// {
	//		case com1:
	//			YourType* command = (YourType*)com->getCommand();
	//			done = doHandleCommand( command );
	//			break;
	//		case com2:
	//		case com3:
	//	}
	//	return done;
	return false;
}

BOOL
ACommander::handleCommand( SDL_Event* e )
{
	BOOL res = false;
	
	if( mouseListener && mouseListener->doesAcceptMouse())
	{
	  	switch( e->type )
	  	{
	  		case SDL_MOUSEBUTTONDOWN:
	  			res = mouseListener->handleMouseDown( e );
	  			break;
	  		case SDL_MOUSEBUTTONUP:
	  			res = mouseListener->handleMouseUp( e );
	  			break;
	  		case SDL_MOUSEMOTION:
	  			res = mouseListener->handleMouseMove( e );
	  			break;
	  	}
	}
	
	if( keyListener && keyListener->doesAcceptKey() )
	{
		switch( e->type )
		{
			case SDL_KEYDOWN:
				res = keyListener->handleKeyDown( e );
				break;
			case SDL_KEYUP:
				res = keyListener->handleKeyUp( e );
				break;
		}
	}
			  	
	if( ! res && listeners )
	{
		ACommander* child;
		int i = listeners->Size() - 1;
		while( i >= 0 && ! res && ( child = (ACommander*)(*listeners)[i] ))
		{
			if( child->doesAcceptMouse() || child->doesAcceptKey())
				res = child->handleCommand( e );
			i--;
		}
	}
	if( ! res )
	{
		switch( e->type )
		{
			case SDL_MOUSEBUTTONDOWN:
				if( doesAcceptMouse())
					res = handleMouseDown( e );
				break;
			
			case SDL_MOUSEBUTTONUP:
				if( doesAcceptMouse())
					res = handleMouseUp( e );
				break;
			
			case SDL_MOUSEMOTION:
				if( doesAcceptMouse())
					res = handleMouseMove( e );
				break;
			
			case SDL_KEYDOWN:
				if( doesAcceptKey())
					res = handleKeyDown( e );
				break;
			
			case SDL_KEYUP:
				if( doesAcceptKey())
				    res = handleKeyUp( e );
				break;
		}
		
	}
	return res;
}

BOOL
ACommander::handleMouseDown( SDL_Event* e )
{
	lastMouseDown = SDL_GetTicks();
	return false;
}

BOOL
ACommander::handleMouseUp( SDL_Event* e )
{
	lastMouseDown = 0;
	return false;
}

BOOL
ACommander::handleMouseMove( SDL_Event* e )
{
	return false;
}

BOOL
ACommander::handleClick( SDL_Event* e )
{
	return false;
}

BOOL
ACommander::handleKeyDown( SDL_Event* e )
{
	if( e->key.keysym.sym == SDLK_TAB && group )
	{
		group->next();
		return true;
	}
	else
		return false;
}

BOOL
ACommander::handleKeyUp( SDL_Event* e )
{
	return false;
}

BOOL
ACommander::idle()
{
	return true;
}
