#ifndef ACOMMANDER_H
#define ACOMMANDER_H

#include "../general.h"
#include "../AList.h"
#include "GenericCommand.h"
#include "UTIL/CommanderGroup.h"

class ACommander : public Disposable
{
	public:
	
	AList<ACommander>*			listeners;
	
	ACommander();
	~ACommander();
	
	// virtual functions
	//	parent commander setting
	virtual void			setCommander( ACommander* commander );
	virtual void			setCommander( ACommander* commander, int id );
	
	// add / remove listener
	virtual int				addListener( ACommander* listener );
	virtual void			removeListener( int index );
	virtual void			removeListener( ACommander* listener );
    virtual ACommander*     getListener( int index);
    virtual int             getSizeListener();

	// Add an explicite listener
	// ( this is rather a new feature. likely to change )
/*	
    We encourage the usage of the List of listeners.

    virtual void			setListener( ACommander* listener );
	ACommander*		getListener();
*/
	int				getCommandType();
	
	// set / reset focus
	virtual void			setFocus( BOOL focus );
	BOOL					isFocus();
	
	// mouse / key event acceptable or not
	virtual void			setAcceptMouse( BOOL accept );
	virtual void			setAcceptKey( BOOL accept );
	// default behavior
	void				setDefaultAcceptMouse( BOOL accept );
	void				setDefaultAcceptKey( BOOL accept );
	// judge
	BOOL				doesAcceptMouse();
	BOOL				doesAcceptKey();
	// This restores to the default status
	void				restoreAcceptStatus();
	
	// direct command execution ( this is rather new feature. likely to change )
	virtual void	setCommandType( int type );
	virtual BOOL	execCommand( GenericCommand* com );
	
	// general event handling functions
	virtual BOOL	handleCommand( SDL_Event* e );		// Should make a Command class ?
	virtual BOOL	handleMouseDown( SDL_Event* e );
	virtual BOOL	handleMouseUp( SDL_Event* e );
	virtual BOOL	handleMouseMove( SDL_Event* e );
	virtual BOOL	handleClick( SDL_Event* e );
	virtual BOOL	handleKeyDown( SDL_Event* e );
	virtual BOOL	handleKeyUp( SDL_Event* e );
	virtual BOOL	idle();
	
	// public functions
	
	int				getCommanderIndex();
	void			setCommanderIndex( int index );
	void			updateCommanderIndex();
	
	void			requestMouseFocus( ACommander* listener, BOOL request );
	void			requestKeyFocus( ACommander* listener, BOOL request );
	
	void			setGroup( CommanderGroup* group );
	
	protected:
	
	BOOL			acceptMouse, acceptKey;
	BOOL			focus, acceptMouseStatus, acceptKeyStatus;

	int				comIndex, lastMouseX, lastMouseY, commandType;
	long			lastMouseDown;					// Keeps time of last mouse down event.
													// Can be used for click.
													
	ACommander		*commander, *mouseListener, *keyListener;
	CommanderGroup*	group;
};

#endif