/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "../general.h"
#include "../Screen.h"
#include "AWindow.h"
#include "AppBase.h"

AWindow::AWindow( SDL_Surface* surface )
	: AWidget( surface, NULL )
{
	setAcceptMouse( true );				// Windows default to accept mouse events.
	setAcceptKey( true );				// ... and also key events !
	setDefaultAcceptKey( true );		// ... also as default behavior
	setDefaultAcceptMouse( true );		// ... also as default behavior
	dragging = false;
	setVisible( false );				// Initially invisible.
}

AWindow::AWindow( AString filepath )
	: AWidget( filepath, NULL )
{
	setAcceptMouse( true );				// Windows default to accept mouse events.
	setAcceptKey( true );				// ... and also key events !
	setDefaultAcceptKey( true );		// ... also as default behavior
	setDefaultAcceptMouse( true );		// ... also as default behavior
	dragging = false;
	setVisible( false );
}

AWindow::~AWindow()
{
}

void
AWindow::close()
{
	hide();
	if( getParent())
		getParent()->removeChild( getPartIndex() );
	
	delete( this );
}

int	AWindow::addWidget( AWidget* window, int x, int y )
  {
	addChild( window, x, y );
	addListener( window );
    return 0;
  }

void AWindow::removeWidget( AWidget* window )
  {
	removeChild( window->getPartIndex() );
	removeListener( window->getCommanderIndex());
  }

// virtual functions

// Inherited from ACommander class

BOOL
AWindow::handleMouseDown( SDL_Event* e )
{
	if( doesAcceptMouse() && getParent() && ptInPart( e->button.x, e->button.y ))
	{
		dragging = true;
		lastMouseX = e->button.x;
		lastMouseY = e->button.y;
		((AppBase*)getParent())->setFrontWindow( this );// Bring this window to the front.
		return true;
	}
	else
	{
		// remove key focus
		requestKeyFocus( NULL, false );
		return false;
	}
}

BOOL
AWindow::handleMouseUp( SDL_Event* e )
{
	if( dragging )
	{
		dragging = false;
		return true;
	}
	else
		return false;
}

BOOL
AWindow::handleMouseMove( SDL_Event* e )
{
	if( dragging /*&& ptInPart( e->button.x, e->button.y )*/ )
	{
		setLocalLocation( getX() + ( e->button.x - lastMouseX ), getY() + ( e->button.y - lastMouseY ));
		lastMouseX = e->button.x;
		lastMouseY = e->button.y;
		return true;
	}
	else
	{
		dragging = false;
		return false;
	}
}
