/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "AppBase.h"

AppBase* AppBase::CurrentApp;

AppBase::AppBase( SDL_Surface* surface )
	: AWindow( surface )
{
//	windowsList = new LIST();
	setVisible( true );				// Although a window is better invisible at the creation,
									// application's main window should be visible all the time...
	animatorList = new AList<GUIAnimator>;
	
	done = false;
	
	AppBase::CurrentApp = this;		// Declare itself as the current application
	
	setDefaultAcceptKey( true );	// accept key event
}

AppBase::~AppBase()
{
//	if( windowsList )
//		delete windowsList;
}

int
AppBase::addWindow( AWindow* window, int x, int y )
{
	addChild( window, x, y );
	addListener( window );
    return children->Size();
}

void
AppBase::removeWindow( AWindow* window )
{
	removeChild( window->getPartIndex() );
	removeListener( window->getCommanderIndex());
	updateChildrenIndex();
	updateCommanderIndex();
}

void
AppBase::setFrontWindow( AWindow* window )
{
	if( getFrontWindow() != window )
	{
		removeWindow( window );
		addWindow( window, window->getX(), window->getY() );
		window->restoreAcceptStatus();
		requestUpdate( window->getRectRef());
		// If the front window changes, focus should be changed, too.
		// NOTE:	this is a default behavior.
		//			In some cases, it should be reset to default listener.
		//			Inherite the AppBase class and do whatever you want.
		requestMouseFocus( NULL, false );
		requestKeyFocus( NULL, false );
	}
}

AWindow*
AppBase::getFrontWindow()
{
	if( children )
		return (AWindow*)(GUIPart*)(*children)[ children->Size() - 1 ];
	else
		return NULL;
}

int
AppBase::eventLoop()
{
	while( ! done )
	{
		SDL_Event e;
		if( SDL_PollEvent( &e ))
			handleCommand( &e );
		else
			idle();
		
		animate();
	}
	return 0;
}

// This eventLoop() function might be useful if main event loop exists
// in other place. Argument defines number of events this function deals with.
int
AppBase::eventLoop( int numEvents )
{
	int i = 0;
	while( i < numEvents )
	{
		SDL_Event e;
		if( SDL_PollEvent( &e ))
			handleCommand( &e );
		else
			idle();
			
		animate();
		i++;
	}
	return 0;
}

BOOL
AppBase::animate()
{
	if( animatorList && animatorList->Size() > 0 )
	{
		for( Uint32 i = 0; i<animatorList->Size(); i++)
			(*animatorList)[i]->animate();
	}
	return true;
}

void
AppBase::addAnimator( GUIAnimator* animator )
{
	if( animatorList )
		animatorList->Add( animator );
}

AWindow*
AppBase::findWindowAt( int x, int y )
{
return NULL;
}

BOOL
AppBase::requestFocus( int windowID, int focusType )
{
	switch( focusType )
	{
		case MOUSE_FOCUS:
			mouseFocusWindowID = windowID;
			break;
		
		case KEY_FOCUS:
			keyFocusWindowID = windowID;
			break;
		
		case MOUSE_AND_KEY_FOCUS:
			mouseFocusWindowID = keyFocusWindowID = windowID;
			break;
		
		default:
			mouseFocusWindowID = 0;
			keyFocusWindowID = 0;
			break;
	}
	
	return true;
}