/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "../general.h"
#include "GUIPart.h"
#include "../DataLoader.h"
#include "../Screen.h"
#include "../../client/Renderer.h"

#include "../FilePath.h"
#include "../Log.h"

GUIPart::GUIPart( SDL_Surface* surface, GUIPart* parent )
{
	this->surface = surface;
	this->parent = parent;
	x = y = 0;
	partRect.x = partRect.y = partRect.w = partRect.h = 0;
	if( surface )
	{
//		SDL_SetColorKey( surface, SDL_SRCCOLORKEY,SDL_MapRGB( surface->format, 0, 0, 0 ));
		partRect.w = surface->w;
		partRect.h = surface->h;
	}
	
	children = NULL;
	if( parent && ! ( partIndex = parent->addChild( this, x, y) ))
		this->parent = NULL;
	
	visible = true;
	hilite = false;
	
}

GUIPart::GUIPart( AString filename, GUIPart* parent )
{
	FilePath path( filename );
	path.getValidFilePath( filename );
	
    SDL_Surface *tmp= SDL_LoadBMP( filename );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)filename);
      }

    this->surface = SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
	 
	this->parent = parent;
	x = y = 0;
	partRect.x = partRect.y = partRect.w = partRect.h = 0;
	if( surface )
	{
//		SDL_SetColorKey( surface, SDL_SRCCOLORKEY,SDL_MapRGB( surface->format, 0, 0, 0 ));
		partRect.w = surface->w;
		partRect.h = surface->h;
	}
	
	children = NULL;
	if( parent && ! ( partIndex = parent->addChild( this, x, y) ))
		this->parent = NULL;
	
	visible = true;
	hilite = false;
	
}

GUIPart::~GUIPart()
{
	if( children )
		children->Clear();
	
	SDL_FreeSurface( surface );
}

//	Virtual functions.

void
GUIPart::show()
{
	setVisible( true );
}

void
GUIPart::hide()
{
	setVisible( false );
}

// Function to acctually draw ( blit ) the surface to video surface.
// Inherite this function for custome drawing.
// localClip is a clip rectangle of this GUI surface, and 
// globalClip is that of real video surface.
// Default behavior is just to blit the GUI surface to video surface.
void
GUIPart::draw( SDL_Rect* localClip, SDL_Rect* globalClip )
{
	if( surface )
		//SDL_BlitSurface( surface, localClip, SDL_GetVideoSurface(), globalClip );
		SDL_BlitSurface( surface, localClip, Screen::GetScreen(), globalClip );
}

// Function to determin if the specified point ( ptx, pty ) is inside of
// the ( clickable, almost in any case, I think ) area.
// Default behavior suppose the GUI part is a simple rectangle.
// Override this function for a much complicated shape.
//
// IMPORTANT:
// ptx and pty are supposed to be globally coordinated.
BOOL
GUIPart::ptInPart( int ptx, int pty )
{
	return ( ptx >= partRect.x && ptx <= partRect.x + partRect.w &&
			 pty >= partRect.y && pty <= partRect.y + partRect.h );
}

// Public functions

void
GUIPart::setSurface( SDL_Surface* surface, BOOL updt )
{
	if( surface )
		SDL_FreeSurface( surface );
	
	requestUpdate( &partRect );			// Update the screen.
	
	this->surface = surface;
	partRect.w = surface->w;
	partRect.h = surface->h;
	if( updt )
		requestUpdate( &partRect );
}

void
GUIPart::setSurface( AString filepath, BOOL updt )
{
	if( surface )
		SDL_FreeSurface( surface );
	
	requestUpdate( &partRect );			// Update the screen.
	
	FilePath path( filepath );
	path.getValidFilePath( filepath );

    SDL_Surface *tmp= SDL_LoadBMP( filepath );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)filepath);
      }

    this->surface = SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);

	partRect.w = surface->w;
	partRect.h = surface->h;
	if( updt )
		requestUpdate( &partRect );
}

SDL_Surface*
GUIPart::getSurface()
{
	return surface;
}

void
GUIPart::setParent( GUIPart* parent, int index)
{
/*	if( this->parent )
		this->parent->removeChild( partIndex );*/
	if( parent && index )
		this->parent = parent;
	else if( parent == NULL )
	{
		this->parent = NULL;
		partIndex = 0;
	}
	partIndex = index;
}

GUIPart*
GUIPart::getParent()
{
	return parent;
}

void
GUIPart::setSize( int w, int h )
{
	partRect.w = w;
	partRect.h = h;
}

// Set the local location of the GUI.
// This x and y specifies the location IN THE PARENT GUI.
void
GUIPart::setLocalLocation( int x, int y, BOOL updt )
{
	int dx = x - this->x;
	int dy = y - this->y;
	
	this->x = x;
	this->y = y;
	SDL_Rect temp;
	temp.x = partRect.x;
	temp.y = partRect.y;
	temp.w = partRect.w;
	temp.h = partRect.h;
	if( parent )
	{
		SDL_Rect r;
		parent->getRect( &r );
		partRect.x = x + r.x;
		partRect.y = y + r.y;
	}
	else
	{
		partRect.x = x;
		partRect.y = y;
	}
	
	if( children )
	{
		int i = 0;
		while( i < children->Size())
		{
			GUIPart* child = (GUIPart*)(*children)[i];
			if( child )
			{
				child->recalcPartRect();
			}
			i++;
		}
	}
	
	if( updt )
	{
/*		SDL_Rect deltaA, deltaB;
		// right-down move
		if( temp.x <= partRect.x && temp.y <= partRect.y )
		{
			deltaA.x = deltaB.x = temp.x;
			deltaA.y = deltaB.y = temp.y;
			deltaA.w = temp.w;
			deltaA.h = partRect.y - temp.y;
		}
		// right-up move
		else if( temp.x <= partRect.x && temp.y >= partRect.y )
		{
			deltaA.x = deltaB.x = temp.x;
			deltaA.y = temp.y;
			deltaB.y =partRect.y + partRect.h;
			deltaA.w = partRect.x - temp.x;
			deltaB.w = temp.w;
			deltaA.h = temp.h;
			deltaB.h = temp.y - partRect.y;
		}
		// left-down move
		else if( temp.x >= partRect.x && temp.y <= partRect.y )
		{
			deltaA.x = temp.x;
			deltaA.y = deltaB.y = temp.y;
			deltaB.x = partRect.x + partRect.w;
			deltaA.w = temp.w;
			deltaB.w = temp.x - partRect.x;
			deltaA.h = partRect.y - temp.y;
			deltaB.h = temp.h;
		}
		// left-up move
		else
		{
			deltaA.x = partRect.x + partRect.w;
			deltaB.x = temp.x;
			deltaA.y = temp.y;
			deltaB.y = partRect.y + partRect.h;
			deltaA.w = temp.x - partRect.x;
			deltaB.w = temp.w;
			deltaA.h = temp.h;
			deltaB.h = temp.y - partRect.h;
		}
		
		if( deltaA.w && deltaA.h )
			requestUpdate( &deltaA );
		if( deltaB.w && deltaB.h )
			requestUpdate( &deltaB );*/
		temp.w += abs( temp.x - partRect.x );
		temp.h += abs( temp.y - partRect.y );
		if( temp.x > partRect.x )
			temp.x = partRect.x;
		if( temp.y > partRect.y )
			temp.y = partRect.y;
		requestUpdate( &temp );			// First update the old rectangle. Not very efficient....
//		requestUpdate( &partRect );		// Then draw the new rectangle.
	}
}

// Update partRect.
void
GUIPart::recalcPartRect()
{
	if( parent )
	{
		SDL_Rect r;
		parent->getRect( &r );
		partRect.x = r.x + x;
		partRect.y = r.y + y;
	}
}

// Keep in mind that you must be careful not to destroy the returned SDL_Rect.
// If not sure, you should get a copy by calling GUIPart::getRect( SDL_Rect* ).
SDL_Rect*
GUIPart::getRectRef()
{
	return &partRect;
}

void
GUIPart::getRect( SDL_Rect* r )
{
	r->x = partRect.x;
	r->y = partRect.y;
	r->w = partRect.w;
	r->h = partRect.h;
}

// This function may seem odd, but useful when there is no surface object
// for this GUI.... as in AppBase class. There may be no graphics to draw,
// but it needs to have some surface which is equal to video surface...
// I'm not sure...
void
GUIPart::setRect( SDL_Rect* r )
{
	partRect.x = r->x;
	partRect.y = r->y;
	partRect.w = r->w;
	partRect.h = r->h;
}

int
GUIPart::getX()
{
	return x;
}

int
GUIPart::getY()
{
	return y;
}

int
GUIPart::getWidth()
{
	return partRect.w;
}

void
GUIPart::setWidth( int w )
{
	partRect.w = w;
}

int
GUIPart::getHeight()
{
	return partRect.h;
}

void
GUIPart::setHeight( int h )
{
	partRect.h = h;
}

void
GUIPart::setHilite( BOOL hilite )
{
	this->hilite = hilite;
}

BOOL
GUIPart::isHilite()
{
	return hilite;
}

void
GUIPart::setVisible( BOOL visible )
{
	this->visible = visible;
	requestUpdate( &partRect );
}

BOOL
GUIPart::isVisible()
{
	return visible;
}

int
GUIPart::getPartIndex()
{
	return partIndex;
}

void
GUIPart::setPartIndex( int index )
{
	partIndex = index;
}

void
GUIPart::updateChildrenIndex()
{
	if( children )
	{
		GUIPart* child;
		int i = 0;
		while( i < children->Size())
		{
			child = (GUIPart*)(*children)[i];
			child->setPartIndex( i + 1 );
			i++;
		}
	}
}

int
GUIPart::addChild( GUIPart* child, int x, int y )
{
	if( ! children )
		children = new LIST();
	
	if( children )
	{
		children->Add( child, children->Size());
		child->setParent( this, children->Size());
		child->setLocalLocation( x, y );
	}
		
	requestUpdate( child->getRectRef() );
	
	return children->Size();
}

void
GUIPart::removeChild( int index )
{
	if( children )
	{
		GUIPart* child = (GUIPart*)children->Remove( index - 1 );
		requestUpdate( child->getRectRef());
		child->setParent( NULL, 0 );
	}
}

void
GUIPart::removeChild( GUIPart* part )
{
	if( children )
	{
		int i = 0;
		GUIPart* child;
		while( i < children->Size())
		{
			child = (GUIPart*)(*children)[i];
			if( child == part )
			{
				children->Remove(i);
				child->setParent( NULL, 0 );
				break;
			}
		}
	}
}

LIST*
GUIPart::getChildren()
{
	return children;
}

// VERY IMPORTANT:
// X and Y of SDL_Rect here is NOT a relative location.
// They MUST be absolute location on video surface.
// We can use GUIPart::partRect for redrawing entire surface of
// the GUI part; or calculate clip region, if appropriate.
//
void
GUIPart::requestUpdate( const SDL_Rect* r )
{
	if( parent )
	{
		parent->requestUpdate( r );
	}
	else				// O.K., this is the AppBase. Dispatch the request.
	{
		if( r )
		{
			prepareDraw( r );
			SDL_Rect clip;
			// Calculate update rectangle of Screen
			clip.x = r->x >= 0 ? r->x : 0;
			clip.y = r->y >= 0 ? r->y : 0;
			clip.w = r->x >= 0 ? r->w : r->w + r->x;
			if( clip.x + clip.w > Screen::GetScreen()->w )
				clip.w -= clip.x + clip.w - Screen::GetScreen()->w;
			clip.h = r->y >= 0 ? r->h : r->h + r->y;
			if( clip.y + clip.h > Screen::GetScreen()->h )
				clip.h -= clip.y + clip.h - Screen::GetScreen()->h;
			
			// Update the screen
			SDL_UpdateRects( Screen::GetScreen(), 1, &clip );
		}
		else
			prepareDraw(r);
/*		
		if( children )
		{
			int i = 0;
			GUIPart* child;
			while( i < children->Size() && ( child = (GUIPart*)(*children)[i] ) )
			{
				child->prepareDraw( r );
				i++;
			}
		}*/	
	}
}

// Private functions

// Actually draw the gfx to the screen
// Called from requestUpdate(), when it reaches the application's base window,
// most presumablly AppBase class.
void
GUIPart::prepareDraw( const SDL_Rect* updateRect )
{
	if( visible )
	{
		SDL_Rect clip, localClip;
		calcClipRect( updateRect, &clip );
		
		// If the calculated update rectangle has no surface ( no width/height )
		// we don't have to draw.
		if( clip.w > 0 && clip.h > 0 )
		{
			localClip.x = clip.x - partRect.x;
			localClip.y = clip.y - partRect.y;
			localClip.w = clip.w;
			localClip.h = clip.h;
		//	SDL_BlitSurface( surface, &localClip, SDL_GetVideoSurface(), &clip );
			draw( &localClip, &clip );
		}
		
		if( children )
		{
			int i = 0;
			GUIPart* child;
			while( i < children->Size() && ( child = (GUIPart*)(*children)[i] ) )
			{
				child->prepareDraw( &clip );
				i++;
			}
		}
	}

}

// Stores the background graphics.
// Usefull to redraw what it was there...
// ...well, it's what ParaGUI does anyway....
void
GUIPart::storeBackground()
{
	if( ! background )
	{
		background = SDL_CreateRGBSurface(
			SDL_SWSURFACE,
			partRect.w, partRect.h,
			16, 0, 0, 0, 0);
	}
	if( background )
	{
		SDL_Rect r;
		r.x = 0;
		r.y = 0;
		r.w = partRect.w;
		r.h = partRect.h;
		SDL_BlitSurface( parent->getSurface(), &partRect, background, &r );
	}

}

// Redraws background.
// Call this function when moving the GUI to other place.
void
GUIPart::restoreBackground()
{
	if( parent )
	{
		SDL_BlitSurface( background, NULL, parent->getSurface(), &partRect );
	}
}

// Calculate what is intersected in given rect.
// Returns the result in clip.
// If nothing is intersected, clip->w and clip->h will be set 0.
void
GUIPart::calcClipRect( const SDL_Rect* rect, SDL_Rect* clip )
{
	// init
	clip->x = clip->y = clip->w = clip->h = 0;
	
	// If no update region is specified,
	// update the whole GUI.
	if( ! rect )
	{
		clip->x = partRect.x;
		clip->y = partRect.y;
		clip->w = partRect.w;
		clip->h = partRect.h;
		return;
	}
	// First, test if rect intersects partRect
	if( rect->x + rect->w < partRect.x )				// far too left
		return;
	if( rect->x > partRect.x + partRect.w )				// far too right
		return;
	if( rect->y + rect->h < partRect.y )				// far too above
		return;
	if( rect->y > partRect.y + partRect.h )				// far too below
		return;
	// O.K., calculate the clip rectangle
	clip->x = partRect.x > rect->x ? partRect.x : rect->x;
	clip->y = partRect.y > rect->y ? partRect.y : rect->y;
	clip->w = partRect.x + partRect.w <= rect->x + rect->w ?
		partRect.w - ( clip->x - partRect.x ) :
		rect->x + rect->w - clip->x;
	clip->h = partRect.y + partRect.h <= rect->y + rect->h ?
		partRect.h - ( clip->y - partRect.y ) :
		rect->y + rect->h - clip->y;
}

// Given the update rectangle, convert it to fit in this GUI object rectangle.
/*void
GUIPart::validateUpdateRect( SDL_Rect* updateRect, SDL_Rect* outRect )
{
	if( updateRect->x + updateRect->w < partRect.x ||
		updateRect->x + updateRect->w > partRect.x ||
		updateRect->y + updateRect->h < partRect.y ||
		updateRect->y + updateRect->h > partRect.y )
	{
		// updateRect is outside of the GUI regtangle.
		outRect->w = outRect->h = 0;
	}
	else
	{
		if( updateRect->x < partRect.x )
			outRect->x = partRect.x;
		if( updateRect->x + updateRect->w > partRect.x + partRect.w )
			outRect->w = partRect.x + partRect.w - updateRect->x;
		if( updateRect->y < partRect.y )
			outRect->y = partRect.y;
		if( updateRect->y + updateRect->h > partRect.y + partRect.h )
			outRect->h = partRect.y + partRect.h - updateRect->y;
	}
}*/
