#ifndef GUIPART_H
#define GUIPART_H

#include "../general.h"
#include "GUIPart.h"
#include "../List.h"

class GUIPart : public Disposable
{
	public:

	LIST*			children;
	
	GUIPart( SDL_Surface* surface, GUIPart* parent = NULL );
	GUIPart( AString filename, GUIPart* parent = NULL );
	~GUIPart();

	// Virtual functions.
	// Override them to implement custom drawing.
	
	virtual void	show();			// Show the window
	virtual void	hide();			// Hide the window ( not destroy )
	virtual void	draw( SDL_Rect* localRect, SDL_Rect* globalRect );	// Actually draw ( blit ).
	virtual BOOL	ptInPart( int ptx, int pty );

	// hilite / visible
	virtual void	setHilite( BOOL hilite );
	virtual BOOL	isHilite();
	virtual void	setVisible( BOOL visible );
	virtual BOOL	isVisible();

	// update request
	virtual void	requestUpdate( const SDL_Rect* r);	// request update.
	
	// adding / removing child GUI
	virtual int		addChild( GUIPart* part, int x, int y);
	virtual void	removeChild( int index );
	virtual void	removeChild( GUIPart* part );

	// Public functions.
	void			setParent( GUIPart* parent, int index );
	GUIPart*		getParent();

	void			setSurface(SDL_Surface* surface, BOOL updt);
	void			setSurface( AString filepath, BOOL updt );
	SDL_Surface*	getSurface();
	
	void			setSize( int w, int h);
	void			setLocalLocation( int x, int y, BOOL updt = TRUE );
	void			recalcPartRect();
	void			setWidth( int w );
	void			setHeight( int h );
	
	SDL_Rect*		getRectRef();
	void			getRect( SDL_Rect* r );
	void			setRect( SDL_Rect* r );
	int				getX();
	int				getY();
	int				getWidth();
	int				getHeight();
	
	int				getPartIndex();
	void			setPartIndex( int index );
	void			updateChildrenIndex();
	
	LIST*			getChildren();
	
//	private:
	protected:
	
	SDL_Rect		partRect;
	int				x, y, partIndex;
	GUIPart*		parent;
	SDL_Surface*	surface;
	SDL_Surface*	background;
	BOOL			hilite, visible;
	
	private:
	
	void			prepareDraw( const SDL_Rect* r );	// Prepare to draw the gfx to the screen.
	void			storeBackground();
	void			restoreBackground();
	void			calcClipRect( const SDL_Rect* rect, SDL_Rect* clip );
	//void			validateUpdateRect( SDL_Rect* updateRect, SDL_Rect* outRect );
};
#endif