/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Label.h"
#include "../Screen.h"

Label::Label( AString bkground, const char* text )
	: GUIPart( bkground, NULL )
{
	this->text = NULL;
	this->font = NULL;
	this->labelSurface = NULL;
	color.r = color.g = color.b = 255;
	setText( text );

}

Label::~Label()
{
	if( font )
	{
	#ifdef USE_SDL_TTF
		TTF_CloseFont( font );
	#endif // USE_SDL_TTF
		delete font;
	}
}

void
Label::draw( SDL_Rect* localClip, SDL_Rect* globalClip )
{
	if( getSurface() )
		SDL_BlitSurface( getSurface(), localClip, Screen::GetScreen(), globalClip );
	if( font )
	{
	#ifdef USE_SDL_TTF
		if( labelSurface )
			SDL_BlitSurface( labelSurface, localClip, Screen::GetScreen(), globalClip );
	#else
		if( text )
			font->Draw( Screen::GetScreen(),
				partRect.x + partRect.w / 2,
				partRect.y + partRect.h / 2
					, text, AFont::CENTER );
	#endif // USE_SDL_TTF
	}
}

void
Label::setText( const char* text )
{
	if( this->text )
		delete[] this->text;
	
	this->text = new char[ strlen( text )];
	strcpy( this->text, text );
	resetText();
}

void
Label::getText( char* outText )
{
	strcpy( text, outText );
}

#ifdef USE_SDL_TTF
void
Label::setFont( TTF_Font* font )
{
	this->font = font;
	resetText();
}

TTF_Font*
Label::getFont()
{
	return font;
}

#else

BitmapFont*
Label::getFont()
{
	return font;
}

#endif	// USE_SDL_TTF

void
Label::setFont( AString filepath )
{
	FilePath path( filepath );
	path.getValidFilePath( filepath );
#ifdef USE_SDL_TTF
	if( font )
		TTF_CloseFont( font );
	font = TTF_OpenFont( filepath, 14 );
#else
	if( font )
		delete font;
	
	font = new BitmapFont( filepath );
#endif // USE_SDL_TTF
	resetText();
}

void
Label::setColor( SDL_Color* color )
{
	this->color.r = color->r;
	this->color.g = color->g;
	this->color.b = color->b;
	this->color.unused = color->unused;
	resetText();
}

SDL_Color*
Label::getColor()
{
	return &color;
}

void
Label::resetText()
{
//	if( font )
//		setSurface( TTF_RenderText_Blended( font, text, color ), false );
//		font->Draw( getSurface(), 0, 0, text, AFont::LEFT );
#ifdef USE_SDL_TTF
	if( labelSurface )
		SDL_FreeSurface( labelSurface );
	if( font && text )
	{
		labelSurface = TTF_RenderText_Blended( font, text, color );
		SDL_SetColorKey( labelSurface, SDL_SRCCOLORKEY,
			SDL_MapRGB( labelSurface->format, 0, 0, 0 ));
	}
		
#endif
}

