/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "LineEdit.h"
#include "../FilePath.h"
#include "../DataLoader.h"
#include "../Screen.h"
#include "../Log.h"
#include "GUICommandSet.h"

#include <string.h>

LineEdit::LineEdit( int w, int h)
/*	: GUIPart( NULL )*/
	: AWidget( NULL )
{
	x = partRect.x = 0;
	y = partRect.y = 0;
	partRect.w = w;
	partRect.h = h;
	
	// Inherites from ACommander
	setDefaultAcceptMouse( true );
	setDefaultAcceptKey( true );
	setAcceptMouse( true );
	setAcceptKey( true );
	
	carretPos = selectStart = selectEnd = 0;		// Init selection
	mouseDragging = false;							// Useful for selecting by mouse
	lastKey = 0;
	viewPos = 0;
	
	// Init text buffer.
	text = new char[ MAX_LEN + 1 ];
	
#ifndef USE_SDL_TTF
	textToDraw = new char[ MAX_LEN + 1 ];			// This is used to actually draw text
#endif
	for( int i = 0; i <= MAX_LEN; i++)
		text[i] = textToDraw[i] = 0;
	
	// Options -- not implemented
	passwordMode = false;				// Password mode is an option.
	
	absorbKeyEvent = false;				// Absorption of keyborad events is an option.
	
	// gfx
	surface = SDL_CreateRGBSurface(
		SDL_SWSURFACE,
		partRect.w, partRect.h,
		DataLoader::GetDepth(), 0, 0, 0, 0);
	
	lineSurface = NULL;
	font = NULL;

	// Carret
	AString carretPath("gfx/gui/LineEdit/carret.bmp");
	FilePath path( carretPath );
	path.getValidFilePath( carretPath );

	SDL_Surface *tmp = SDL_LoadBMP( carretPath );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)carretPath);
      }

	if( tmp )
		SDL_SetColorKey( tmp, SDL_SRCCOLORKEY,SDL_MapRGB( tmp->format, 0, 0, 0 ));
    carret = SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);

	
	// LineEditGroup
	group = NULL;
	// return-key call back function
	returnFunc = NULL;
	// default command type
	commandType = ARIANNE_COM_TEXT_SET;
}

LineEdit::~LineEdit()
{
	if( text )
		delete text;
#ifndef USE_SDL_TTF
	if( textToDraw )
		delete textToDraw;
#endif // USE_SDL_TTF
	if( surface )
		SDL_FreeSurface( surface );
	if( lineSurface )
		SDL_FreeSurface( lineSurface );
}

void
LineEdit::draw( SDL_Rect* localRect, SDL_Rect* globalRect )
{
	if( surface )
		SDL_BlitSurface( surface, localRect, Screen::GetScreen(), globalRect );
#ifdef USE_SDL_TTF
	if( lineSurface )
		SDL_BlitSurface( lineSurface, localRect, Screen::GetScreen(), globalRect );
#else
	if( font && text )
	{
		int w = 0;
		Uint32 i = 0;
		while( i < strlen( text ) && w < partRect.w )
		{
			textToDraw[i] = text[i + viewPos];
			textToDraw[ i + 1 ] = 0;
			w = font->GetWidth( textToDraw );
			i++;
		}
		
		font->Draw( Screen::GetScreen(),
			partRect.x, partRect.y, textToDraw, AFont::LEFT + AFont::TOP );
	}
		
#endif // USE_SDL_TTF
	
	// Draw carret
	if( isFocus() && carret && textToDraw && font )
	{
	#ifndef USE_SDL_TTF
		textToDraw[ carretPos - viewPos ] = 0;
		int cx = partRect.x + font->GetWidth( textToDraw );
		SDL_Rect cr;
		cr.x = cx ; cr.y = partRect.y;
		cr.w = carret->w ; cr.h = carret->h;
		SDL_BlitSurface( carret, NULL, Screen::GetScreen(), &cr );
	#endif // USE_SDL_TTF
	}
}

BOOL
LineEdit::handleMouseDown( SDL_Event* e )
{
	if( ptInPart( e->button.x, e->button.y ))
	{
		lastMouseDown = SDL_GetTicks();
		return true;
	}
	else
		return false;
}

BOOL
LineEdit::handleMouseMove( SDL_Event* e )
{
	if( lastMouseDown > 0 && ! ptInPart( e->motion.x, e->motion.y ))
	{
		lastMouseDown = 0;
		return true;
	}
		
	return false;
}

BOOL
LineEdit::handleMouseUp( SDL_Event* e )
{
	if( /*lastMouseDown > 0 && */ptInPart( e->button.x, e->button.y ))
	{
		if( commander )
		{
			commander->requestKeyFocus( this, true );
		}
		carretPos = getPosition( e->button.x );
		return true;
	}
	else
		return false;
}


BOOL
LineEdit::handleKeyDown( SDL_Event* e )
{
	if( isFocus() && doesAcceptKey() )
	{
	// We need here to determin if the key pressed is displayable character or not
		switch( e->key.keysym.sym )
		{
			case SDLK_LEFT:			// Left arrow
				if( carretPos > 0 )
					carretPos --;
				if( viewPos > 0 )
					viewPos --;
				break;
			case SDLK_RIGHT:		// Right arrow
				if( carretPos < strlen( text ) - 1 )
					carretPos ++;
				int i;
				for( i = viewPos; i <= carretPos ; i++ )
					textToDraw[i - viewPos] = text[i];
				textToDraw[i] = 0;
				if( font->GetWidth( textToDraw ) > partRect.w )
					viewPos++;
				break;
			
			case SDLK_DELETE:		// Delete or,
			case SDLK_BACKSPACE:	// Back Space
				if( carretPos > 0 )
					carretPos = deleteCharAt( carretPos - 1 );
				else
					carretPos = deleteCharAt( carretPos );
				if( viewPos > 0 )
					viewPos --;
				adjustVisibleArea();
				break;
			
			case SDLK_TAB:
				if( group )
					group->next();
				break;
			
			case SDLK_RETURN:
				if( returnFunc )
					returnFunc( text );
				if( listeners )
				{
                for(Uint32 i=0;i<listeners->Size();i++)
                  {
				  char* com = new char[ strlen( text ) + 1];
				  memcpy( com, text, strlen( text ));
				  com[strlen( text )] = 0;
				  (*listeners)[i]->execCommand(
						new GenericCommand( com, getCommandType() ));
                  }
				}
				setText(NULL);
				break;
			
			default:				// Default -- insert the char
				int pos = insertCharAt( (Uint8)e->key.keysym.unicode, carretPos );
				if( pos >= 0 )
					carretPos = pos + 1;	// Moves the carret.
				
				adjustVisibleArea();		// Adjust visible text, so that the printed text
											// fits in the partRect.
		}
		requestUpdate( &partRect );
		
		return true;
	}
	else
		return false;
}

BOOL
LineEdit::handleKeyUp( SDL_Event* e )
{
	return false;
}

const char*
LineEdit::getText()
{
	return text;
}

// Be sure to allocate enough memory for outText.
void
LineEdit::getText( char* outText)
{
	strcpy( outText, text );
}

void
LineEdit::setText( char* inText )
{
	if( ! inText )
		text[0] = textToDraw[0] = 0;
	else
	{
		if( strlen( inText ) > MAX_LEN )
			inText[MAX_LEN] = 0;
		
		strcpy( text, inText );
	}
	viewPos = 0;
	carretPos = 0;
	requestUpdate( &partRect );
}

void
LineEdit::clearText()
{
	text[0] = 0;
	
}

void
LineEdit::setPasswordMode( BOOL mode )
{
	passwordMode = mode;
	requestUpdate( &partRect );
}

BOOL
LineEdit::isPasswordMode()
{
	return passwordMode;
}

int
LineEdit::getSelectStart()
{
	return selectStart;
}

int
LineEdit::getSelectEnd()
{
	return selectEnd;
}

int
LineEdit::getSelectionLength()
{
	return selectEnd - selectStart;
}

// Be sure to allocate enough memory for outText
void
LineEdit::getSelection( char* outText )
{
	strcpy( outText, text );
}

void
LineEdit::select( int start, int length )
{
	selectStart = start;
	selectEnd = start + length;
	if( selectStart < selectEnd )
		selectEnd = selectStart;
	if( selectStart < 0 )
		selectStart = 0;
	if( selectEnd > MAX_LEN )
		selectEnd = MAX_LEN;
}

void
LineEdit::setReturnFunction( void (*f)(char*))
{
	returnFunc = f;
}

void
LineEdit::removeReturnFunction()
{
	returnFunc = NULL;
}

#ifdef USE_SDL_TTF
void
LineEdit::setFont( TTF_Font* font )
{
	this->font = font;
	resetText();
}

TTF_Font*
LineEdit::getFont()
{
	return font;
}

#else

BitmapFont*
LineEdit::getFont()
{
	return font;
}

#endif	// USE_SDL_TTF

void
LineEdit::setFont( AString filepath )
{
	FilePath path( filepath );
	path.getValidFilePath( filepath );
#ifdef USE_SDL_TTF
	if( font )
		TTF_CloseFont( font );
	font = TTF_OpenFont( filepath, 14 );
#else
	if( font )
		delete font;
	
	font = new BitmapFont( filepath );
#endif // USE_SDL_TTF
	resetText();
}

// Private functions

// insert a char at pos. Returns the actual position of the inserted character.
// IMPORTANT: pos is supposed to be 0 based number. So text[0] is pos = 0.

int
LineEdit::insertCharAt( char c, Uint8 pos )
{
	if( strlen( text ) < MAX_LEN )	// There's a room
	{
		if( pos < strlen( text ))
		{
			memmove( text + pos + 1, text + pos, strlen( text ) - pos);
			text[pos] = c;
			resetText();
			return pos;
		}
		else			// You can pass LineEdit::MAX_LEN as pos value to add to the end.
		{
			text[ strlen( text ) + 1] = 0;
			text[ strlen( text )] = c;
			resetText();
			return strlen( text ) - 1;
		}
	}
	return -1;
}

// inserts a potion of text at pos. Returns the actual position of the inserted text.
// IMPORTANT: pos is supposed to be 0 based number. So text[0] is pos = 0;

int
LineEdit::insertTextAt( char* t, Uint8 pos )
{
	if( t && strlen( text ) <= MAX_LEN - strlen( t )) // There's a room
	{
		if( pos < strlen( text ))
		{
			memmove( text + pos + strlen( t ), text + pos, strlen( text ) - pos );
			memcpy( text + pos, t, strlen( t ));
			resetText();
			return pos;
		}
		else		// You can pass LineEdit::MAX_LEN as pos value to add to the end
		{
			text[ strlen( text ) + strlen( t )] = 0;
			memcpy( text + strlen( text ), t, strlen( t ));
			resetText();
			return strlen( text ) - 1;
		}
	}
	return -1;
}

int
LineEdit::deleteCharAt( Uint8 pos )
{
	if( pos <= strlen( text ))
	{
		Uint32 i;
		for( i = pos; i < strlen( text ); i ++)
		{
			text[i] = text[ i + 1];
		}
		text[i] = 0;
		
		resetText();
		return pos;
	}
	return -1;
}

int
LineEdit::deleteTextAt( Uint8 pos, Uint8 num )
{
	if( pos <= strlen( text ) && num > 0 )
	{
		num = (Uint32)(pos + num) > strlen( text ) ? strlen( text ) - pos : num ;
		memmove( text + pos, text + pos + num, strlen( text ) - ( pos + num ));
		return pos;
	}
	return -1;
}

void
LineEdit::resetText()
{
//	if( font )
//		setSurface( TTF_RenderText_Blended( font, text, color ), false );
//		font->Draw( getSurface(), 0, 0, text, AFont::LEFT );
#ifdef USE_SDL_TTF
	if( lineSurface )
		SDL_FreeSurface( lineSurface );
	if( font && text )
	{
		lineSurface = TTF_RenderText_Blended( font, text, color );
		SDL_SetColorKey( lineSurface, SDL_SRCCOLORKEY,
			SDL_MapRGB( lineSurface->format, 0, 0, 0 ));
	}
		
#endif
}

void
LineEdit::adjustVisibleArea()
{
	if( font && text )
	{
		int w = 0;
		int i = 0;
		if( carretPos < viewPos )
			viewPos = carretPos;
			
		while( (Uint32)(i + viewPos) < strlen( text ) && w <= partRect.w )
		{
			textToDraw[i] = text[i + viewPos];
			textToDraw[ i + 1 ] = 0;
			w = font->GetWidth( textToDraw );
			i++;
		}
		if( w > partRect.w && carretPos >= i - 1 + viewPos )
		{
			i = 1;
			while( w > partRect.w )
			{
				viewPos++;
				w = font->GetWidth( textToDraw + i );
				i++;
			}
		}
	}
	
}

// This function is used internally to get carret position when clicked.
// Hope you understand what I'm struggling...!
Uint8
LineEdit::getPosition( int x )
{
	int w = 0;
	int i;
	
	for( i = 0; (Uint32)(i + viewPos) < strlen( text ); i++ )
	{
		textToDraw[ i ] = text[ i + viewPos ];
		textToDraw[ i + 1 ] = 0;
		w = font->GetWidth( textToDraw );
		if( w > x - partRect.x )
			break;
	}
	
	if( w > x - partRect.x )
		return i + viewPos + 1;
	else
		return strlen(text) > 0 ? strlen( text ) - 1 : 0;
}
