#ifndef LINEEDIT_H
#define LINEEDIT_H

#include "../general.h"
#include "GUIPart.h"
#include "ACommander.h"
#include "AWidget.h"

#ifdef USE_SDL_TTF
  #include "SDL_ttf.h"
  #include "freetype.h"
#else
  #include "../../client/Font.h"
#endif // USE_SDL_TTF

class LineEdit : /*public GUIPart, public ACommander*/ public AWidget
{
	public:
	
		enum
		{
			KEY_REPEAT = 100,
			MAX_LEN = 255
		};
		
		LineEdit( int w, int h );
		~LineEdit();
		
		// Inherits from GUIPart
		virtual void draw( SDL_Rect* localRect, SDL_Rect* globalRect );
		
		// Inherites from ACommander
		virtual BOOL handleMouseDown( SDL_Event* e );
		virtual BOOL handleMouseUp( SDL_Event* e );
		virtual BOOL handleMouseMove( SDL_Event* e );
		virtual BOOL handleKeyDown( SDL_Event* e );
		virtual BOOL handleKeyUp( SDL_Event* e );
		
		const char*	getText();
		void		getText( char* outText );
		void		setText( char* inText );
		void		clearText();
		void		setPasswordMode( BOOL mode );
		BOOL		isPasswordMode();
		int			getSelectStart();
		int			getSelectEnd();
		int			getSelectionLength();
		void		getSelection( char* outText );
		void		select( int start, int length );
		
		// Install / Remove call back proc
		
		void		setReturnFunction( void (*f)(char*));
		void		removeReturnFunction();
		
		// Font stuff
		
		#ifdef USE_SDL_TTF
		void			setFont( TTF_Font* font );
		TTF_Font*		getFont();
		#else
		BitmapFont*		getFont();
		#endif // USE_SDL_TTF
		void			setFont( AString filepath );
		void			setColor( SDL_Color* color );
		SDL_Color*		getColor();
		
	protected:
		char	*text;
	#ifndef USE_SDL_TTF
		char	*textToDraw;
	#endif
		Uint8		carretPos, selectStart, selectEnd, viewPos;
		char	lastKey;
		SDL_keysym lastKeySym;
		Uint32	lastKeyDown;
		BOOL	mouseDragging;
		BOOL	passwordMode;
		
	private:
		
		BOOL		absorbKeyEvent;
		SDL_Color		color;
	#ifdef USE_SDL_TTF
		TTF_Font*		font;
	#else
		BitmapFont*		font;
	#endif // USE_SDL_TTF
		SDL_Surface*	backgfound;
		SDL_Surface*	lineSurface; // holds text surface. will be rendered over background
		SDL_Surface*	carret;
		
		void			(*returnFunc)(char*);
		
		int			insertCharAt( char c, Uint8 pos );
		int			insertTextAt( char* t, Uint8 pos );
		int			deleteCharAt( Uint8 pos );
		int			deleteTextAt( Uint8 pos, Uint8 num );
		void		resetText();
		void		adjustVisibleArea();
		Uint8		getPosition( int x );
};

#endif LINEEDIT_H