/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Meter.h"
#include "../Screen.h"
#include "../FilePath.h"
#include "../Log.h"
#include "GUICommandSet.h"

Meter::Meter( AString bg_path, AString meter_path, AString cover_path, Uint8 mode )
	: AWidget( bg_path, NULL )
{
	if( surface )
		SDL_SetColorKey( surface, SDL_SRCCOLORKEY,SDL_MapRGB( surface->format, 0, 0, 0 ));
	
	FilePath path( meter_path );
	path.getValidFilePath( meter_path );

	SDL_Surface *tmp = SDL_LoadBMP( meter_path );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)meter_path);
      }

	if( tmp )
		SDL_SetColorKey( tmp, SDL_SRCCOLORKEY, SDL_MapRGB( tmp->format, 0, 0, 0 ));
    meterSurface=SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
    
    tmp = SDL_LoadBMP( cover_path );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)cover_path);
      }

	if( tmp )
		SDL_SetColorKey( tmp, SDL_SRCCOLORKEY, SDL_MapRGB( tmp->format, 0, 0, 0 ));
    coverSurface=SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
		
	max_value = 255;
	currentValue = 0;
	setDefaultAcceptMouse( false );
	setDefaultAcceptKey( false );
	setAcceptMouse( false );
	setAcceptKey( false );
	this->mode = mode;
}

Meter::~Meter()
{
	if( surface )
		SDL_FreeSurface( surface );
	if( meterSurface )
		SDL_FreeSurface( meterSurface );
	if( coverSurface )
		SDL_FreeSurface( coverSurface );
}

void
Meter::setValue( Uint32 v )
{
	currentValue = v;
	if( currentValue < 0 )
		currentValue = 0;
	if( currentValue > 255 )
		currentValue = 255;
}

Uint32
Meter::getValue()
{
	return currentValue;
}

BOOL
Meter::execCommand( GenericCommand* com )
{
	BOOL done = false;
	switch( com->type )
	{
		case ARIANNE_COM_SET_VALUE:
			Uint32* v = (Uint32*)com->GetCommand();
			setValue( *v );
			done = true;
			break;
	}
	return done;
}

void
Meter::draw( SDL_Rect* localClip, SDL_Rect* globalClip )
{
	// First draw 'surface' --- background,
	// then meterSurface --- which represents 'meter' part of the gfx,
	// and at last coverSurface --- which is drawn over 'meter', so that you can give a
	// glass cover ( or whatver ) touch to whole meter GUI..!
	if( surface )
		SDL_BlitSurface( surface, localClip, Screen::GetScreen(), globalClip );
	if( meterSurface )
	{
		SDL_Rect meterClip, meterGlobalClip;
		meterClip.x = 0;
//		meterClip.w = meterSurface->w;
		meterClip.y = meterSurface->h * ( 100 - currentValue * 100 / max_value ) / 100;
//		meterClip.h = meterSurface->h - meterClip.y;

		// Find intersect
		meterClip.x = meterClip.x > localClip->x ? meterClip.x : localClip->x;
		meterClip.y = meterClip.y > localClip->y ? meterClip.y : localClip->y;
		meterClip.w = meterSurface->w < localClip->x + localClip->w ?
			meterSurface->w - meterClip.x : localClip->x + localClip->w - meterClip.x;
		meterClip.h = meterSurface->h < localClip->y + localClip->h ?
			meterSurface->h - meterClip.y : localClip->y + localClip->h - meterClip.y;
		
		// Calc dest rect
		meterGlobalClip.x = meterClip.x + partRect.x;
		meterGlobalClip.y = meterClip.y + partRect.y;
		SDL_BlitSurface( meterSurface, &meterClip, Screen::GetScreen(), &meterGlobalClip );
	}
	if( coverSurface )
		SDL_BlitSurface( coverSurface, localClip, Screen::GetScreen(), globalClip );
}
