/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Table.h"
#include "../FilePath.h"
#include "../DataLoader.h"
#include "../Screen.h"
#include "../Log.h"
#include "GUICommandSet.h"

Table::Table()
	: AWidget(NULL)
{
	dataList = NULL;
	listSize = 0;
	cell_back = cell_hilite = cell_focus = NULL;
	scOffsetX = scOffsetY = 10;
	viewX = viewY = 0;
	cellWidth = cellHeight = 0;
	tableSurface = NULL;
	lastHiliteIndex = lastFocusIndex = 0;
}

Table::~Table()
{
	if( dataList )
		delete dataList;
	if( cell_back )
		SDL_FreeSurface( cell_back );
	if( cell_hilite )
		SDL_FreeSurface( cell_hilite );
	if( cell_focus )
		SDL_FreeSurface( cell_focus );
	if( tableSurface )
		SDL_FreeSurface( tableSurface );
}

void
Table::draw( SDL_Rect* localClip, SDL_Rect* globalClip)
{
	if( tableSurface )
	{
		SDL_Rect viewClip;
		viewClip.x = localClip->x + viewX;
		viewClip.y = localClip->y + viewY;
		viewClip.w = localClip->w;
		viewClip.h = localClip->h;
		SDL_BlitSurface( tableSurface, &viewClip, Screen::GetScreen(), globalClip );
	}
}

BOOL
Table::execCommand( GenericCommand* com )
{
	BOOL done = false;
	switch( com->type )
	{
		case ARIANNE_COM_SCROLL_UP:
			scrollUp();
			done = true;
			break;
		case ARIANNE_COM_SCROLL_DOWN:
			scrollDown();
			done = true;
			break;
		case ARIANNE_COM_SCROLL_RIGHT:
			scrollRight();
			done = true;
			break;
		case ARIANNE_COM_SCROLL_LEFT:
			scrollLeft();
			done = true;
			break;
	}
	return done;
}

BOOL
Table::handleMouseDown( SDL_Event* e )
{
	if( ptInPart( e->button.x, e->button.y ))
		return true;
	else
		return false;
}

BOOL
Table::handleMouseMove( SDL_Event* e )
{
	if( ptInPart( e->motion.x, e->motion.y ) )
	{
		Uint32 index = 
			( viewY + ( e->motion.y - partRect.y )) / cellHeight * numCols + 
				( viewX + ( e->motion.x - partRect.x )) / cellWidth;
		if( lastHiliteIndex && lastHiliteIndex - 1 != index )
			hiliteCell( lastHiliteIndex - 1, false );
		hiliteCell( index, true );
		return true;
	}
	else
	{
		if( lastHiliteIndex )
			hiliteCell( lastHiliteIndex - 1, false );
		return false;
	}
}

BOOL
Table::handleMouseUp( SDL_Event* e )
{
	if( ptInPart( e->button.x, e->button.y ))
	{
		Uint32 index = 
			( viewY + ( e->button.y - partRect.y )) / cellHeight * numCols + 
				( viewX + ( e->button.x - partRect.x )) / cellWidth;
		if( lastFocusIndex && lastFocusIndex - 1 != index )
			focusCell( lastFocusIndex - 1, false );
		
		focusCell( index, true );
		return true;
	}
	else
	{
		return false;
	}
}

// Access to cell data
// Add cell to the first empty cell.
// Not very usefull....
BOOL
Table::addCell( TableCell* cell )
{
	if( ! dataList )
		//dataList = new AList<TableCell, numCols * numRows - 1>;
		dataList = makeNewList( numCols * numRows );
	if( dataList )
	{
		for( Uint32 i = 0; i < listSize; i++)
		{
			if( dataList[i] == NULL )
			{
				dataList[i] = cell;
				updateTable( true );
				return true;
			}
		}
	}
	
	return false;
	
}

BOOL
Table::addCellAt( TableCell* cell, Uint8 col, Uint8 row )
{
	if( ! dataList )
		//dataList = new AList< TableCell, numCols * numRows - 1>;
		dataList = makeNewList( numCols * numRows );
	
	if( dataList )
	{
		Uint32 index = getCellIndexAt( col, row );
		if( index < listSize )
		{
			dataList[index] = cell;
			updateTable(true);
			return true;
		}
	}
	
	return false;
}

void
Table::getCell( TableCell* outCell, Uint8 index )
{
	outCell = NULL;
	if( dataList && index < listSize)
	{
		outCell = (TableCell*)dataList[index];
	}
}

void
Table::getCell( TableCell* outCell, Uint8 col, Uint8 row )
{
	getCell( outCell, ( row - 1 ) * numCols + col - 1 );
}

void
Table::getCellAt( TableCell* outCell, int x, int y )
{
	Uint8 col = ( x + viewX ) / cellWidth + 1;
	Uint8 row = ( y + viewY ) / cellHeight + 1;
	getCell( outCell, col, row );
}

// Table size initialization
BOOL
Table::setTableSize( Uint8 num_cols, Uint8 num_rows, BOOL update)
{
	if( num_cols <= 0 || num_rows <= 0 )
		return false;
	numCols = num_cols;
	numRows = num_rows;
	
	if( dataList )
	{
//		AList<TableCell>* tmp = dataList;
//		dataList = new AList< TableCell, num_cols * num_rows - 1>;
		TableCell** tmp = dataList;
		Uint32 oldSize = listSize;
		dataList = makeNewList( num_cols * num_rows );
		for( Uint32 i = 0; i < ( listSize < oldSize ? listSize : oldSize ); i++ )
		{
			dataList[i] = tmp[i];
		}
			
		delete tmp;
	}
	else
		dataList = makeNewList( num_cols * num_rows );
	
	if( tableSurface )
		SDL_FreeSurface( tableSurface );
	
	tableSurface = SDL_CreateRGBSurface( SDL_SWSURFACE, num_cols * cellWidth, num_rows * cellHeight,
		DataLoader::GetDepth(), 0, 0, 0, 0 );
	
	if( tableSurface )
		updateTable( update );

    return 0;	
}

// Cell gfx stuff
	// normal background
void
Table::setCellSurface( AString filepath )
{
	if( cell_back )
		SDL_FreeSurface( cell_back );
	FilePath path( filepath );
	path.getValidFilePath( filepath );

    SDL_Surface *tmp= SDL_LoadBMP( filepath );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)filepath);
      }

    cell_back = SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);

	if( cell_back )
	{
		cellWidth = cell_back->w;
		cellHeight = cell_back->h;
		scOffsetX = cellWidth / 2;
		scOffsetY = cellHeight / 2;
	}
}

void
Table::setCellSurface( SDL_Surface* sur )
{
	if( cell_back )
		SDL_FreeSurface( cell_back );
	cell_back = sur;
	if( cell_back )
	{
		cellWidth = cell_back->w;
		cellHeight = cell_back->h;
	}
}

SDL_Surface*
Table::getCellSurface()
{
	return cell_back;
}

	// hilited background
void
Table::setCellHiliteSurface( AString filepath )
{
	if( cell_hilite )
		SDL_FreeSurface( cell_hilite );
	FilePath path( filepath );
	path.getValidFilePath( filepath );
    SDL_Surface *tmp= SDL_LoadBMP( filepath );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)filepath);
      }
    cell_hilite = SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
}

void
Table::setCellHiliteSurface( SDL_Surface* sur )
{
	if( cell_hilite )
		SDL_FreeSurface( cell_hilite );
	cell_hilite = sur;
}

SDL_Surface*
Table::getCellHiliteSurface()
{
	return cell_hilite;
}

	// focused background
void
Table::setCellFocusSurface( AString filepath )
{
	if( cell_focus )
		SDL_FreeSurface( cell_focus );
	FilePath path( filepath );
	path.getValidFilePath( filepath );
    SDL_Surface *tmp= SDL_LoadBMP( filepath );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)filepath);
      }
    cell_focus = SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
}

void
Table::setCellFocusSurface( SDL_Surface* sur )
{
	if( cell_focus )
		SDL_FreeSurface( cell_focus );
	cell_focus = sur;
}

SDL_Surface*
Table::getCellFocusSurface()
{
	return cell_focus;
}

void
Table::setCellSize( Uint8 w, Uint8 h )
{
	cellWidth = w;
	cellHeight = h;
}

void
Table::setCellWidth( Uint8 w )
{
	cellWidth = w;
}

void
Table::setCellHeight( Uint8 h )
{
	cellHeight = h;
}

Uint8
Table::getCellWidth()
{
	return cellWidth;
}

Uint8
Table::getCellHeight()
{
	return cellHeight;
}

// Scrolling stuff
void
Table::setScrollOffsetX( Uint8 offset )
{
	scOffsetX = offset;
}

void
Table::setScrollOffsetY( Uint8 offset )
{
	scOffsetY = offset;
}

void
Table::setScrollOffset( Uint8 offset )
{
	scOffsetX = scOffsetY = offset;
}

Uint8
Table::getScrollOffsetX()
{
	return scOffsetX;
}

Uint8
Table::getScrollOffsetY()
{
	return scOffsetY;
}

	// handy func. Scroll num_unit * offset pixels
void
Table::scroll( int x_units, int y_units )
{
}

void
Table::scrollX( int num_unit )
{
}

void
Table::scrollY( int num_unit )
{
}

	// directly reset view osition
void
Table::setViewX( Uint8 x )
{
	viewX = x;
	requestUpdate( &partRect );
}

void
Table::setViewY( Uint8 y )
{
	viewY = y;
	requestUpdate( &partRect );
}

void
Table::scrollUp()
{
	if( viewY >= scOffsetY )
		viewY -= scOffsetY;
	if( viewY < 0 )
		viewY = 0;
}

void
Table::scrollDown()
{
	viewY += scOffsetY;
	if( viewY + partRect.h > tableSurface->h )
		viewY = tableSurface->h - partRect.h;
}

void
Table::scrollRight()
{
	viewX += scOffsetX;
	if( viewX + partRect.w > tableSurface->w )
		viewX = tableSurface->w - partRect.w;
}

void
Table::scrollLeft()
{
	if( viewX >= scOffsetX )
		viewX -= scOffsetX;
	if( viewX < 0 )
		viewX = 0;
}

Uint8
Table::getViewX()
{
	return viewX;
}

Uint8
Table::getViewY()
{
	return viewY;
}

void
Table::setViewLoc( Uint8 x, Uint8 y )
{
	viewX = x;
	viewY = y;
	requestUpdate( &partRect );
}

/*// Font stuff... We don't need them, actually.

#ifdef USE_SDL_TTF

void
Table::setFont( TTF_Font* font )
{
}

TTF_Font*
Table::getFont()
{
}

#else
BitmapFont*
Table::getFont()
{
}
#endif // USE_SDL_TTF

void
Table::setFont( AString filepath )
{
}

*/

//
// Protected functions
//

void
Table::updateTable( BOOL update )
{
	for( Uint32 i = 0; i <= getCellIndexAt( numCols, numRows ); i++ )
		updateCell( i );
	if( update )
		requestUpdate( &partRect );
}

// index should be 0 based.
// This func is to called internally.
// Use updateCell( col, row )
void
Table::updateCell( Uint32 index )
{
	if( dataList )
	{
		int x = ((index + 1) % numCols - 1 ) * cellWidth;
		if( x < 0 )
			x = ( numCols - 1 ) * cellWidth;
		int y = index / numCols * cellHeight;
		SDL_Rect cellRect;
		cellRect.x = x;
		cellRect.y = y;
		cellRect.w = cellWidth;
		cellRect.h = cellHeight;
		
		TableCell* cell = dataList[index];
		if( cell )
		{
			if( cell->focus && cell_focus )
				SDL_BlitSurface( cell_focus, NULL, tableSurface, &cellRect );
			else if( cell->hilite && cell_hilite )
				SDL_BlitSurface( cell_hilite, NULL, tableSurface, &cellRect );
			else if( cell_back )
				SDL_BlitSurface( cell_back, NULL, tableSurface, &cellRect );
			if( cell->getSurface())
			{
				SDL_Rect gfxr;
				gfxr.x = cellRect.x + ( cellRect.w - cell->getSurface()->w ) / 2;
				gfxr.y = cellRect.y + ( cellRect.h - cell->getSurface()->h ) / 2;
				gfxr.w = cell->getSurface()->w;
				gfxr.h = cell->getSurface()->h;
				SDL_BlitSurface( cell->getSurface(), NULL, tableSurface, &gfxr );
			}
		}
		else
			SDL_BlitSurface( cell_back, NULL, tableSurface, &cellRect );
	}
}

// Both col and row are 1 based number.
void
Table::updateCell( Uint8 col, Uint8 row )
{
	updateCell(( row - 1 ) * numCols + ( col - 1 ));
}
// request cell hilite / focus.
// Un-set other hilited / focused cell.
// ( should we use CommanderGroup ? hmmm....)
void
Table::hiliteCell( Uint32 index, BOOL hilite )
{
	if( hilite && dataList && index < listSize )
	{
		if( lastHiliteIndex )
		{
			TableCell* cell = (TableCell*)dataList[lastHiliteIndex ];
			if( cell )
			{
				cell->hilite = false;
				updateCell( lastHiliteIndex - 1 );
			}
			lastHiliteIndex = 0;
		}
		TableCell* cell = (TableCell*)dataList[index];
		if( cell )
		{
			cell->hilite = true;
			updateCell(index);
			lastHiliteIndex = index + 1;
		}
	}
	else if( dataList && index < listSize )
	{
		TableCell* cell = (TableCell*)dataList[index];
		if( cell )
		{
			cell->hilite = false;
			updateCell( index );
		}
	}
	SDL_Rect cellR;
	cellR.x = index % numCols * cellWidth;
	cellR.y = index / numCols * cellHeight;
	cellR.w = cellWidth;
	cellR.h = cellHeight;
	requestUpdate( &cellR );
}

void
Table::focusCell( Uint32 index, BOOL focus )
{
	if( focus && dataList && index < listSize )
	{
		if( lastFocusIndex )
		{
			TableCell* cell = ( TableCell*)dataList[index];
			if( cell )
			{
				cell->focus = false;
				updateCell( lastFocusIndex - 1 );
			}
			lastFocusIndex = 0;
		}
		TableCell* cell = (TableCell*)dataList[index];
		if( cell )
		{
			cell->focus = true;
			updateCell( index );
			lastFocusIndex = index + 1;
		}
	}
	else if( dataList && index < listSize )
	{
		TableCell* cell = (TableCell*)dataList[index];
		if( cell )
		{
			cell->focus = false;
			updateCell( index );
		}
	}
	SDL_Rect cellR;
	cellR.x = index % numCols * cellWidth;
	cellR.y = index / numCols * cellHeight;
	cellR.w = cellWidth;
	cellR.h = cellHeight;
	requestUpdate( &cellR );
}
//
// Private function
//

Uint32
Table::getCellIndexAt( Uint8 col, Uint8 row )
{
	return ( row - 1 ) * numCols + ( col - 1 );
}

TableCell**
Table::makeNewList( Uint32 size )
{
	TableCell** newList = new TableCell*[size];
	for( Uint32 i = 0; i < size; i++)
		newList[i] = NULL;
	listSize = size;
	return newList;
}
