#ifndef TABLE_H
#define TABLE_H

#include "../general.h"
#include "AWidget.h"
#include "TableCell.h"
#include "../List.h"

#ifdef USE_SDL_TTF
#include "SDL_ttf.h"
#include "freetype.h"
#else
#include "../../client/Font.h"
#endif // USE_SDL_TTF


class Table : public AWidget
{
	public:
	
		Table();
		~Table();
		
		// Inherits from GUIPart
		// This function only updates view area ( copy table onto screen ).
		// Actual 'table' update is done internally when any change occurs.
		virtual void draw( SDL_Rect* localRect, SDL_Rect* globalRect );
		
		// Inherites from ACommander
		virtual BOOL handleMouseDown( SDL_Event* e );
		virtual BOOL handleMouseUp( SDL_Event* e );
		virtual BOOL handleMouseMove( SDL_Event* e );
		
		// Access to cell data
		virtual BOOL	addCell( TableCell* cell );
		virtual BOOL	addCellAt( TableCell* cell, Uint8 col, Uint8 row );
		
		virtual void	getCell( TableCell* outCell, Uint8 index );
		virtual void	getCell( TableCell* outCell, Uint8 col, Uint8 row );
		virtual void	getCellAt( TableCell* outCell, int x, int y );
		
		// command handling
		virtual BOOL	execCommand( GenericCommand* com );
		
		// Table size initialization
		BOOL			setTableSize( Uint8 num_cols, Uint8 num_rows, BOOL update = true );
		
		// Cell gfx stuff
			// normal background
		void			setCellSurface( AString filepath );
		void			setCellSurface( SDL_Surface* surface );
		SDL_Surface*	getCellSurface();
			// hilited background
		void			setCellHiliteSurface( AString filepath );
		void			setCellHiliteSurface( SDL_Surface* surface );
		SDL_Surface*	getCellHiliteSurface();
			// focused background
		void			setCellFocusSurface( AString filepath );
		void			setCellFocusSurface( SDL_Surface* surface );
		SDL_Surface*	getCellFocusSurface();
			// Explicitely specify cell size.
			// Automatically done when cell surface is set.
		void			setCellSize( Uint8 w, Uint8 h );
		void			setCellWidth( Uint8 w );
		void			setCellHeight( Uint8 h );
		Uint8			getCellWidth();
		Uint8			getCellHeight();
		
		// Scrolling stuff
		void			setScrollOffsetX( Uint8 offset );
		void			setScrollOffsetY( Uint8 offset );
		void			setScrollOffset( Uint8 offset );
		Uint8			getScrollOffsetX();
		Uint8			getScrollOffsetY();
			// handy func. Scroll num_unit * offset pixels
		void			scroll( int x_units, int y_units );
		void			scrollX( int num_unit );
		void			scrollY( int num_unit );
			// they might be useful for call back routine
		void			scrollUp();
		void			scrollDown();
		void			scrollRight();
		void			scrollLeft();
			// directly reset view position.
		void			setViewX( Uint8 x );
		void			setViewY( Uint8 y );
		Uint8			getViewX();
		Uint8			getViewY();
		void			setViewLoc( Uint8 x, Uint8 y );

/*		
		#ifdef USE_SDL_TTF
		void			setFont( TTF_Font* font );
		TTF_Font*		getFont();
		#else
		BitmapFont*		getFont();
		#endif // USE_SDL_TTF
		void			setFont( AString filepath );
*/
	protected:
	
		int				viewX, viewY;
		Uint8			numCols, numRows, cellWidth, cellHeight;
		Uint8			scOffsetX, scOffsetY;		// Offsets for scrolling

		//AList<TableCell>*		dataList;					// actual cell data.
													// I don't think it should be public...
		TableCell**		dataList;
		Uint32			listSize;
		Uint32			lastHiliteIndex;			// I say it's confusing... I know,
		Uint32			lastFocusIndex;				// but these 2 are 1 based index... sorry.
		
		// Update of internal table.
		// It's separate from 'draw' func.
		// Table is re-painted when any of its content has changed ( new one / status changed...)
		// 'draw' func just copies 'view' area of 'table' to 'screen'.
		
			// whole table update
		void			updateTable( BOOL update );
			// a cell update
		virtual void	updateCell( Uint32 index );
		virtual void	updateCell( Uint8 col, Uint8 row );
		
		void			hiliteCell( Uint32 index, BOOL hilite );
		void			focusCell( Uint32 index, BOOL focus );
		
	private:

	#ifdef USE_SDL_TTF
		TTF_Font*		font;
	#else
		BitmapFont*		font;
	#endif // USE_SDL_TTF
		SDL_Surface*	cell_back;		// Background gfx for a cell
		SDL_Surface*	cell_hilite;	// Cell hilited gfx
		SDL_Surface*	cell_focus;		// Cell focused gfx
		SDL_Surface*	tableSurface;
		
		Uint32			getCellIndexAt( Uint8 col, Uint8 row );
		TableCell**			makeNewList( Uint32 size );
};

#endif //TABLE_H