/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "TableCell.h"
#include "../FilePath.h"
#include "../Log.h"

TableCell::TableCell( void* data )
{
	cellData = data;
	name = new char[256];
	for( int i = 0; i < 256; i++ )
		name[i] = 0;
	
	surface = NULL;
	
	hilite = focus = false;
}

TableCell::~TableCell()
{
	if( name )
		delete name;
	
	if( surface )
		SDL_FreeSurface( surface );
}

void
TableCell::setName( char* newName )
{
	if( strlen( newName ) > 256 )
		newName[255] = 0;
	
	strcpy( name, newName );
}

// Be sure to allocate enough memory for outName
void
TableCell::getName( char* outName )
{
	strcpy( outName, name );
}

void
TableCell::setSurface( AString filepath )
{
	if( surface )
		SDL_FreeSurface( surface );
		
	FilePath path( filepath );
	path.getValidFilePath( filepath );

	SDL_Surface *tmp = SDL_LoadBMP( filepath );
    if(tmp==NULL)
      {
      Log::Error("Can't open file: %s",(char const*)filepath);
      }
	if( tmp )
		SDL_SetColorKey( tmp, SDL_SRCCOLORKEY, SDL_MapRGB( tmp->format, 0, 0, 0 ));
    surface=SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
}

void
TableCell::setSurface( SDL_Surface* gfx )
{
	if( surface )
		SDL_FreeSurface( surface );
	
	surface = gfx;
	if( surface )
		SDL_SetColorKey( surface, SDL_SRCCOLORKEY, SDL_MapRGB( surface->format, 0, 0, 0 ));
}

SDL_Surface*
TableCell::getSurface()
{
	return surface;
}
