/***************************************************************************
    copyright            : (C) 2000 by Masahiro Minami
    email                : elsur@aaa.letter.co.jp
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "TextBox.h"
#include "../FilePath.h"
#include "../DataLoader.h"
#include "../Screen.h"
#include "GUICommandSet.h"

#define	TEMP_FILE_BUFFER	256				// this is for reading temporary file. Should be changed ?
#define LINE_BREAK_CHAR		'\n'			// because this is for temporary file only, it doesn't matter what
											// char is used for line break char.

TextBox::TextBox( Uint16 w, Uint16 h )
	:AWidget( NULL, NULL )
{
	partRect.x = partRect.y = 0;
	partRect.w = w;
	partRect.h = h;
	
	setDefaultAcceptMouse( false );
	setAcceptMouse( false );
	setDefaultAcceptKey( false );
	setAcceptKey( false );
	font = NULL;
	content = NULL;
	textList = NULL;
	
	topLine = bottomLine = textListSize = textListCur = 0;
	
	logfile = NULL;		// Log file
	tempfile = NULL;	// and temporary file
	tempCurStart = tempCurEnd = 0;
}

TextBox::~TextBox()
{
	if( font )
	#ifdef USE_SDL_TTF
		TTF_CloseFont( font );
	#else
		delete font;
	#endif // USE_SDL_TTF
	if( content )
		SDL_FreeSurface( content );
	if( textList )
		delete textList;
	
	if( logfile )				// close log file and temporary file, if exist
		fclose( logfile );
	if( tempfile )
		fclose( tempfile );
}

// drawing

void
TextBox::draw( SDL_Rect* localClip, SDL_Rect* globalClip )
{
	if( content )
	{
		SDL_BlitSurface( content, localClip, Screen::GetScreen(), globalClip );
	}
}

void
TextBox::addLine( char* newText )
{
	if( textList && font && newText )
	{
		int i, j, w, len;
		char* tmp = new char[256];			// o.k., I know it's not pretty...
		len = strlen( newText );
		j = 0;
		
		if( logfile )						// chat log
		{
			fprintf( logfile, newText );
			fprintf( logfile, "\n" );
		}
		
		for( i = 0; i < len /*&& textListCur < textListSize*/; i++)
		{
			tmp[j] = newText[i];
			tmp[j+1] = 0;
			w = font->GetWidth( tmp );
			
			if( w > partRect.w )			// reached the right edge of the view
			{
				tmp[j] = 0;					// ...break the line
				textList[ textListCur ] = tmp;
				
				// We should be careful not to exceed the text list size
				if( textListCur + 1 < textListSize )
				{
						textListCur++;
					if( topLine == 0 )			// update pointers
						topLine = 1;
					if( bottomLine == 0 )
						bottomLine = 1;
					else if( ( bottomLine + 1 - topLine ) * font->GetHeight(NULL) > partRect.h - font->GetHeight(NULL) )
					{
						topLine ++;
						bottomLine ++;
					}
					else
						bottomLine ++;
				}
				else
					scrollDownTextList();
				
				j = 0;
				i --;						// put back the char
				tmp = new char[ 256 ];		// ...and the new one
				tmp[0] = 0;					// make sure clear text.
			}
			else
				j++;
		}
		
		if( strlen( tmp ))
		{
			textList[ textListCur ] = tmp;
			
			// We should be careful not to exceed the text list size
			if( textListCur + 1 < textListSize )
			{
					textListCur++;
				if( topLine == 0 )			// update pointers
					topLine = 1;
				if( bottomLine == 0 )
					bottomLine = 1;
				else if( ( bottomLine + 1 - topLine ) * font->GetHeight(NULL) > partRect.h - font->GetHeight(NULL) )
				{
					topLine ++;
					bottomLine ++;
				}
				else
					bottomLine ++;
			}
			else
				scrollDownTextList();
			
		}
		
	}
}

BOOL
TextBox::execCommand( GenericCommand* com )
{
	BOOL done = false;
	switch( com->type )
	{
		case ARIANNE_COM_TEXT_ADD:
			addLine( (char*)com->GetCommand());
			updateContent();
			done = true;
			break;
		case ARIANNE_COM_SCROLL_UP:
			scrollUp();
			done = true;
			break;
		case ARIANNE_COM_SCROLL_DOWN:
			scrollDown();
			done = true;
			break;
	}
	return done;
}

void
TextBox::scroll( int num_char, int num_line )
{
}

void
TextBox::scrollX( int num_char )
{
}

void
TextBox::scrollY( int num_line )
{
}

void
TextBox::scrollUp()
{
	if( content && font && textList && topLine > 1)
	{
		// hmmm performance cost ??
		SDL_Surface* tmp = SDL_CreateRGBSurface(
			SDL_SWSURFACE,
			partRect.w, partRect.h,
			content->format->BitsPerPixel, 0, 0, 0, 0 );
		
		SDL_Rect clip, dest;
		clip.x = dest.x = 0;
		clip.w = dest.w = partRect.w;
		clip.h = dest.h = partRect.h - font->GetHeight(NULL);
		clip.y = 0;
		dest.y = font->GetHeight(NULL);
		
		SDL_BlitSurface( content, &clip, tmp, &dest );
		
		topLine --;
		bottomLine --;
		
		font->Draw( tmp, 0, 0, textList[ topLine - 1 ], AFont::LEFT + AFont::TOP );
		
		SDL_FreeSurface( content );
		content = tmp;
		
		requestUpdate( &partRect );
	}
}

void
TextBox::scrollDown()
{
	if( content && font && textList && bottomLine < textListCur &&
		( bottomLine + 1 - topLine ) * font->GetHeight(NULL) > partRect.h - font->GetHeight(NULL))
	{
		// hmmm performance cost ??
		SDL_Surface* tmp = SDL_CreateRGBSurface(
			SDL_SWSURFACE,
			partRect.w, partRect.h,
			content->format->BitsPerPixel, 0, 0, 0, 0 );
		
		SDL_Rect clip, dest;
		clip.x = dest.x = 0;
		clip.w = dest.w = partRect.w;
		clip.h = dest.h = partRect.h - font->GetHeight(NULL);
		clip.y = font->GetHeight(NULL);
		dest.y = 0;
		
		SDL_BlitSurface( content, &clip, tmp, &dest );
		
		topLine ++;
		bottomLine ++;
		
		font->Draw( tmp, 0, ( bottomLine - topLine ) * font->GetHeight( NULL ), textList[ bottomLine - 1], AFont::LEFT + AFont::TOP );
		
		SDL_FreeSurface( content );
		content = tmp;
	}
}

void
TextBox::scrollRight()
{
}

void
TextBox::scrollLeft()
{
}

#ifdef USE_SDL_TTF
void
TextBox::setFont( TTF_Font* font )
{
}

TTF_Font*
TextBox::getFont()
{
	return font;
}

#else // USE_SDL_TTF

BitmapFont*
TextBox::getFont()
{
	return font;
}

#endif // USE_SDL_TTF

void
TextBox::setFont( AString filepath )
{
	FilePath path( filepath );
	path.getValidFilePath( filepath );
#ifdef USE_SDL_TTF
	if( font )
		TTF_CloseFont( font );
	font = TTF_OpenFont( filepath, 14 );
#else
	if( font )
		delete font;
	
	font = new BitmapFont( filepath );
#endif // USE_SDL_TTF
	
	if( textList )						// Not an inteligent way... hmmm (TODO)
		delete textList;
	if( content )
		SDL_FreeSurface( content );
	content = SDL_CreateRGBSurface(
		SDL_SWSURFACE,
		partRect.w, partRect.h,
		DataLoader::GetDepth(), 0, 0, 0, 0 );
	
	textList = new char*[1000];			// Change this ! it's for debug.
	textListSize = 1000;
	textListCur = 0;
}

void
TextBox::setSize( int w, int h )
{
	partRect.w = w;
	partRect.h = h;
	if( content )
		SDL_FreeSurface( content );
	content = SDL_CreateRGBSurface(
		SDL_SWSURFACE,
		partRect.w, partRect.h,
		DataLoader::GetDepth(), 0, 0, 0, 0 );
}

void
TextBox::setLogFile( AString filename )
{
/*	TempLock lock(m_mutex);
	assert(filename!=NULL);*/
	
	FilePath path( filename );
	path.getValidFilePath( filename );
	
	logfile = fopen( filename, "wb" );
	
}

void
TextBox::setTempFile( AString filename )
{
/*	TempLock lock(m_mutex);
	assert(filename!=NULL);*/
	
	FilePath path( filename );
	path.getValidFilePath( filename );
	
	tempfile = fopen( filename, "wb");
	if( tempfile )
	{
		tempCurStart = tempCurEnd = 0;
	}
}

// protected functions
void
TextBox::updateContent( BOOL update )
{
	if( textList && content && font )
	{
		// hmmm.... think a better way.
		SDL_Surface* tmp = SDL_CreateRGBSurface(
			SDL_SWSURFACE,
			partRect.w, partRect.h,
			content->format->BitsPerPixel, 0, 0, 0, 0 );
		
		for( Uint32 i = topLine; i <= bottomLine; i++ )
		{
			if( textList[ i - 1 ] )
				font->Draw( tmp, 0, (i - topLine) * font->GetHeight(NULL),
					textList[i - 1], AFont::TOP + AFont::LEFT );
		}
		SDL_FreeSurface( content );
		content = tmp;
	}
	if( update )
		requestUpdate( &partRect );
}

// private functions

	// These 2 functions are used to scroll internal text list. They are still in progress
void
TextBox::scrollUpTextList()
{
	// delete the pointer;)
	if( textList[ textListSize - 1 ] )
		delete textList[ textListSize - 1 ];
	
	for( int i = textListSize - 1; i > 0 ; i -- )
		textList[i] = textList[i - 1];
	
	textList[0] = NULL;		// clear the top most line.... MAKE SURE FILL IT !
}

void
TextBox::scrollDownTextList()
{
	// delete the pointer;)
	if( textList[ 0 ] )
		delete textList[ 0 ];
	
	for( int i = 0; i < textListSize; i ++ )
		textList[i] = textList[ i + 1 ];
	
	textList[ textListSize - 1 ] = NULL; // clear the down most line .... MAKE SURE FILL IT !
}

/*
int
TextBox::readPrevLine()
{
	// there's no more lines before / or no temporary file
	if( tempCurStart == 0 || ! tempfile )
		return 0;
	// create buffer [TODO] what should be done with this buffer size ?
	char* buf = new char[ TEMP_FILE_BUFFER ];
	if( ! buf )
		return 0;
	
	// adjust the start point to begin read
	tempCurStart -= 1 + TEMP_FILE_BUFFER;
	if( tempCurStart < 0 )
		tempCurStart = 0;
	fseek( tempfile, tempCurStart, 0 );
	// read into buffer
	int len = fread( buf, TEMP_FILE_BUFFER, 1, tempfile );
	int i = len - 1;
	while( i >= 0 )
	{
		if( buf[i] == LINE_BREAK_CHAR )
			break;
		else
			i--;
	}
	
	// couldn't find the line break ---- this should not happen !
	if( i<0 )
		return 0;
	// adjust temporary start point;
	tempCurStart += i + 1;
	
}

int
TextBox::readNextLine()
{
}*/

