#ifndef TEXTBOX_H
#define TEXTBOX_H

#include "../general.h"
#include "AWidget.h"
#include "../List.h"

#ifdef USE_SDL_TTF
#include "SDL_ttf.h"
#include "freetype.h"
#else
#include "../../client/Font.h"
#endif // USE_SDL_TTF

class TextBox : public AWidget
{
	public:
	
		TextBox( Uint16 w, Uint16 h );
		~TextBox();
		
		// Inherits from GUIPart
		// This function only updates view area ( copy table onto screen ).
		// Actual 'table' update is done internally when any change occurs.
		virtual void draw( SDL_Rect* localRect, SDL_Rect* globalRect );
		
		// add line of text
		virtual void	addLine( char* newText );
		
		// command execution
		virtual BOOL	execCommand( GenericCommand* com );
		
		// scroll stuff
		void			scroll( int num_char, int num_line );
		void			scrollX( int num_char );
		void			scrollY( int num_line );
			// they might be useful for call back routine
		void			scrollUp();
		void			scrollDown();
		void			scrollRight();
		void			scrollLeft();

		// font setting
		#ifdef USE_SDL_TTF
		void			setFont( TTF_Font* font );
		TTF_Font*		getFont();
		#else
		BitmapFont*		getFont();
		#endif // USE_SDL_TTF
		void			setFont( AString filepath );
		int				getFontHeight();
		int				getFontWidth( char c );
		
		void			setSize( int w, int h );
		
		void			setLogFile( AString filename );
		void			setTempFile( AString filename );
		
	protected:
	
		Uint32			topLine, bottomLine;
		char**			textList;
		Uint32			textListSize, textListCur;
		
		// this is where text is drawn.
		SDL_Surface*	content;

	#ifdef USE_SDL_TTF
		TTF_Font*		font;
	#else
		BitmapFont*		font;
	#endif // USE_SDL_TTF
		
		// Update of internal content.
		// It's separate from 'draw' func.
		// Content is re-painted when any of its content has changed ( new one / status changed...)
		// 'draw' func just copies 'view' area of 'content' to 'screen'.
		
			// whole table update
		void			updateContent( BOOL update = true );
		
	private:
		// Log file and temporary file
		FILE	*logfile;
		FILE	*tempfile;
		// Pointers to temp file---corresponds to view start and view end.
		long	tempCurStart, tempCurEnd;
		
//		int		readPrevLine();				// They are still in progress.
//		int		readNextLine();				// You should not try;)
		void	scrollUpTextList();
		void	scrollDownTextList();
};

#endif //TEXTBOX_H