/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This an abstraction of a common Message.

    All the messages are translated to this generic messages.
***************************************************************************/
#include "GenericMessage.h"
#include "Log.h"

/** This is the class constructor. **/
GenericMessage::GenericMessage():
  ID(0),
  Type(0),
  SubType(0),
  Length(0)  
  {
  Data=new AVector();
  }


/** This is the class destructor. **/
GenericMessage::~GenericMessage()
  {
  if(Data) 
    { 
    delete Data; 
    Data=0;
    }
  }

/** This reads the type from the Net. */
void GenericMessage::GetType(Socket *socket)
  {
  if(socket->isError()==TRUE)
    {
    Type=0;
    return;
    }

  socket->Recv(&this->Type,sizeof(Uint8),1);
  }

/** This reads the subtype from the Net. */
void GenericMessage::GetSubType(Socket *socket)
  {
  assert(socket!=NULL);
  
  if(socket->isError()==TRUE)
    {
    SubType=0;
    return;
    }

  socket->Recv(&this->SubType,sizeof(Uint8),1);
  }

void GenericMessage::GetFlags(Socket *socket)
  {
  assert(socket!=NULL);

  if(socket->isError()==TRUE)
    {
    Flags=RAW_DATA;
    return;
    }

  socket->Recv(&this->Flags,sizeof(Uint8),1);
  }

/** This reads the length from the Net. */
void GenericMessage::GetLength(Socket *socket)
  {
  assert(socket!=NULL);

  if(socket->isError()==TRUE)
    {
    Length=0;
    return;
    }

  Uint8 Word[4];

  socket->Recv(Word,sizeof(Uint8),4);
  this->Length=SDLNet_Read32(Word);

  /* We modify it to correct endian problems. */
  }

/** This reads the data from the Net. */
void GenericMessage::GetData(Socket *socket)
  {
  assert(socket!=NULL);

  if(socket->isError()==TRUE)
    {
    if(Data)
      {
      delete Data;
      }

    Data=0;
    return;
    }

  if(Data)
    {
    delete Data; 
    }

  Data = new AVector;
  Data->SetVectorSize(Length);

  socket->Recv(Data->Get(),sizeof(Uint8),Length);
  }

/** This writes the type to the Net. */
void GenericMessage::PutType(Socket *socket) const
  {
  assert(socket!=NULL);

  socket->Send(&this->Type,sizeof(Uint8),1);
  }

/** This writes the subtype to the Net. */
void GenericMessage::PutSubType(Socket *socket) const
  {
  assert(socket!=NULL);

  socket->Send(&this->SubType,sizeof(Uint8),1);
  }

void GenericMessage::PutFlags(Socket *socket) const
  {
  assert(socket!=NULL);

  socket->Send(&this->Flags,sizeof(Uint8),1);
  }

/** This writes the lenght to the Net. */
void GenericMessage::PutLength(Socket *socket) const
  {
  assert(socket!=NULL);

  /* We modify it to correct endian problems. */
  Uint8 Word[4];
  SDLNet_Write32(Length,Word);

  socket->Send(Word,sizeof(Uint8),4);
  }

/** This writes the data to the Net. */
void GenericMessage::PutData(Socket *socket) const
  {
  assert(socket!=NULL);

  socket->Send(this->Data->Get(),sizeof(Uint8),Length);
  }

Uint8 shouldCompress(Uint32 size)
  {
  if(size<128)
    {
    return 0;
    }

  return 1;
  }


Socket& operator>>(Socket& socket, GenericMessage& msg)
  {
  msg.GetType(&socket);
  msg.GetSubType(&socket);
  msg.GetFlags(&socket);
  msg.GetLength(&socket);
  msg.GetData(&socket);
  
  msg.TimeToken=SDL_GetTicks();

  if(socket.isError()==FALSE)
    {
    if((msg.Flags&BZIP2_COMPRESSED_DATA)==BZIP2_COMPRESSED_DATA)
      {
      AVector *Descomp=msg.Data->Decompress();
      delete msg.Data;
      msg.Data=Descomp;
      msg.Length=msg.Data->GetSize();
      }
    }

  return socket;
  }

Socket& operator<<(Socket&socket, GenericMessage const& msg)
  {
  AVector *Comp=msg.Data->Compress(shouldCompress(msg.Data->GetVectorSize()));

  Uint8 *tmp=new Uint8[1+1+1+4+Comp->GetSize()];

  tmp[0]=msg.Type;
  tmp[1]=msg.SubType;
  if(shouldCompress(msg.Data->GetVectorSize())==TRUE)
    {
    tmp[2]=BZIP2_COMPRESSED_DATA;
    }
  else 
    {
    tmp[2]=RAW_DATA;
    }

  SDLNet_Write32(Comp->GetSize(),&tmp[3]);
  memcpy(&tmp[7],Comp->Get(),Comp->GetSize());

  socket.Send(tmp,sizeof(Uint8),1+1+1+4+Comp->GetSize());

  delete tmp;
  delete Comp;

  return socket;
  }

void GenericMessage::SetID(Uint32 ID)
  {
  this->ID=ID;
  }

Uint32 GenericMessage::GetID()
  {
  return ID;
  }

Uint32 GenericMessage::GetTimeToken()
  {
  return TimeToken;
  }

/** Dumps the GenericMessage to any kind of ostream, useful for debugging purposes */
void GenericMessage::Dump(std::ostream& out) const
  {
  out << "---BEGIN MESSAGE---\n";
  out << "T:" << int(Type) << " ST:" << int(SubType) << " L:" << int(Length) << "\n" << out.flush();
  for(Uint16 i=0; i!=Length; ++i)
    out << ((Data->Get())[i]) << " ";
  out << "\n";
  out << "---END MESSAGE---\n";
  }
