/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This a Logger implementation.
    It has different levels for the messages from DEBUG to ERROR.
***************************************************************************/
#include "Log.h"
#include "templock.h"


FILE *Log::Handle=stderr;
Mutex Log::m_mutex;

void Log::SetLogFile(FILE *Handle)
  {
  TempLock lock(m_mutex);
  assert(Handle!=NULL);

  Log::Handle=Handle;
  }

void Log::SetLogFile(char *FileName)
  {
  TempLock lock(m_mutex);
  assert(FileName!=NULL);

  Handle=fopen(FileName,"wb");
  if(Handle==NULL)
    {
    Log::Handle=Handle;
    }
  }

void Log::Warning(const char *fmt, ...)
  {
  TempLock lock(m_mutex);
  assert(fmt!=NULL);

  va_list ap;

  fprintf(Handle, "WARNING: ");
  va_start(ap, fmt);
  vfprintf(Handle, fmt, ap);
  va_end(ap);
  fprintf(Handle, "\n");
  fflush(Handle);
  }

void Log::Debug(const char *fmt, ...)
  {
  TempLock lock(m_mutex);
  assert(fmt!=NULL);

  #ifdef DEBUG
    va_list ap;

    fprintf(Handle, "DEBUG: ");
    va_start(ap, fmt);
    vfprintf(Handle, fmt, ap);
    va_end(ap);
    fprintf(Handle, "\n");
    fflush(Handle);
  #endif
  }

void Log::Error(const char *fmt, ...)
  {
  TempLock lock(m_mutex);
  assert(fmt!=NULL);

  va_list ap;

  fprintf(Handle, "ERROR: ");
  va_start(ap, fmt);
  vfprintf(Handle, fmt, ap);
  va_end(ap);

  fprintf(Handle,"\n");
  fflush(Handle);

  exit(1);
  }
