/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
***************************************************************************/
#include "MapObject.h"
#include "FilePath.h"
#include "Log.h"



MapObjects::MapObjects()
  {
  MapPiece=new AList<ObjectPiece>();
  }

MapObjects::~MapObjects()
  {
  delete MapPiece;
  }

void MapObjects::CreateNewObjectPiece(Uint32 x_init,Uint32 y_init,Uint32 x_end,Uint32 y_end)
  {
  ObjectPiece *tmp=new ObjectPiece();

  tmp->x_init=x_init;
  tmp->y_init=y_init;
  tmp->x_end=x_end;
  tmp->y_end=y_end;

  MapPiece->Add(tmp);
  }


BOOL MapObjects::SavePiece(AString FileName,Uint32 x_init,Uint32 y_init,Uint32 x_end,Uint32 y_end)
  {
  FilePath path( FileName );
  path.getValidFilePath( FileName );

  for(Uint32 i=0;i<MapPiece->Size();i++)
    {
    if((*MapPiece)[i]->x_init>=x_init && (*MapPiece)[i]->x_end<=x_end)
      {
      if((*MapPiece)[i]->y_init>=y_init && (*MapPiece)[i]->y_end<=y_end)
        {
        AVector *Vector=new AVector();
        (*MapPiece)[i]->ToVector(Vector);

        Uint8 *Data=Vector->Get();
        AUint32 Size=Vector->GetVectorSize();

        FileName=FileName+".object";

        FILE *Handle=fopen((const char *)FileName,"wb");
        if(Handle==NULL)
          {
          Log::Error("ERROR: Unable to open file %s",(const char *)FileName);
          }

        AVector *tmpVector=new AVector;
        Size.ToVector(tmpVector);

        fwrite(tmpVector->Get(),1,tmpVector->GetVectorSize(),Handle);
        Uint32 writed=fwrite(Data,1,Size,Handle);

        assert(writed==Size);

        fclose(Handle);

        delete Vector;
        delete tmpVector;
        }
      }
    }

  return TRUE;
  }

BOOL MapObjects::LoadPiece(AString FileName,Uint32 x_init,Uint32 y_init,Uint32 x_end,Uint32 y_end)
  {
  FilePath path( FileName );
  path.getValidFilePath( FileName );

  FileName=FileName+".object";

  FILE *Handle=fopen((const char *)FileName,"rb");
  if(Handle==NULL)
    {
    Log::Error("ERROR: Unable to open file %s",(const char *)FileName);
    }

  AVector *tmpVector=new AVector();
  tmpVector->SetVectorSize(4);
  tmpVector->SetIndex(0);

  Uint8 buffer[4];
  fread(buffer,4,sizeof(Uint8),Handle);
  tmpVector->AddUint8(buffer,4);

  AUint32 tmpSize;
  tmpSize.FromVector(tmpVector);
  Uint32 Size=(Uint32)tmpSize;
  delete tmpVector;

  Uint8 *Data=new Uint8[Size];
  Uint32 readed=fread(Data,1,Size,Handle);

  assert(readed==Size);

  fclose(Handle);

  ObjectPiece *tmp=new ObjectPiece;

  AVector *Vector=new AVector();
  Vector->SetVectorSize(Size);
  Vector->AddUint8(Data,Size);
  Vector->SetIndex(0);

  tmp->FromVector(Vector);

  MapPiece->Add(tmp);

  delete Vector;
  return TRUE;
  }


void MapObjects::Clear()
  {
  for(Uint32 i=0;i<MapPiece->Size();i++)
    {
    (*MapPiece)[i]->Clear();
    }
  }

/** This method returns a List of all the objects that are in the piece contained 
    fully in the given box */
AList<Object> *MapObjects::GetListObjects(Uint32 x_init,Uint32 y_init,Uint32 x_end,Uint32 y_end)
  {
  AList<Object> *tmp=new AList<Object>;

  for(Uint32 i=0;i<MapPiece->Size();i++)
    {
    if((*MapPiece)[i]->x_init>=x_init && (*MapPiece)[i]->x_end<=x_end)
      {
      if((*MapPiece)[i]->y_init>=y_init && (*MapPiece)[i]->y_end<=y_end)
        {
        (*tmp)+(*MapPiece)[i]->Objects;
        }
      }
    }


  return tmp;
  }

/** This looks for an object in ALL the Map Pieces */
Object *MapObjects::GetObject(Uint32 uID)
  {
  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    Object *tmp=(*MapPiece)[j]->GetObject(uID);
    if(tmp!=NULL)
      {
      return tmp;
      }
    }
  
  return NULL;
  }

Object *MapObjects::GetObject(Object *object)
  {
  assert(object!=NULL);

  Object *tmp;
  ObjectPiece *ExpectedPiece;
  Uint32 j;

  /* We look for it in its piece */
  for(j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->isInside(object)==TRUE)
      {
      tmp=(*MapPiece)[j]->GetObject(object->mID);
      if(tmp!=NULL)
        {
        return tmp;
        }
      ExpectedPiece=(*MapPiece)[j];
      break;
      }
    }
  
  /* Now in the neighbors */
  if(tmp==NULL)
    {
    for(j=0;j<MapPiece->Size();j++)
      {
      if((*MapPiece)[j]->isNeighbor(ExpectedPiece)==TRUE)
        {
        tmp=(*MapPiece)[j]->GetObject(object->mID);
        if(tmp!=NULL)
          {
          return tmp;
          }
        }
      }  
    }

  /* Now everywhere */
  for(j=0;j<MapPiece->Size();j++)
    {
    tmp=(*MapPiece)[j]->GetObject(object->mID);
    if(tmp!=NULL)
      {
      return tmp;
      }
    }

  return NULL;
  }

/** This add an object to a MapPiece, object position determine MapPiece */
void MapObjects::AddObject(Object *object)
  {
  assert(object!=NULL);

  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->isInside(object)==TRUE)
      {
      (*MapPiece)[j]->AddObject(object);
      return;
      }
    }  
  }

/** This remove an object from it expected ( based on position ) MapPiece, 
    if it is not found there, we look for it in the neighbor MapPieces. */
void MapObjects::RemoveObject(Object *object)
  {
  assert(object!=NULL);

  BOOL Found;
  ObjectPiece *ExpectedPiece;
  Uint32 j;

  for(j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->isInside(object)==TRUE)
      {
      Found=(*MapPiece)[j]->RemoveObject(object);
      ExpectedPiece=(*MapPiece)[j];
      break;
      }
    }  

  if(Found==FALSE)
    {
    for(j=0;j<MapPiece->Size();j++)
      {
      if((*MapPiece)[j]->isNeighbor(ExpectedPiece)==TRUE)
        {
        Found=(*MapPiece)[j]->RemoveObject(object);
        if(Found==TRUE) return;
        }
      }  
    }
  }

/** This remove an object looking it in ALL the Map pieces. */
void MapObjects::RemoveObject(Uint32 uID)
  {
  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->RemoveObject(uID)==TRUE)
      {
      return;
      }
    }
  }


/* This method will add a piece of terrain to the Map or update it */
void MapObjects::UpdateObject(Object *object)
  {
  assert(object!=NULL);

  BOOL Found;
  ObjectPiece *ExpectedPiece;
  Uint32 j;

  for(j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->isInside(object)==TRUE)
      {
      Found=(*MapPiece)[j]->UpdateObject(object);
      ExpectedPiece=(*MapPiece)[j];
      break;
      }
    }

  if(Found==FALSE)
    {
    for(j=0;j<MapPiece->Size();j++)
      {
      if((*MapPiece)[j]->isNeighbor(ExpectedPiece)==TRUE)
        {
        Found=(*MapPiece)[j]->UpdateObject(object);
        if(Found==TRUE) return;
        }
      }  
    }

  if(Found==FALSE)
    {
    AddObject(object);
    }
  }

/* This returns a list of the terrain tiles that are inside the circle. */
AList<Object> *MapObjects::isInRange(Position& Center, Uint32 Range)
  {
  AList<Object> *list=new AList<Object>();

  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    (*MapPiece)[j]->isInRange(list,Center,Range);
    }

  return list;
  }

Uint32 MapObjects::GetNumObjects()
  {
  Uint32 Num=0;

  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    Num+=(*MapPiece)[j]->GetNumObjects();
    }

  return Num;
  }


void MapObjects::GetSize(Uint32 *x1,Uint32 *y1,Uint32 *x2,Uint32 *y2)
  {
  Uint32 MinX,MinY,MasX,MasY;
  
  MinX=(*MapPiece)[0]->x_init;
  MinY=(*MapPiece)[0]->y_init;
  MasX=(*MapPiece)[0]->x_end;
  MasY=(*MapPiece)[0]->y_end;

  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->x_init<MinX)
      {
      MinX=(*MapPiece)[j]->x_init;
      }

    if((*MapPiece)[j]->y_init<MinY)
      {
      MinY=(*MapPiece)[j]->y_init;
      }

    if((*MapPiece)[j]->x_end>MasX)
      {
      MasX=(*MapPiece)[j]->x_end;
      }

    if((*MapPiece)[j]->y_end>MasY)
      {
      MasY=(*MapPiece)[j]->y_end;
      }
    }

  x1[0]=MinX;
  y1[0]=MinY;
  x2[0]=MasX;
  y2[0]=MasY;
  }
