/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
***************************************************************************/
#include "MapTerrain.h"
#include "FilePath.h"
#include "Log.h"


MapTerrains::MapTerrains()
  {
  MapPiece=new AList<TerrainPiece>();
  }

MapTerrains::~MapTerrains()
  {
  delete MapPiece;
  }

void MapTerrains::CreateNewTerrainPiece(Uint32 x_init,Uint32 y_init,Uint32 x_end,Uint32 y_end)
  {
  TerrainPiece *tmp=new TerrainPiece();

  tmp->x_init=x_init;
  tmp->y_init=y_init;
  tmp->x_end=x_end;
  tmp->y_end=y_end;

  MapPiece->Add(tmp);
  }


BOOL MapTerrains::SavePiece(AString FileName,Uint32 x_init,Uint32 y_init,Uint32 x_end,Uint32 y_end)
  {
  FilePath path( FileName );
  path.getValidFilePath( FileName );

  for(Uint32 i=0;i<MapPiece->Size();i++)
    {
    if((*MapPiece)[i]->x_init>=x_init && (*MapPiece)[i]->x_end<=x_end)
      {
      if((*MapPiece)[i]->y_init>=y_init && (*MapPiece)[i]->y_end<=y_end)
        {
        AVector *Vector=new AVector();        
        (*MapPiece)[i]->ToVector(Vector);

        AVector *CompressData=Vector->Compress();

        Uint8 *Data=CompressData->Get();
        AUint32 Size=CompressData->GetVectorSize();

        FileName=FileName+".terrain";

        FILE *Handle=fopen((const char *)FileName,"wb");
        if(Handle==NULL)
          {
          Log::Error("ERROR: Unable to open file %s",(const char *)FileName);
          }

        AVector *tmpVector=new AVector;
        Size.ToVector(tmpVector);

        fwrite(tmpVector->Get(),1,tmpVector->GetVectorSize(),Handle);
        Uint32 writed=fwrite(Data,1,Size,Handle);

        assert(writed==Size);

        fclose(Handle);

        delete CompressData;
        delete Vector;
        delete tmpVector;
        }
      }
    }

  return TRUE;
  }

BOOL MapTerrains::LoadPiece(AString FileName,Uint32 x_init,Uint32 y_init,Uint32 x_end,Uint32 y_end)
  {
  FilePath path( FileName );
  path.getValidFilePath( FileName );

  FileName=FileName+".terrain";

  FILE *Handle=fopen((const char *)FileName,"rb");
  if(Handle==NULL)
    {
    Log::Error("ERROR: Unable to open file %s",(const char *)FileName);
    }

  AVector *tmpVector=new AVector();
  tmpVector->SetVectorSize(4);
  Uint8 buffer[4];
  fread(buffer,4,sizeof(Uint8),Handle);
  tmpVector->AddUint8(buffer,4);
  tmpVector->SetIndex(0);

  AUint32 tmpSize;
  tmpSize.FromVector(tmpVector);
  Uint32 Size=(Uint32)tmpSize;
  delete tmpVector;

  Uint8 *Data=new Uint8[Size];
  Uint32 readed=fread(Data,1,Size,Handle);

  assert(readed==Size);

  fclose(Handle);

  TerrainPiece *tmp=new TerrainPiece;

  AVector *Vector=new AVector();
  Vector->SetVectorSize(Size);
  Vector->AddUint8(Data,Size);
  Vector->SetIndex(0);

  AVector *DecompressData=Vector->Decompress();

  tmp->FromVector(DecompressData);

  MapPiece->Add(tmp);

  delete DecompressData;
  delete Vector;
  return TRUE;
  }

void MapTerrains::Clear()
  {
  for(Uint32 i=0;i<MapPiece->Size();i++)
    {
    (*MapPiece)[i]->Clear();
    }
  }

/** This method returns a List of all the Terrains that are in the piece contained 
    fully in the given box */
AList<Terrain> *MapTerrains::GetListTerrains(Uint32 x_init,Uint32 y_init,Uint32 x_end,Uint32 y_end)
  {
  AList<Terrain> *tmp=new AList<Terrain>;

  for(Uint32 i=0;i<MapPiece->Size();i++)
    {
    if((*MapPiece)[i]->x_init>=x_init && (*MapPiece)[i]->x_end<=x_end)
      {
      if((*MapPiece)[i]->y_init>=y_init && (*MapPiece)[i]->y_end<=y_end)
        {
        (*tmp)+(*MapPiece)[i]->Terrains;
        }
      }
    }


  return tmp;
  }

/** This looks for an Terrain in ALL the Map Pieces */
Terrain *MapTerrains::GetTerrain(Uint32 x1,Uint32 y1,Uint32 z1)
  {
  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    Terrain *tmp=(*MapPiece)[j]->GetTerrain(x1,y1,z1);
    if(tmp!=NULL)
      {
      return tmp;
      }
    }
  
  return NULL;
  }

/** This add an Terrain to a MapPiece, Terrain position determine MapPiece */
void MapTerrains::AddTerrain(Terrain *terrain)
  {
  assert(terrain!=NULL);

  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->isInside(terrain)==TRUE)
      {
      (*MapPiece)[j]->AddTerrain(terrain);
      return;
      }
    }  
  }

/** This remove an Terrain from it expected ( based on position ) MapPiece, 
    if it is not found there, we look for it in the neighbor MapPieces. */
void MapTerrains::RemoveTerrain(Terrain *terrain)
  {
  assert(terrain!=NULL);

  BOOL Found;
  TerrainPiece *ExpectedPiece;
  Uint32 j;

  for(j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->isInside(terrain)==TRUE)
      {
      Found=(*MapPiece)[j]->RemoveTerrain(terrain);
      ExpectedPiece=(*MapPiece)[j];
      break;
      }
    }  

  if(Found==FALSE)
    {
    for(j=0;j<MapPiece->Size();j++)
      {
      if((*MapPiece)[j]->isNeighbor(ExpectedPiece)==TRUE)
        {
        Found=(*MapPiece)[j]->RemoveTerrain(terrain);
        if(Found==TRUE) return;
        }
      }  
    }
  }

/** This remove an Terrain looking it in ALL the Map pieces. */
void MapTerrains::RemoveTerrain(Uint32 x1,Uint32 y1,Uint32 z1)
  {
  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->RemoveTerrain(x1,y1,z1)==TRUE)
      {
      return;
      }
    }
  }


/* This method will add a piece of terrain to the Map or update it */
void MapTerrains::UpdateTerrain(Terrain *terrain)
  {
  assert(terrain!=NULL);

  BOOL Found;
  TerrainPiece *ExpectedPiece;
  Uint32 j;

  for(j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->isInside(terrain)==TRUE)
      {
      Found=(*MapPiece)[j]->UpdateTerrain(terrain);
      ExpectedPiece=(*MapPiece)[j];
      break;
      }
    }

  if(Found==FALSE)
    {
    for(j=0;j<MapPiece->Size();j++)
      {
      if((*MapPiece)[j]->isNeighbor(ExpectedPiece)==TRUE)
        {
        Found=(*MapPiece)[j]->UpdateTerrain(terrain);
        if(Found==TRUE) return;
        }
      }  
    }

  if(Found==FALSE)
    {
    AddTerrain(terrain);
    }
  }

/* This returns a list of the terrain tiles that are inside the circle. */
AList<Terrain> *MapTerrains::isInRange(Uint32 x1,Uint32 y1,Uint32 x2,Uint32 y2)
  {
  AList<Terrain> *list=new AList<Terrain>();

  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    (*MapPiece)[j]->isInRange(list,x1,y1,x2,y2);
    }

  return list;
  }

Uint32 MapTerrains::GetNumTerrains()
  {
  Uint32 Num=0;

  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    Num+=(*MapPiece)[j]->GetNumTerrains();
    }

  return Num;
  }


void MapTerrains::GetSize(Uint32 *x1,Uint32 *y1,Uint32 *x2,Uint32 *y2)
  {
  Uint32 MinX,MinY,MasX,MasY;
  
  MinX=(*MapPiece)[0]->x_init;
  MinY=(*MapPiece)[0]->y_init;
  MasX=(*MapPiece)[0]->x_end;
  MasY=(*MapPiece)[0]->y_end;

  for(Uint32 j=0;j<MapPiece->Size();j++)
    {
    if((*MapPiece)[j]->x_init<MinX)
      {
      MinX=(*MapPiece)[j]->x_init;
      }

    if((*MapPiece)[j]->y_init<MinY)
      {
      MinY=(*MapPiece)[j]->y_init;
      }

    if((*MapPiece)[j]->x_end>MasX)
      {
      MasX=(*MapPiece)[j]->x_end;
      }

    if((*MapPiece)[j]->y_end>MasY)
      {
      MasY=(*MapPiece)[j]->y_end;
      }
    }

  x1[0]=MinX;
  y1[0]=MinY;
  x2[0]=MasX;
  y2[0]=MasY;
  }

