/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is the other piece of the Network layer.
    This class manages the Queue to feet the lower layer and get messages
    from there.

    Most of the code of this class is done on each message.
***************************************************************************/
#include "MessageManager.h"
#include "Log.h"

/**
This is the class constructor.
**/
MessageManager::MessageManager()
  {
  mlQueues=new Queue*[MESSAGES_TYPE_NUMBER];
  for(int i=0;i<MESSAGES_TYPE_NUMBER;i++)
    {
    mlQueues[i]=0;
    }
  }

/**
This is the class destructor.
**/
MessageManager::~MessageManager()
  {
  delete mlQueues;
  }

/**
This returns the number of queues
**/
int MessageManager::SizeQueues()
  {
  return MESSAGES_TYPE_NUMBER;
  }

/**
This returns the queue at position Position.
**/
Queue *MessageManager::GetQueue(int Position)
  {
  assert(Position<MESSAGES_TYPE_NUMBER);

  return mlQueues[Position];
  }

/**
This add a queue to the Queue List
**/
void MessageManager::AddQueue(Queue *queue)
  {
  assert(queue!=NULL);
  assert(queue->GetType()<MESSAGES_TYPE_NUMBER);

  mlQueues[queue->GetType()]=queue;
  }


/**
This method will test if there is any message of the specified type.
**/
BOOL MessageManager::IsEmpty(char type,char subtype,Uint32 ID)
  {
  /* We look in the Type Queue for a Message of the specifie subtype . */
  /* We try to find what type of Message we have. */
  Queue *tmp=GetQueue(type);
  if(tmp!=NULL)
    {
    if(tmp->isInAvaliable(subtype,ID)>0)
      {
      return FALSE;
      }        
    } 
  else
    {
    /* ERROR this is an unknown message type. */
    return TRUE;
    }

  /* We have a message that hasn't any of the known types. */
  return TRUE;
  }

