/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This class start the Network layer.
    This is just a container for the two classes that are the Net Layer.
***************************************************************************/
#include "NetworkInit.h"
#include "Log.h"
#include "Messages.h"
#include "TimedWait.h"
#include "DataLoader.h"
#include <iostream>


NetworkInit::NetworkInit():
  mMesMan(0),
  mNetMan(0)
  {
  if(SDL_WasInit(SDL_INIT_TIMER)!=SDL_INIT_TIMER)
    {
    SDL_InitSubSystem(SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
    }

  mNetMan=new NetworkManager;
  mMesMan=new MessageManager;
  for(int i=0;i<MESSAGES_TYPE_NUMBER;i++)
    {
    Queue *tmp=new Queue;
    tmp->SetType(i);

    mNetMan->AddQueue(tmp);
    mMesMan->AddQueue(tmp);
    }
  }

NetworkInit::~NetworkInit()
  {
  if(mMesMan) delete mMesMan;
  if(mNetMan) delete mNetMan;
  }
MessageManager *NetworkInit::GetMessageManager()
  {
  return mMesMan;
  }

NetworkManager *NetworkInit::GetNetworkManager()
  {
  return mNetMan;
  }

/**
We create all the data structures and initialize it.
**/
void NetworkInit::Connect(char const* server,Uint16 port)
  {
  Log::Debug("NetworkInit::Init(%s ,%i)",server,port);

  Socket *tmp;
  if(DataLoader::GetUseProxy()==0)
    {
    if(server)
      {
      tmp=new Socket();

      tmp->ResolveHost(server,port);
      tmp->Open();

      if(tmp->isError()==TRUE)
        {
        Log::Error("Can't Open connection to server");
        }
      }
      
    else
      {
      Log::Error("Invalid server Name");
      }
    }
  else
    {
    /* EXPERIMENTAL: Needs to be tested. */
    /* We want to use a proxy */
    const char *ProxyName=(const char *)(DataLoader::GetProxyName());
    Uint16 ProxyPort=DataLoader::GetProxyPort();

    tmp=new Socket();

    tmp->ResolveHost(ProxyName,ProxyPort);
    tmp->Open();

    if(tmp->isError()==TRUE)
      {
      Log::Error("ERROR: Can't Open connection to proxy");
      }

    SocketSet *tmpSet=new SocketSet(1);
    tmpSet->Add(tmp);
    
    char *ConnectString=new char[256];
    sprintf(ConnectString,"CONNECT %s:%d HTTP/1.0\r\n\r\n",server,port);

    tmp->Send(ConnectString,strlen(ConnectString),1);
    delete ConnectString;

    char *RecievedString=new char[1024];
    Uint16 j=0;

    BOOL done=FALSE;
    while(done==FALSE)
      {
      tmpSet->Check(1000);
      if(tmpSet->isReady(tmp))
        {
        tmp->Recv(&RecievedString[j],1,1);
        j++;
        }
      else
        {
        done=TRUE;
        }
      }
    char *tmpBuffer=new char[256];
    Uint16 CodeNum;

    sscanf(RecievedString,"%s %d",tmpBuffer,&CodeNum);
    Log::Error("Proxy Server %s:%d does not support HTTP Tunnelling",
              (const char *)(DataLoader::GetProxyName()),(Uint16)(DataLoader::GetProxyPort()));

    delete tmpBuffer;
    delete RecievedString;

    delete tmpSet;
    }

  mNetMan->SetSocket(tmp);

  chrono_start=SDL_GetTicks();
  }

/**
The start running it.
**/
void NetworkInit::Run()
  {
  Log::Debug("Starting Network Layer");
  #ifdef macintosh
    mNetMan->startThread();
  #else
  #ifndef THREADLESS
    mNetMan->start();
  #endif
  #endif
  }

void NetworkInit::Shutdown()
  {
  Log::Debug("Stopping Network Layer");
  #ifndef macintosh				// Mac version needs cleaning up here... Not implemented yet.
  #ifndef THREADLESS
  mNetMan->stop();
  #endif
  #endif
//  SDL_Delay(1000);

  Log::Debug("Flushing messages");
  mNetMan->Flush();

  chrono_end=SDL_GetTicks();

  TimedWait timer;
  
  Message_S2C_LOGOUT mes_s2c_logout(mMesMan);
  while(mes_s2c_logout.isEmpty(0)==TRUE)
    {
    if(timer.hasExpired(2000)==TRUE)
      {
      Socket *tmp=mNetMan->GetSocket();
      Log::Warning("Client didn't get Server Bye string");
      Log::Debug("---- Statistics ----");
      Log::Debug("Bytes sent    : %i",tmp->GetBytesSent());
      Log::Debug("Bytes recieved: %i",tmp->GetBytesRecv());  
      float time_elapsed=((float)(chrono_end-chrono_start)/(float)1000.0);
      Log::Debug("KB/s  sent    : %f",((float)tmp->GetBytesSent()/1024.0)/time_elapsed);
      Log::Debug("KB/s  recieved: %f",((float)tmp->GetBytesRecv()/1024.0)/time_elapsed);
      return;
      }

    mNetMan->loop();
    }

  mes_s2c_logout.Get(0);
  Log::Debug("SERVER said \"%s\"",(const char *)(mes_s2c_logout.GetByeString()));
 
  Socket *tmp=mNetMan->GetSocket();
  Log::Debug("---- Statistics ----");
  Log::Debug("Bytes sent    : %i",tmp->GetBytesSent());
  Log::Debug("Bytes recieved: %i",tmp->GetBytesRecv());  
  float time_elapsed=((float)(chrono_end-chrono_start)/(float)1000.0);
  Log::Debug("KB/s  sent    : %f",((float)tmp->GetBytesSent()/1024.0)/time_elapsed);
  Log::Debug("KB/s  recieved: %f",((float)tmp->GetBytesRecv()/1024.0)/time_elapsed);
  }