/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is the class that send/recieve generic messages.
    This is the lower piece of the Network layer.
***************************************************************************/
#include "NetworkManager.h"
#include "Log.h"
#ifdef macintosh
#include "../MAC_FILES/ServerSocketSender.h"
#include "../MAC_FILES/ServerSocketReceiver.h"
#endif

/**
This is the class constructor.
**/
NetworkManager::NetworkManager()
#ifndef macintosh
	:LoopThread()
#endif
  {
  mlQueues=new Queue*[MESSAGES_TYPE_NUMBER];

  /* There is no bnadwidth limit. */
  mBandWidthLimit=0;

  /* Initially we aren't running it */
  mRunning=FALSE;

  mSocket=NULL;
  mSocketSet=NULL;
  }

/**
This is the class destructor.
**/
NetworkManager::~NetworkManager()
  {
  /* There is no bandwidth limit. */
  mBandWidthLimit=0;

  /* Initially we aren't running it */
  mRunning=FALSE;

  if(mSocketSet) delete mSocketSet;
  if(mSocket) delete mSocket;
  }


/**
This method set the bandwidth limit.
Network manager will TRY not to send more than this.
Use 0 for no limit.
**/
void NetworkManager::SetBandWidthLimit(int BytesPerSec)
  {
  mBandWidthLimit=BytesPerSec;
  }

/**
This method returns the actual BandWidthLimit.
**/
int NetworkManager::GetBandWidthLimit()  
  {
  return mBandWidthLimit;
  }
  

/**
This returns the number of queues
**/
int NetworkManager::SizeQueues()
  {
  return MESSAGES_TYPE_NUMBER;
  }

/**
This returns the queue at position Position.
**/
Queue *NetworkManager::GetQueue(int Position)
  {
  if(Position<MESSAGES_TYPE_NUMBER)
    {
    return mlQueues[Position];
    }

  return NULL;
  }

/**
This add a queue to the Queue List
**/
void NetworkManager::AddQueue(Queue *queue)
  {
  if(queue->GetType()<MESSAGES_TYPE_NUMBER)
    {
    mlQueues[queue->GetType()]=queue;
    }
  }

/**
This set the Socket where all the data is send and where the data is recieved
**/
void NetworkManager::SetSocket(Socket *socket)
  {
  mSocket=socket;
  mSocketSet=new SocketSet(1);
  mSocketSet->Add(mSocket);
  }

/**
This set the Socket where all the data is send and where the data is recieved
**/
void NetworkManager::SetSocket(Socket *socket,SocketSet *socketset)
  {
  mSocket=socket;
  mSocketSet=socketset;
  }

/**
This returns the Socket.
**/
Socket *NetworkManager::GetSocket()
  {
  return mSocket;
  }

#ifdef macintosh
void
NetworkManager::startThread()
{
	ServerSocketSender* sender = new ServerSocketSender( this );
	ServerSocketReceiver* receiver = new ServerSocketReceiver( this );
	if( sender && receiver )
	{
		sender->startSending();
		receiver->startReceiving();
	}
	else
	{
		Log::Error("Could not start ServerSocketSender/ServerSocketReceiver");
	}
}
void
NetworkManager::stopThread()
{
	if( senderID )
		::DisposeThread( senderID, nil, false );
	if( receiverID )
		::DisposeThread( receiverID, nil, false );
}
#endif


/**
This method will run the Network Manager in diferent thread, that is created on 
the call to the method.
**/
void NetworkManager::loop()
  {
  /* Now we send our data if there is any. */
//  Log::Debug("Sending outgoing messages");
  int i;
  for(i=0;i<SizeQueues();i++)
    {
    if(GenericMessage *GenMes=GetQueue(i)->GetOutMessage())
      {
      Log::Debug("Sending message");

      *mSocket<<*GenMes;

      /* Free the memory. */
      delete(GenericMessage*) GenMes;

      if(mSocket->isError()==TRUE)
        {
        /* Aborting. Severe error found */
        Log::Error("Socket error on SENDING, CLOSING connection");
        return;
  	    }
      }
	}
//  Log::Debug("Checking for incoming messages");
  /* Here we wait until a message is reached and then we read it and copy it
   to the correct queue. We set a timeout of 0 ms  */

  mSocketSet->Check(0);

  if(mSocketSet->isReady(mSocket)==TRUE)
    {
    GenericMessage *GenMes=new GenericMessage;

    *mSocket>>*GenMes;

    if(mSocket->isError()==TRUE)
      {
      /* Aborting. Severe error found */
      Log::Error("Socket error on RECIEVING, CLOSING connection");
      return;
      }

    Log::Debug("Received message");
    /* GenMes->Dump(std::cerr);  -- THIS IS UNUSEFULL, TOO MUCH INFO -- */ 
    /* Now we look for the correct queue where to add this type of Message. */
    /* A hash table or even a simple array will make miracles here. */
    Queue *tmp=GetQueue(GenMes->Type);
    if(tmp!=NULL)
      {
	  tmp->AddInMessage(GenMes);
      } 
    else
      {
  	  /* ERROR this is an unknown message type. */
      Log::Warning("Unknown message type.");
	  }
	}
  }

void NetworkManager::Flush()
  {
  Log::Debug("Flushing remaining messages");
  BOOL isData=TRUE;

  while(isData==TRUE)
    {
    isData=FALSE;
    int i;
    for(i=0;i<SizeQueues();i++)
      {
      Queue *tmp=GetQueue(i);
      GenericMessage *GenMes=tmp->GetOutMessage();

      if(GenMes!=NULL)
        {
        *mSocket<<*GenMes;
        isData=TRUE;

        if(mSocket->isError()==TRUE)
          {
          Log::Error("Socket error on SENDING, CLOSING connection");
          return;
  	      }
        }
      }
	}

  }

/**
This method will tell you if the Network manager is runnig
**/
BOOL NetworkManager::IsRunning()
  {
  return mRunning;
  }
