/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is the NetworkInit class for the server.
    It has some differences with the one in the client, but the basic
    idea behind is the same.
***************************************************************************/
#include "NetworkServerInit.h"
#include "PlayerID.h"
#include "Log.h"
NetworkServerInit::NetworkServerInit():
  mMesMan(0),
  mNetMan(0)
  {
  if(SDL_WasInit(SDL_INIT_TIMER)!=SDL_INIT_TIMER)
    {
    SDL_InitSubSystem(SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
    }
  }

NetworkServerInit::~NetworkServerInit()
  {
  if(mMesMan) delete mMesMan;
  if(mNetMan) delete mNetMan;
  }

MessageManager *NetworkServerInit::GetMessageManager()
  {
  return mMesMan;
  }

NetworkServerManager *NetworkServerInit::GetNetworkServerManager()
  {
  return mNetMan;
  }


/**
We create all the data structures and initialize it.
**/
void NetworkServerInit::Init(char const* server,Uint16 port)
  {
  Log::Debug("NetworkServerInit::Init(%s,%d)",server,port);
  mNetMan=new NetworkServerManager();
  mMesMan=new MessageManager();

  for(int i=0;i<MESSAGES_TYPE_NUMBER;++i)
    {
    Queue *tmp=new Queue;
    tmp->SetType(i);

    mNetMan->AddQueue(tmp);
    mMesMan->AddQueue(tmp);
    }

  if(server==NULL)
    {
    Socket *tmp=new Socket;

    tmp->ResolveHost(server,port);
    tmp->Open();
	
    if(tmp->isError()==TRUE)
      {
      Log::Error("Socket error on CREATING");
      }

    mNetMan->SetSocket(tmp);
    }
  else
    {
    Log::Error("Trying to create a server on another machine");
    }
  }

/**
The start running it.
**/
void NetworkServerInit::Run()
  {
  Log::Debug("NetworkServerInit::Run");
  // On Mac, we start another thread for listening to in-coming connection
  #ifndef THREADLESS
  #ifdef macintosh
  mNetMan->startListening();
  #else
  mNetMan->start();
  #endif
  #endif
  }

void NetworkServerInit::Shutdown()
  {
  Log::Debug("Stopping Network Layer");
  #ifndef THREADLESS
  #ifndef macintosh
  mNetMan->stop();
  #endif
  #endif
  }