/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is the CORE class for the server.
    It has some differences with the one in the client, but the basic
    idea behind is the same.
***************************************************************************/
#include "NetworkServerManager.h"
#include "Log.h"
#include <iostream>

#ifdef macintosh
#include "SDL_net.h"
#include "SDLnetsys.h"
#include <OpenTransport.h>
#include <OpenTptInternet.h>

#include "../MAC_FILES/ClientSocketReceiver.h"
#include "../MAC_FILES/ClientSocketSender.h"

#endif

/**
This is the class constructor.
**/
NetworkServerManager::NetworkServerManager()
  #ifndef macintosh
  :LoopThread()
  #endif
  {
  mlQueues=new Queue*[MESSAGES_TYPE_NUMBER];

  /* There is no bnadwidth limit. */
  mBandWidthLimit=0;

  /* Initially we aren't running it */
  mRunning=FALSE;
  
  mSocket=NULL;
  mSocketSet=NULL;
  
  for(int i=0;i<MAX_CONNECTIONS_PER_SERVER;i++)
    {
    mlSockets[i]=NULL;
    isLogout[i]=TRUE;
    #ifdef macintosh
   	clients[i] = 0;
    #endif
    }

  LastIndexUsed=0;
  Errors=0;   
  }
  

/**
This is the class destructor.
**/
NetworkServerManager::~NetworkServerManager()
  {
  delete mlQueues;

  for(int i=0;i<MAX_CONNECTIONS_PER_SERVER;i++)
    {
    if(mlSockets[i])
      {
      delete mlSockets[i];
      }
    }

  delete mSocket;

  /* There is no bandwidth limit. */
  mBandWidthLimit=0;

  /* Initially we aren't running it */
  mRunning=FALSE;
  }


/**
This method set the bandwidth limit.
Network manager will TRY not to send more than this.
Use 0 for no limit.
**/
void NetworkServerManager::SetBandWidthLimit(int BytesPerSec)
  {
  mBandWidthLimit=BytesPerSec;
  }

/**
This method returns the actual BandWidthLimit.
**/
int NetworkServerManager::GetBandWidthLimit()  
  {
  return mBandWidthLimit;
  }
  
/**
This returns the number of queues
**/
int NetworkServerManager::SizeQueues()
  {
  return MESSAGES_TYPE_NUMBER;
  }

/**
This returns the queue with type Type.
**/
Queue *NetworkServerManager::GetQueue(int Type)
  {
  if(Type<MESSAGES_TYPE_NUMBER)
    {
    return mlQueues[Type];
    }

  return NULL;
  }

/**
This add a queue to the Queue List
**/
void NetworkServerManager::AddQueue(Queue *queue)
  {
  if(queue->GetType()<MESSAGES_TYPE_NUMBER)
    {
    mlQueues[queue->GetType()]=queue;
    }
  }

/**
This set the Socket where all the data is send and where the data is recieved
**/
void NetworkServerManager::SetSocket(Socket *socket)
  {
  assert(socket!=NULL);
   
  mSocket=socket;
  mSocketSet=new SocketSet(1+MAX_CONNECTIONS_PER_SERVER);
  mSocketSet->Add(mSocket);
  }

/**
This set the Socket where all the data is send and where the data is recieved
**/
void NetworkServerManager::SetSocket(Socket *socket,SocketSet *socketset)
  {
  mSocket=socket;
  mSocketSet=socketset;
  }

/**
This returns the Socket.
**/
Socket *NetworkServerManager::GetSocket()
  {
  return mSocket;
  }

/**
This returns the Socket.
**/
Socket *NetworkServerManager::GetSocket(Uint16 i)
  {
  assert(i<MAX_CONNECTIONS_PER_SERVER);

  return mlSockets[i];
  }

// Not used on Mac any more.
#ifndef macintosh
/**
This method will run the Network Manager in diferent thread, that is created on 
the call to the method.
**/
void NetworkServerManager::loop()
  {
//  Log::Debug("NetworkServerManager::loop");
  mSocketSet->Check(0);
  TestForNewConnections();

  for(int i=0;i<MAX_CONNECTIONS_PER_SERVER;i++)
    {
    if(mlSockets[i]!=NULL) 
      {
      SendDataToClient(i);
      }

    /* It can't be joined because there can be errors in SendData */
    if(mlSockets[i]!=NULL)
      {
      RecvDataFromClient(i); 
      }

    if(mlSockets[i]!=NULL)
      {
      if(mlSockets[i]->isError()==TRUE)
        {
        CloseConnection(i);
        }
      }
    }
  } 
#endif


/**
This method will tell you if the Network manager is runnig
**/
BOOL NetworkServerManager::IsRunning()
  {
  return mRunning;
  }

void NetworkServerManager::CloseConnection(Uint16 i)
  {
  Log::Debug("CLOSING connection");
  mSocketSet->Del(mlSockets[i]);
  mlSockets[i]->Close();
  mlSockets[i]=NULL;
  }

Uint32 NetworkServerManager::GetErrors()
  {
  return Errors;
  }

  
// TestForNewConnections() and 
// SendDataToClient/RecvDataFromClient() are NOT USED ON MAC
#ifndef macintosh
void NetworkServerManager::TestForNewConnections()
  {
  //Log::Debug("NetworkServerManager::TestForNewConnections");
  // Not used on Mac any more
  if(mSocketSet->isReady(mSocket)==TRUE)
    {
    /* We have a new connection incomming */
    Socket *tmp=mSocket->Accept();
    if(tmp==NULL)
      {
      /* Connection not accepted */
      Log::Warning("Erron on new connection");
      return;
      }

    int i;
    Log::Debug("New connection incoming");
    for(i=LastIndexUsed;i<MAX_CONNECTIONS_PER_SERVER;i++)
      {
      if(mlSockets[i]==NULL)
        {
        mlSockets[i]=tmp;
        isLogout[i]=FALSE;
        mSocketSet->Add(tmp);
        LastIndexUsed=(i+1)%MAX_CONNECTIONS_PER_SERVER;
        Log::Debug("Connection added ID=%i",i);        

        /* We clean now the Queues from Old messages. */
        for(int j=0;j<MESSAGES_TYPE_NUMBER;j++)
          {
          Queue *tmp=GetQueue(j);
          
          GenericMessage *mes=tmp->GetOutMessage(i);

          while(mes!=NULL)
            {
            delete mes;
            mes=tmp->GetOutMessage();
            }
          }
        return;
        }
      }

    for(i=0;i<LastIndexUsed;i++)
      {
      if(mlSockets[i]==NULL)
        {
        mlSockets[i]=tmp;
        isLogout[i]=FALSE;
        mSocketSet->Add(tmp);
        LastIndexUsed=(i+1)%MAX_CONNECTIONS_PER_SERVER;
        Log::Debug("Connection added ID=%i",i);

        /* We clean now the Queues from Old messages. */
        for(int j=0;j<MESSAGES_TYPE_NUMBER;j++)
          {
          Queue *tmp=GetQueue(j);
          
          GenericMessage *mes=tmp->GetOutMessage(i);

          while(mes!=NULL)
            {
            delete mes;
            mes=tmp->GetOutMessage();
            }
          }

        return;
        }
      }

    /* There is no room. */
    /* Perhaps we should sent him some data to know of the problem */
    Log::Warning("+++++Maximum number of connections reached+++++");
    delete tmp;
    }
  }

void NetworkServerManager::SendDataToClient(Uint32 ID)
  {
  //Log::Debug("NetworkServerManager::SendDataToClient(Uint32 ID =%i)",ID);
  /* Now we send our data if there is any. */
  if(mlSockets[ID]->isError()==FALSE)
  for(int i=0;i<SizeQueues();i++)
    {
    GenericMessage *GenMes=GetQueue(i)->GetOutMessage(ID);
    while(GenMes!=NULL)
      {
      *mlSockets[ID]<<*GenMes;

      if(GenMes->Type==ARIANNE_ADMIN && GenMes->SubType==ARIANNE_LOGOUT)
        {
        /* We send a LOGOUT to the client. So we can close this socket. */
        isLogout[ID]=TRUE;
        mSocketSet->Del(mlSockets[ID]);
        mlSockets[ID]->Close();
        mlSockets[ID]=NULL;
        delete GenMes;
        return;
        }

      delete GenMes;

      if(mlSockets[ID]->isError()==TRUE)
        {
        if(isLogout[ID]==FALSE)
          {
          Errors++;
          Log::Warning("Socket error on SENDING, CLOSING connection ID=%i",ID);
          mSocketSet->Del(mlSockets[ID]);
          isLogout[ID]=TRUE;
          mlSockets[ID]->Close();
          mlSockets[ID]=NULL;
          return;
          }
        }
      
      GenMes=GetQueue(i)->GetOutMessage(ID);
      }
    } 
  }

void NetworkServerManager::RecvDataFromClient(Uint32 ID)
  {
  //Log::Debug("NetworkServerManager::RecvDataFromClient(Uint32 ID =%i)",ID);
  /* Here we wait until a message is reached and then we read it and copy it
  to the correct queue. We set a timeout of 100 ms  */
  if(mlSockets[ID]->isError()==FALSE)
  if(mSocketSet->isReady(mlSockets[ID])==TRUE)
    {
    GenericMessage *GenMes=new GenericMessage;
 
    *mlSockets[ID]>>*GenMes;

    if(mlSockets[ID]->isError()==TRUE)
      {
      if(isLogout[ID]==FALSE)
        {
        Errors++;
        Log::Warning("Socket error on RECIEVING, CLOSING connection ID=%i",ID);
        mSocketSet->Del(mlSockets[ID]);
        isLogout[ID]=TRUE;
        mlSockets[ID]->Close();
        mlSockets[ID]=NULL;
        return;
        }
      }

    /* We store the correct ID as it isn't sent over the net. */
    GenMes->ID=ID;

    /* Now we look for the correct queue where to add this type of Message. */
    Queue *tmp=GetQueue(GenMes->Type);
    if(tmp!=NULL)
      {
      tmp->AddInMessage(GenMes);
      } 
    else
      {
      /* ERROR this is an unknown message type. */
      Log::Warning("Unknown Message Type"); 
      }
    }
  }
#endif

// Below are the Mac specific functions.   
#ifdef macintosh
void NetworkServerManager::startListening()
{
	ThreadID thread_ID;
	OSErr theErr = NewThread( kCooperativeThread, listenThreadProc, 
		reinterpret_cast<void*>(this), 0, 0, nil, &thread_ID );
}
int NetworkServerManager::getNewID()
{
	int i;
	for(i=0;i<MAX_CONNECTIONS_PER_SERVER;i++)
	{
		if(! clients[i])
		{
			break;
		}
	}
	if(i==MAX_CONNECTIONS_PER_SERVER)
	{
		/* There is no room. */
		/* Perhaps we should sent him some data to know of the problem */
		Log::Warning("+++++Maximum number of connections reached+++++");
		//delete sock;
		return -1;
	}
	return i;

}

void NetworkServerManager::lockID( int id)
{
	clients[id] = 1;
}

void NetworkServerManager::freeID( int id )
{
	clients[id] = 0;
}
BOOL NetworkServerManager::addNewSocket( Socket* sock )
{
	int i;
	Log::Debug("New connection incoming");
	for(i=0;i<MAX_CONNECTIONS_PER_SERVER;i++)
	{
		if(mlSockets[i]==NULL)
		{
			mlSockets[i]=sock;
			mSocketSet->Add(sock);
			Log::Debug("Connection added");
			break;
		}
	}
	if(i==MAX_CONNECTIONS_PER_SERVER)
	{
		/* There is no room. */
		/* Perhaps we should sent him some data to know of the problem */
		Log::Warning("Maximum number of connections reached");
		//delete sock;
		return false;
	}
	return true;
}

pascal void* listenThreadProc( void* netManagerPtr )
{
	while(1)
	{
		int newID = ((NetworkServerManager*) netManagerPtr)->getNewID();
		if( newID >= 0 )
		{
			Socket* newSocket = ((NetworkServerManager*)netManagerPtr)->GetSocket()->Accept();
			if( newSocket )
			{
						ClientSocketReceiver* receiver = new ClientSocketReceiver(
							newID, newSocket, (NetworkServerManager*)netManagerPtr );
						ClientSocketSender* sender = new ClientSocketSender(
							newID, newSocket, (NetworkServerManager*)netManagerPtr );
						if( receiver && sender )
						{
							receiver->startReceiving();
							sender->startSending();
							((NetworkServerManager*) netManagerPtr )->lockID(newID);
						}
				}
			}
	}
}

#endif //macintosh
