#include "ObjectPiece.h"

ObjectPiece::ObjectPiece()
  {
  Objects=new AList<Object>();
  x_init=y_init=x_end=y_end=0;
  }

ObjectPiece::~ObjectPiece()
  {
  delete Objects;
  }


Uint32 ObjectPiece::GetSize()
  {
  return 4*sizeof(Uint32)+Objects->GetSize();
  }

void ObjectPiece::ToVector(AVector *Vector)
  {
  assert(Vector!=NULL);

  AUint32 tmp;
  tmp=x_init;
  tmp.ToVector(Vector);
  tmp=y_init;
  tmp.ToVector(Vector);
  tmp=x_end;
  tmp.ToVector(Vector);
  tmp=y_end;
  tmp.ToVector(Vector);

  Objects->ToVector(Vector);
  }

void ObjectPiece::FromVector(AVector *Vector)
  {
  assert(Vector!=NULL);
   
  AUint32 tmp;
  tmp.FromVector(Vector);
  x_init=tmp;
  tmp.FromVector(Vector);
  y_init=tmp;
  tmp.FromVector(Vector);
  x_end=tmp;
  tmp.FromVector(Vector);
  y_end=tmp;

  Objects->FromVector(Vector);
  }

void ObjectPiece::Clear()
  {
  Objects->Delete();
  }

BOOL ObjectPiece::isInside(Uint32 x,Uint32 y)
  {
  if(x>=x_init && x<x_end)
    {
    if(y>=y_init && y<y_end)
      {
      return TRUE;
      }
    }

  return FALSE;
  }

BOOL ObjectPiece::isInside(Object *piece)
  {
  assert(piece!=NULL);
   
  Uint32 x,y,z;

  piece->Pos.GetXYZ(&x,&y,&z);

  if(x>=x_init && x<x_end)
    {
    if(y>=y_init && y<y_end)
      {
      return TRUE;
      }
    }

  return FALSE;
  }


Object *ObjectPiece::GetObject(Uint32 uID)
  {
  for(Uint32 i=0;i<Objects->Size();i++)
    {
    Object *tmp=(*Objects)[i];
    if(tmp->mID==uID)
      {
      return tmp;
      }
    }
  
  return NULL;
  }

void ObjectPiece::AddObject(Object *object)
  {
  assert(object!=NULL);
   
  Objects->Add(object);
  }

BOOL ObjectPiece::UpdateObject(Object *object)
  {
  assert(object!=NULL);
   
  for(Uint32 i=0;i<Objects->Size();i++)
    {
    Object *tmp=(*Objects)[i];
    if(tmp->mID==object->mID)
      {
      Objects->Free(i);
      Objects->Add(object);
      return TRUE;
      }
    }

  return FALSE;
  }

BOOL ObjectPiece::RemoveObject(Object *object)
  {
  assert(object!=NULL);
   
  for(Uint32 i=0;i<Objects->Size();i++)
    {
    Object *tmp=(*Objects)[i];
    if(tmp->mID==object->mID)
      {
      Objects->Free(i);
      return TRUE;
      }
    }

  return FALSE;
  }

BOOL ObjectPiece::RemoveObject(Uint32 uID)
  {
  for(Uint32 i=0;i<Objects->Size();i++)
    {
    Object *tmp=(*Objects)[i];
    if(tmp->mID==uID)
      {
      Objects->Free(i);
      return TRUE;
      }
    }

  return FALSE;
  }

BOOL ObjectPiece::isNeighbor(ObjectPiece *tmp)
  {
  assert(tmp!=NULL);
   
  Uint32 dx=(x_end-x_init)/2;
  Uint32 dy=(y_end-y_init)/2;
  
  if(tmp->isInside(x_init-dx,y_init-dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init,y_init-dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_end+dx,y_init-dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init-dx,y_init+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init+dx,y_init+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_end+dx,y_init+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init-dx,y_end+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init,y_end+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_end+dx,y_end+dy)==TRUE)
    {
    return TRUE;
    }

  return FALSE;
  }

void ObjectPiece::isInRange(AList<Object> *list,Position& Center, Uint32 Range)
  {
  assert(list!=NULL);
   
  Uint32 x,y,z;
  Center.GetXYZ(&x,&y,&z);

  if(x_init>x+Range ||x_end<((x>Range)?(x-Range):0))
    {
    return;
    }

  if(y_init>y+Range ||y_end<((y>Range)?(y-Range):0))
    {
    return;
    }

  for(Uint32 i=0;i<Objects->Size();i++)
    {
    Object *tmp=(*Objects)[i];

    if(tmp->Pos.Distance(Center)<Range)
      {
      list->Add(tmp);
      }
    }
  }



Uint32 ObjectPiece::GetNumObjects()
  {
  return Objects->Size();
  }

