/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This an abstraction of a Position.

    This describe all about the position, so the movement can be 
    interpreted later.
    Only lacks of acceleration, but I think that we don't need it
***************************************************************************/
#include "Position.h"
#include <cmath>

/**
This is the class constructor.
**/
Position::Position()
  {
  x=y=z=lastX=lastY=lastZ=0;
  yaw=roll=pitch=0;
  speedX=speedY=speedZ=0;
  speedYaw=speedRoll=speedPitch=0;
  }

/**
This is the class destructor.
**/
Position::~Position()
  {
  }
/**
This set the position of the object.
**/
void Position::SetXYZ(Uint32 x,Uint32 y,Uint32 z)
  {
  this->x=x;
  this->y=y;
  this->z=z;
  }

/**
This set the orientation of the object.
**/
void Position::SetYRP(Uint8 yaw,Uint8 roll,Uint8 pitch)
  {
  this->yaw=yaw;
  this->roll=roll;
  this->pitch=pitch;
  }

/**
This returns the position of the object.
**/
void Position::GetXYZ(Uint32 *x,Uint32 *y,Uint32 *z)
  {
  x[0]=(Uint32)this->x;
  y[0]=(Uint32)this->y;
  z[0]=(Uint32)this->z;
  }

/**
This returns the last position of the object before a move action.
This will be usefull for doing the collision detection.
**/
void Position::GetXYZLast(Uint32 *x,Uint32 *y,Uint32 *z)
  {
  x[0]=(Uint32)this->lastX;
  y[0]=(Uint32)this->lastY;
  z[0]=(Uint32)this->lastZ;
  }

/**
This returns the orientation of the object.
**/
void Position::GetYRP(Uint8 *yaw,Uint8 *roll,Uint8 *pitch)
  {
  yaw[0]=(Uint32)this->yaw;
  roll[0]=(Uint32)this->roll;
  pitch[0]=(Uint32)this->pitch;
  }

/**
This set the speed of the object
**/
void Position::SetMovementSpeed(Uint8 x,Uint8 y,Uint8 z)
  {
  speedX=x;
  speedY=y;
  speedZ=z;
  }

/**
This set the speed of the object
**/
void Position::SetRotationSpeed(Uint8 yaw,Uint8 roll,Uint8 pitch)
  {
  speedYaw=yaw;
  speedRoll=roll;
  speedPitch=pitch;
  }


void Position::MoveTo(Uint32 x,Uint32 y,Uint32 z)
  {
  lastX=this->x;
  lastY=this->y;
  lastZ=this->z;

  this->x=x;
  this->y=y;
  this->z=z;

  /* Now we calculate the orientation. */
  Sint32 dx,dy;
  dx=x-lastX;
  dy=y-lastY;

  if((dx==0)&&(dy==0)) return;

  float Angle=(float)(atan2(dy,dx)+3.14);
  
  /* Normalize it */
  Angle=Angle/(float)(3.14f*2.0f);

  roll=(Uint8)(Angle*255);
  }

/**
This modifies the position of the object moving it following forward or backward
the current orientation.
**/
void Position::Move(Uint8 Direction,Uint32 lastTime,Uint32 ActualTime)
  {
  /* TODO */
  }

/**
This change the current orientation of the object.
**/
void Position::Rotate(Uint8 Direction,Uint32 lastTime,Uint32 ActualTime)
  {
  /* TODO */
  }

Uint16 Position::GetSize()
  {
  return x.GetSize()+y.GetSize()+z.GetSize()+
         lastX.GetSize()+lastY.GetSize()+lastZ.GetSize()+
         yaw.GetSize()+roll.GetSize()+pitch.GetSize()+
         speedX.GetSize()+speedY.GetSize()+speedZ.GetSize()+
         speedYaw.GetSize()+speedRoll.GetSize()+speedPitch.GetSize();
  }

void Position::ToVector(AVector *Vector)
  {
  assert(Vector!=NULL);
   
  x.ToVector(Vector);
  y.ToVector(Vector);
  z.ToVector(Vector);
  lastX.ToVector(Vector);
  lastY.ToVector(Vector);
  lastZ.ToVector(Vector);
  yaw.ToVector(Vector);
  roll.ToVector(Vector);
  pitch.ToVector(Vector);
  speedX.ToVector(Vector);
  speedY.ToVector(Vector);
  speedZ.ToVector(Vector);
  speedYaw.ToVector(Vector);
  speedRoll.ToVector(Vector);
  speedPitch.ToVector(Vector);
  }

void Position::FromVector(AVector *Vector)
  {
  assert(Vector!=NULL);
   
  x.FromVector(Vector);
  y.FromVector(Vector);
  z.FromVector(Vector);
  lastX.FromVector(Vector);
  lastY.FromVector(Vector);
  lastZ.FromVector(Vector);
  yaw.FromVector(Vector);
  roll.FromVector(Vector);
  pitch.FromVector(Vector);
  speedX.FromVector(Vector);
  speedY.FromVector(Vector);
  speedZ.FromVector(Vector);
  speedYaw.FromVector(Vector);
  speedRoll.FromVector(Vector);
  speedPitch.FromVector(Vector);
  }


Uint32 Position::Distance(Position& pos)
  {
  int x2=(this->x-pos.x);
  int y2=(this->y-pos.y);
  int z2=(this->z-pos.z);
  return (Uint32)sqrt(x2*x2+y2*y2+z2*z2);
  }