/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is a simply queue, with In and Out lists.

    There are methods for looking the desired message.
***************************************************************************/
#include "Queue.h"

/**
This is the class constructor.
**/
Queue::Queue()
  {
  }

/**
This is the class destructor.
**/
Queue::~Queue()
  {
  }
  
/**
This clears the list of In messages.
**/
void Queue::ClearIn()
  {
  mInMessage.Clear();
  }

/**
This clears the list of Out messages.
**/
void Queue::ClearOut()
  {
  mOutMessage.Clear();
  }

/**
This add a message to the list of In messages.
**/
void Queue::AddInMessage(GenericMessage *GenMes)
  {
  /* We should sort the queue here */

  mInMessage.Add(GenMes,0);  
  }

/**
This get a message from the list of In messages and remove it.
**/
GenericMessage *Queue::GetInMessage(Uint32 ID)
  {
  /* We should sort the queue here */

  /* If it is empty we return NULL */
  if(mInMessage.Size()==0)
    {
    return NULL;
    }

  for(int i=0;i<mInMessage.Size();i++)
    {
    if(((GenericMessage *)mInMessage[i])->ID==ID)
      {
      GenericMessage *tmp=(GenericMessage *)mInMessage[i];
      mInMessage.Remove(i);
      return tmp;
      }
    }
 
  return NULL;
  }

/**
This get the first message of the specified subtype. 
**/
GenericMessage *Queue::GetInMessage(Uint8 subtype,Uint32 ID)
  {
  for(int i=0;i<mInMessage.Size();i++)
    {
    if(((GenericMessage *)mInMessage[i])->SubType==subtype)
      {
      if(((GenericMessage *)mInMessage[i])->ID==ID)
        {
        GenericMessage *tmp=(GenericMessage *)mInMessage[i];
        mInMessage.Remove(i);
        return tmp;
        }
      }
    }

  return NULL;
  }

/**
This returns the number of subtype messages in the In queue, 
**/ 
Uint32 Queue::isInAvaliable(Uint8 subtype,Uint32 ID)
  {
  Uint32 Number=0;

  for(int i=0;i<mInMessage.Size();i++)
    {
    if(((GenericMessage *)mInMessage[i])->SubType==subtype)
      {
      if(((GenericMessage *)mInMessage[i])->ID==ID)
        {
        Number++;
        }
      }
    }

  return Number;
  }

/**
This get the first message of the specified subtype. 
**/
GenericMessage *Queue::GetOutMessage(Uint8 subtype,Uint32 ID)
  {
  for(int i=0;i<mOutMessage.Size();i++)
    {
    if(((GenericMessage *)mOutMessage[i])->SubType==subtype)
      {
      if(((GenericMessage *)mOutMessage[i])->ID==ID)
        {
        GenericMessage *tmp=(GenericMessage *)mOutMessage[i];
        mOutMessage.Remove(i);
        return tmp;
        }
      }
    }

  return NULL;
  }

/**
This returns the number of subtype messages in the Out queue, 
**/ 
Uint32 Queue::isOutAvaliable(Uint8 subtype,Uint32 ID)
  {
  Uint32 Number=0;

  for(int i=0;i<mOutMessage.Size();i++)
    {
    if(((GenericMessage *)mOutMessage[i])->SubType==subtype)
      {
      if(((GenericMessage *)mOutMessage[i])->ID==ID)
        {
        Number++;
        }
      }
    }

  return Number;
  }


/**
This add a message to the list of Out messages.
**/
void Queue::AddOutMessage(GenericMessage *GenMes)
  {
  /* We should sort the queue here */

  mOutMessage.Add(GenMes,0);
  }

/**
This get a message from the list of Out messages and remove it.
**/
GenericMessage *Queue::GetOutMessage(Uint32 ID)
  {
  /* We should sort the queue here */

  /* If it is empty we return NULL */
  if(mOutMessage.Size()==0)
    {
    return NULL;
    }

  for(int i=0;i<mOutMessage.Size();i++)
    {
    if(((GenericMessage *)mOutMessage[i])->ID==ID)
      {
      GenericMessage *tmp=(GenericMessage *)mOutMessage[i];
      mOutMessage.Remove(i);
      return tmp;
      }
    }
 
  return NULL;
  }

/**
This set the type of messages this queue will store.
**/
void Queue::SetType(int Type)
  {
  mType=Type;
  }

/**
This returns the type of messages this queue will store.
**/
int Queue::GetType()
  {
  return mType;
  }


