/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is an abstraction of a Screen.
***************************************************************************/
#include "Screen.h"
#include "Log.h"

SDL_Surface *Screen::screen;
SDL_Surface *Screen::game_screen;

SDL_Surface *Screen::GetScreen()
  {
  return screen;
  }

/** This returns a buffer screen.
 GFXData::Show() draws on this screen now, so that game gfx ( map/characters...)
 are drawn separately from other GUI gfx.It is convenient for partial updating 
 of screen. */
SDL_Surface *Screen::GetGameScreen()
  {
  return game_screen;
  }

void Screen::SetScreen(SDL_Surface *vscreen)
  {
  assert(vscreen!=NULL);

  screen=vscreen;
  }


void Screen::SetSize(int x,int y,char depth,BOOL Fullscreen)
  {
  Log::Debug("Screen: Creating Screen...()");

  if(Fullscreen==TRUE)
    {
    screen=SDL_SetVideoMode(x,y,depth,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);
    }
  else
    {
    screen=SDL_SetVideoMode(x,y,depth,SDL_HWSURFACE|SDL_DOUBLEBUF);
    }
 
  if(screen==NULL)
    {
    Log::Error("SDL Error: %s",SDL_GetError());
    }

  game_screen=SDL_CreateRGBSurface(SDL_SWSURFACE, screen->w, screen->h,
                                      screen->format->BitsPerPixel, 0, 0, 0, 0 );
  }

void Screen::GetSize(int *w,int *h)
  {
  assert(w!=NULL);
  assert(h!=NULL);

  w[0]=screen->w;
  h[0]=screen->h;
  }


int Screen::Init()
  {
  Log::Debug("Screen::Init()");

  if(SDL_WasInit(SDL_INIT_VIDEO)!=SDL_INIT_VIDEO)
    {
    if(SDL_InitSubSystem(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE)<0)
      {
      Log::Error("SDL Error: %s",SDL_GetError());
      }
    }

  SDL_WM_SetCaption(ARIANNE_HARDCODE_VERSION,NULL);
  return 0;
  }

int Screen::Shutdown()
  {
  if( game_screen )
    {
  	SDL_FreeSurface( game_screen );
    }

  return 0;
  }

