/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This is an abstraction of a socket using SDL_net.
    Yes, I know it can be done better, but really is going to be little
    used so it doesn't matter that it has an ugly interface.
***************************************************************************/
#include "Socket.h"
#include "Log.h"

Uint32 Socket::NumSocketsOpen=0;

/**
This is the class constructor
**/
Socket::Socket():
  TCP(0)
  {
  if(NumSocketsOpen==0)
    {
    SDLNet_Init();
    }

  NumSocketsOpen++;
  mError=FALSE;

  SocketBytesSent=0;
  SocketBytesRecieved=0;
  }

/**
This is the class destructor
**/
Socket::~Socket()
  {
  if(TCP!=NULL)
    {
    Close();
    TCP=0;
    }

  NumSocketsOpen--;

  if(NumSocketsOpen==0)
    {
    SDLNet_Quit();
    }
  }
  
   
/**
This bind a socket with an address
**/
void Socket::ResolveHost(char const*Server,Uint16 Port)
  {
  SDLNet_ResolveHost(&IP,const_cast<char*>(Server),Port);
  }

/**
This returns the name of the ip address/
**/
char *Socket::ResolveIP()
  {
  return SDLNet_ResolveIP(&IP);
  }


int Socket::Open()
  {
  this->TCP=SDLNet_TCP_Open(&IP);

  if(TCP!=NULL) 
    {
	return 1;
	}

  mError=TRUE;
  return 0;
  }

/**
This method is called by a server socket to accept new connections.
If there is a timeout, a NULL is returned.
**/
Socket *Socket::Accept()
  {
  if(TCP==NULL)
    {
    /* Severe error. Aborting execution. */
    mError=TRUE;
    return NULL;
    }

  Socket *tmp=new Socket();
  tmp->TCP=SDLNet_TCP_Accept(TCP);

  return tmp;
  }

/**
This waits until there is data avaliable on the socket.
**/
IPaddress *Socket::GetPeerAddress()
  {
  if(TCP==NULL)
    {
    /* Severe error. Aborting execution. */
    mError=TRUE;
    return NULL;
    }

  return SDLNet_TCP_GetPeerAddress(TCP);
  }

/**
This reads data from the socket.
**/
Sint32 Socket::Recv(void *Data,Uint32 size_elem,Uint32 n_elems)
  {
  TempLock lock(mutex);

  if(TCP==NULL)
    {
    /* Severe error. Aborting execution. */
    mError=TRUE;
    return -1;
    }

  assert(Data!=NULL);

  if(Data==NULL)
    {
    /* Severe error. Aborting execution. */
    Log::Warning("Error on socket: Calling with Data NULL");
    mError=TRUE;
    return -1;
    }

  if(mError==TRUE)
    {
    Log::Warning("Error on socket before recieving");
    return -1;
    }
 

  Uint32 tmp=SDLNet_TCP_Recv(TCP,(char *)Data,size_elem*n_elems);
  if(tmp!=(int)size_elem*n_elems)
    {
    Log::Warning("Error on socket after recieving, Recieved %i bytes instead of %i bytes",tmp,size_elem*n_elems);
    mError=TRUE;
    return -1;
    }

  SocketBytesRecieved+=tmp;

  return tmp;
  }

/**
This writes data to the socket.
**/
Sint32 Socket::Send(void const* Data,Uint32 size_elem,Uint32 n_elems)
  {
  TempLock lock(mutex);

  if(TCP==NULL)
    {
    /* Severe error. Aborting execution. */
    mError=TRUE;
    return -1;
    }

  assert(Data!=NULL);

  if(Data==NULL)
    {
    /* Severe error. Aborting execution. */
    Log::Warning("Error on socket: Calling with Data NULL");
    mError=TRUE;
    return -1;
    }
  if(mError==TRUE)
    {
    Log::Warning("Error on socket before sending");
    return -1;
    }

  Uint32 tmp=SDLNet_TCP_Send(TCP,(char *)Data,size_elem*n_elems);
  if(tmp!=(int)size_elem*n_elems)
    {
    Log::Warning("Error on socket after sending, Send %i bytes instead of %i bytes",tmp,size_elem*n_elems);
    mError=TRUE;
    return -1;
    }

  SocketBytesSent+=tmp;

  return tmp;
  }

/**
This close the connection
**/
void Socket::Close()
  {
  SDLNet_TCP_Close(TCP);
  }


BOOL Socket::isError()
  {
  return mError;
  }

void Socket::SetError()
  {
  mError=TRUE;
  }

Uint32 Socket::GetBytesSent()
  {
  return SocketBytesSent;
  }

Uint32 Socket::GetBytesRecv()
  {
  return SocketBytesRecieved;
  }

Uint32 Socket::ResetBytesSent()
  {
  TempLock lock(mutex);

  Uint32 tmp=SocketBytesSent;
  SocketBytesSent=0;
  return tmp;
  }

Uint32 Socket::ResetBytesRecv()
  {
  TempLock lock(mutex);

  Uint32 tmp=SocketBytesRecieved;
  SocketBytesRecieved=0;
  return tmp;
  }
