#include "TerrainPiece.h"

TerrainPiece::TerrainPiece()
  {
  Terrains=new AList<Terrain>();
  x_init=y_init=x_end=y_end=0;
  }

TerrainPiece::~TerrainPiece()
  {
  delete Terrains;
  }


Uint32 TerrainPiece::GetSize()
  {
  return 4*sizeof(Uint32)+Terrains->GetSize();
  }

void TerrainPiece::ToVector(AVector *Vector)
  {
  assert(Vector!=NULL);
   
  AUint32 tmp;
  tmp=x_init;
  tmp.ToVector(Vector);
  tmp=y_init;
  tmp.ToVector(Vector);
  tmp=x_end;
  tmp.ToVector(Vector);
  tmp=y_end;
  tmp.ToVector(Vector);

  Terrains->ToVector(Vector);
  }

void TerrainPiece::FromVector(AVector *Vector)
  {
  assert(Vector!=NULL);
   
  AUint32 tmp;
  tmp.FromVector(Vector);
  x_init=tmp;
  tmp.FromVector(Vector);
  y_init=tmp;
  tmp.FromVector(Vector);
  x_end=tmp;
  tmp.FromVector(Vector);
  y_end=tmp;

  Terrains->FromVector(Vector);
  }

void TerrainPiece::Clear()
  {
  Terrains->Delete();
  }

BOOL TerrainPiece::isInside(Uint32 x,Uint32 y)
  {
  if(x>=x_init && x<x_end)
    {
    if(y>=y_init && y<y_end)
      {
      return TRUE;
      }
    }

  return FALSE;
  }

BOOL TerrainPiece::isInside(Terrain *piece)
  {
  assert(piece!=NULL);
   
  if(piece->x1>=x_init && piece->x1<x_end)
    {
    if(piece->y1>=y_init && piece->y1<y_end)
      {
      return TRUE;
      }
    }

  return FALSE;
  }


Terrain *TerrainPiece::GetTerrain(Uint32 x1,Uint32 y1,Uint32 z1)
  {
  for(Uint32 i=0;i<Terrains->Size();i++)
    {
    Terrain *tmp=(*Terrains)[i];
    if(tmp->x1==x1 && tmp->y1==y1 && tmp->z1==z1)
      {
      return tmp;
      }
    }
  
  return NULL;
  }

void TerrainPiece::AddTerrain(Terrain *terrain)
  {
  assert(terrain!=NULL);
   
  Terrains->Add(terrain);
  }

BOOL TerrainPiece::UpdateTerrain(Terrain *terrain)
  {
  assert(terrain!=NULL);
   
  for(Uint32 i=0;i<Terrains->Size();i++)
    {
    Terrain *tmp=(*Terrains)[i];
    if(tmp->x1==terrain->x1 && tmp->y1==terrain->y1 && tmp->z1==terrain->z1)
      {
      Terrains->Free(i);
      Terrains->Add(terrain);
      return TRUE;
      }
    }

  return FALSE;
  }

BOOL TerrainPiece::RemoveTerrain(Terrain *terrain)
  {
  assert(terrain!=NULL);
   
  for(Uint32 i=0;i<Terrains->Size();i++)
    {
    Terrain *tmp=(*Terrains)[i];
    if(tmp->x1==terrain->x1 && tmp->y1==terrain->y1 && tmp->z1==terrain->z1)
      {
      Terrains->Free(i);
      return TRUE;
      }
    }

  return FALSE;
  }

BOOL TerrainPiece::RemoveTerrain(Uint32 x1,Uint32 y1,Uint32 z1)
  {
  for(Uint32 i=0;i<Terrains->Size();i++)
    {
    Terrain *tmp=(*Terrains)[i];
    if(tmp->x1==x1 && tmp->y1==y1 && tmp->z1==z1)
      {
      Terrains->Free(i);
      return TRUE;
      }
    }

  return FALSE;
  }

BOOL TerrainPiece::isNeighbor(TerrainPiece *tmp)
  {
  assert(tmp!=NULL);
   
  Uint32 dx=(x_end-x_init)/2;
  Uint32 dy=(y_end-y_init)/2;
  
  if(tmp->isInside(x_init-dx,y_init-dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init,y_init-dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_end+dx,y_init-dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init-dx,y_init+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init+dx,y_init+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_end+dx,y_init+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init-dx,y_end+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_init,y_end+dy)==TRUE)
    {
    return TRUE;
    }

  if(tmp->isInside(x_end+dx,y_end+dy)==TRUE)
    {
    return TRUE;
    }

  return FALSE;
  }

void TerrainPiece::isInRange(AList<Terrain> *list,Uint32 x1,Uint32 y1,Uint32 x2,Uint32 y2)
  {
  assert(list!=NULL);
   
  if((x1<x_init && x2<x_init) || (x1>x_end && x2<x_end))
    {
    return;
    }

  if((y1<y_init && y2<y_init) || (y1>y_end && y2<y_end))
    {
    return;
    }

  for(Uint32 i=0;i<Terrains->Size();i++)
    {
    Terrain *tmp=(*Terrains)[i];

    if(tmp->isInside(x1,y1,x2,y2)==TRUE)
      {
      list->Add(tmp);
      }
    }
  }



Uint32 TerrainPiece::GetNumTerrains()
  {
  return Terrains->Size();
  }

