/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This class is a container for all the maps.
    It animate the world on each call to animate.
***************************************************************************/
#include "World.h"
#include "FilePath.h"
#include "Log.h"

World::World()
  {
  mapObjects=new MapObjects();
  mapTerrains=new MapTerrains();
  ListWorlds=new AList<WorldManagers>;
  }

World::~World()
  {
  delete mapObjects;
  delete mapTerrains;
  delete ListWorlds;
  }

BOOL World::SetWorld(AString DataPath,AString FileName)
  {
  BOOL result=TRUE;
  Uint32 LastTime=SDL_GetTicks();
  Log::Debug("Loading World from Disk");

  AString DataWorldFile=DataPath+FileName;
  FilePath path( DataWorldFile );
  path.getValidFilePath( DataWorldFile );

  DataWorldFile=DataWorldFile+".Config";

  FILE *Handle=fopen((const char *)(DataWorldFile),"rb");
  if(Handle==NULL)
    {
    Log::Error("ERROR: Unable to open file %s",(const char *)(DataWorldFile));
    }

  Uint32 NumPieces;
  char *Buffer=new char[80];

  fgets(Buffer,80,Handle);
  sscanf(Buffer,"%i",&NumPieces);
  
  for(Uint32 i=0;i<NumPieces;i++)
    {
    Uint32 x_init,y_init,x_end,y_end;
  
    fgets(Buffer,80,Handle);
    char *NewFileName=new char[256];
    sscanf(Buffer,"%i %i %i %i %s",&x_init,&y_init,&x_end,&y_end,NewFileName);

    /* By now we only create empty pieces, later we will load it */
    mapTerrains->CreateNewTerrainPiece(x_init,y_init,x_end,y_end);
    mapObjects->CreateNewObjectPiece(x_init,y_init,x_end,y_end);

    delete NewFileName;
    }

  delete Buffer;

  Log::Debug("Loading World from Disk --DONE in %f sec--",float(SDL_GetTicks()-LastTime)/1000.0);
  return result;
  }

BOOL World::LoadWorld(AString DataPath,AString FileName)
  {
  BOOL result=TRUE;
  Uint32 LastTime=SDL_GetTicks();
  Log::Debug("Loading World from Disk");

  AString DataWorldFile=DataPath+FileName;
  FilePath path( DataWorldFile );
  path.getValidFilePath( DataWorldFile );

  DataWorldFile=DataWorldFile+".Config";

  FILE *Handle=fopen((const char *)(DataWorldFile),"rb");
  if(Handle==NULL)
    {
    Log::Error("ERROR: Unable to open file %s",(const char *)(DataWorldFile));
    }

  Uint32 NumPieces;
  char *Buffer=new char[80];

  fgets(Buffer,80,Handle);
  sscanf(Buffer,"%i",&NumPieces);
  
  for(Uint32 i=0;i<NumPieces;i++)
    {
    Uint32 x_init,y_init,x_end,y_end;
  
    fgets(Buffer,80,Handle);
    char *NewFileName=new char[256];
    sscanf(Buffer,"%i %i %i %i %s",&x_init,&y_init,&x_end,&y_end,NewFileName);

    AString File_To_Load=DataPath+NewFileName;

    result&=mapTerrains->LoadPiece((const char *)File_To_Load,x_init,y_init,x_end,y_end);
    
    /* By now we only create empty pieces, later we will load it */
    mapObjects->CreateNewObjectPiece(x_init,y_init,x_end,y_end);
    delete NewFileName;
    }

  delete Buffer;

  Log::Debug("Loading World from Disk --DONE in %f sec--",float(SDL_GetTicks()-LastTime)/1000.0);
  return result;
  }

BOOL World::SaveWorld(AString DataPath,AString FileName)
  {
  BOOL result=TRUE;
  Uint32 LastTime=SDL_GetTicks();
  Log::Debug("Saving World to Disk");

  AString DataWorldFile=DataPath+FileName;
  FilePath path( DataWorldFile );
  path.getValidFilePath( DataWorldFile );

  DataWorldFile=DataWorldFile+".Config";

  FILE *Handle=fopen((const char *)(DataWorldFile),"rb");
  if(Handle==NULL)
    {
    Log::Error("ERROR: Unable to open file %s",(const char *)(DataWorldFile));
    }

  Uint32 NumPieces;
  char *Buffer=new char[80];

  fgets(Buffer,80,Handle);
  sscanf(Buffer,"%i",&NumPieces);
  
  for(Uint32 i=0;i<NumPieces;i++)
    {
    Uint32 x_init,y_init,x_end,y_end;
  
    fgets(Buffer,80,Handle);
    char *NewFileName=new char[256];
    sscanf(Buffer,"%i %i %i %i %s",&x_init,&y_init,&x_end,&y_end,NewFileName);

    AString File_To_Save=DataPath+NewFileName;

    result&=mapTerrains->SavePiece((const char *)File_To_Save,x_init,y_init,x_end,y_end);
    delete NewFileName;
    }

  delete Buffer;

  Log::Debug("Saving World to Disk --DONE in %f sec--",float(SDL_GetTicks()-LastTime)/1000.0);
  return result;
  }
  

void World::ClearObjects()
  {
  mapObjects->Clear();
  }

void World::ClearTerrains()
  {
  mapTerrains->Clear();
  }

AList<Object> *World::GetListObjects()
  {
  return mapObjects->GetListObjects();
  }

AList<Terrain> *World::GetListTerrains()
  {
  return mapTerrains->GetListTerrains();
  }


void World::UpdateTerrain(Terrain *terrain)
  {
  mapTerrains->UpdateTerrain(terrain);
  }

void World::UpdateObject(Object *object)
  {
  mapObjects->UpdateObject(object);
  }



AList<Terrain> *World::isInRange(Uint32 x1,Uint32 y1,Uint32 x2,Uint32 y2)
  {
  return mapTerrains->isInRange(x1,y1,x2,y2);
  }

AList<Object> *World::isInRange(Position& Center, Uint32 Range)
  {
  return mapObjects->isInRange(Center,Range);
  }

Object *World::GetObject(Uint32 uID)
  {
  return mapObjects->GetObject(uID);
  }

Object *World::GetObject(Object *object)
  {
  assert(object!=NULL);

  return mapObjects->GetObject(object);
  }

void World::AddObject(Object *object)
  {
  assert(object!=NULL);

  mapObjects->AddObject(object);
  }

void World::RemoveObject(Object *object)
  {
  assert(object!=NULL);

  mapObjects->RemoveObject(object);
  }

void World::RemoveObject(Uint32 uID)
  {
  mapObjects->RemoveObject(uID);
  }

Uint32 World::GetNumObjects()
  {
  return mapObjects->GetNumObjects();
  }

Uint32 World::GetNumTerrains()
  {
  return mapTerrains->GetNumTerrains();
  }

void World::AddWorldManagers(WorldManagers *wm)
  {
  assert(wm!=NULL);

  ListWorlds->Add(wm);
  }

void World::RemoveWorldManagers(Uint32 i)
  {
  WorldManagers *tmp=ListWorlds->Remove(i);
  delete tmp;
  }

Uint32 World::SizeWorldManagers()
  {
  return ListWorlds->Size();
  }

WorldManagers *World::GetWorldManagers(Uint32 i)
  {
  return (*ListWorlds)[i];
  }

void World::CallWorldManagers()
  {
  for(Uint32 i=0;i<ListWorlds->Size();i++)
    {
    (*ListWorlds)[i]->Animate(this);
    }
  }

void World::GetSize(Uint32 *x1,Uint32 *y1,Uint32 *x2,Uint32 *y2)
  {
  mapTerrains->GetSize(x1,y1,x2,y2);
  }
