/* common/astring.h

Project : Arianne RPG
Created : 27 Jun 2000

Simple string class without bells and whistles.

*/

#ifndef __ARIANNE_ASTRING_H
#define __ARIANNE_ASTRING_H
#ifndef __cplusplus
#error C++ compiler required.
#endif

#include "general.h"

// The simple string class
class AString 
{
protected:
    char *value;
    void assign(const char *newvalue);
public:
    AString(AString const& original);

    /** create an AString-object using sprintf()-like syntax
    TODO: determine the string-length while constructing the content to prevent
    writing past the end of the array
     */
    static AString Format(char const* format, ...);

    AString(const char *initvalue = NULL);

    AString(const char *head, const char *tail);

    ~AString(void) { assign(NULL); };

    operator const char *(void) const{ return value; };

    const char * operator= (AString const& newvalue)
	{ if (this != &newvalue) assign(newvalue); return value; };
	
    const char * operator= (const char *newvalue)
	{ if (newvalue != value) assign(newvalue); return value; };

    AString& operator+= (const char *newtail);

    bool operator==(char const* str)
    { if((!str) || (!value)) return false;
      return 0==strcmp(str,value);}

    friend AString operator+ (AString const& s1, const char *s2)
	{ return AString(s1,s2); };
	
    Uint16 GetSize()
      {
      return strlen(value)+2+1; 
      };

    void ToVector(AVector *Vector)
      {
      AVector *tmp=new AVector;
      tmp->SetVectorSize(GetSize());

      Uint8 Word[2];
      SDLNet_Write16(GetSize()-2,Word);
      tmp->AddUint8(Word,2);
  
      tmp->AddUint8((Uint8*)value,GetSize()-2);
      (*Vector)+tmp;
      delete tmp;
      }

    void FromVector(AVector *Vector)
      {
      Uint8 Word[2];
      Vector->GetUint8(Word,2);
      Uint16 length=SDLNet_Read16(Word);
      if(value)
        {
        delete[] value;
        }

      value=new char[length];
      Vector->GetUint8((Uint8*)value,length);
      }


    /** strdup()s this string, use free() to deallocate the retrieved array(not delete[]) !*/
    char *str();
};

#endif // __ARIANNE_ASTRING_H
