/* common/inifile.h

Project : Arianne RPG
Created : 27 Jun 2000

Implement basic Windows-style profile files.

*/

#ifndef __ARIANNE_INIFILE_H
#define __ARIANNE_INIFILE_H

#if defined(_Windows) || defined(__WIN32__) || defined(WIN32)

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define GetIniFileStr GetPrivateProfileString
#define PutIniFileStr WritePrivateProfileString

#else

#include <sys/types.h>

#ifndef BOOL
#define BOOL int
#endif

#ifndef FALSE
#define FALSE (0)
#endif

#ifndef TRUE
#define TRUE (1)
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#ifndef NUL
#define NUL ((char)0)
#endif

#ifdef __cplusplus
    extern "C" {
#endif

/*  Retrive a character string from the ini file in the buffer.
    If key==NULL it returns a double NUL terminated list of keys.
    The default value must never be NULL.
    Section and key names are case insensitive.
    Returns the number of characters placed in the buffer
*/
size_t GetIniFileStr(const char *section, const char *key, const char *defval,
	char *buffer, size_t buflen, const char *inifile);

/*  Store a character string in the ini file.
    If value==NULL it deletes the entire key.
    If key==NULL it deletes the entire section.
    Returns TRUE is successfull.
*/   
BOOL PutIniFileStr(const char *section, const char *key, const char *value,
	const char *inifile);

#ifdef __cplusplus
    }
#endif

#endif /* Windows */

#endif /* __ARIANNE_INIFILE_H */
