/***************************************************************************
                          mutex.h  -  description
                             -------------------
    begin                : Sat Jun 24 2000
    copyright            : (C) 2000 by Ulrich Eckhardt
    email                : Doomster@knuut.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MUTEX_H
#define MUTEX_H
/**Whichever kind of mutex we use, it should go in here.
danger: when implementing, this mutex must have a lock count and you should be able to
lock it several times with the same thread.
-> doesnt work with (native)SDL-threads !!!!

Proposal: take a struct with two mutices, the ID of the current owner(the thread who
locked it) and a reference counter. To lock it, you could lock the first, compare your own
ID with the owner and increase the ref-count if equal. If another thread owns it, youd release
the first and lock the second, thus forcing a schedule() on your own thread until the second is
unlocked.
To unlock, you again lock the first, decrement the ref-count and -if zero- unlock the second.
The first could even be a global^B^B^B^B^B^B static one (to reduce memory usage).
  *@author Ulrich Eckhardt
  */
#include <SDL/SDL_mutex.h>
class Mutex
{
public: 
	Mutex(bool locked = false);
	/** uncomment this virtual if you derive from it! */
	/*virtual*/ ~Mutex();
	int lock();
	int unlock();
private:
	/** do not use */
	Mutex(Mutex const&);
	/** do not use */
	Mutex& operator=(Mutex const&);
	
	SDL_mutex* m_SDLMutex;
};

#endif
