/* common/network.h

Project : Arianne RPG
Created : 26 Jun 2000

Everything common to both client and server to handle data exchange
over a network.

Currently supports TCP/IPv4 and UDP/IPv4

*/

#ifndef __ARIANNE_NETWORK_H
#define __ARIANNE_NETWORK_H
#ifndef __cplusplus
#error C++ compiler required.
#endif

#include "general.h"
#if defined(_Windows) || defined(__WIN32__) || defined(WIN32)
#include <winsock.h>
#else
#include <sys/socket.h>
#endif

class NetAddress {
public:
     virtual BOOL SetFromNumber(char *numeric_addr) = 0;
};

class IPv4Address : public NetAddress {
};

// Abstract base class
class NetConnection {
protected:
    int status;
    int virtual _write(const char *buf, size_t n);
    int virtual _read(char *buf, size_t n);
public:
    NetConnection();
    virtual ~NetConnection();
    BOOL virtual GetPeerName(char * address) = 0;
    void virtual KeepRunning(void);
    int GetStatus(void) { return status; };
    NetConnection& operator << (char c);
    NetConnection& operator << (short s);
    NetConnection& operator << (int i);
    friend NetConnection& ping(NetConnection& _this);
};

class TcpConnection : public NetConnection {
    BOOL virtual GetPeerName(char * address);
};

class UdpConnection : public NetConnection {
protected:
    sockaddr mypeer; // because we may share the socket
    int16_t conn_id; // used when masquerading
};

#endif // __ARIANNE_NETWORK_H
