/***************************************************************************
    copyright            : (C) 2000 by 
    email                : 
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* common/settings.cpp

Project : Arianne RPG
Created : 27 Jun 2000

C++ wrapper for Windows-style profile files.

*/

#include "settings.h"
#ifndef macintosh
#include <malloc.h>
#endif
#include <string.h>
#include <stdio.h>
#include <fstream>
#include <string>
#define LOAD_STR_SIZE  2048
#define LOAD_INT_SIZE  64

    /*  Retrive a character string from the ini file in the buffer.
    If key==NULL it returns a double NUL terminated list of keys.
    The default value must never be NULL.
    Section and key names are case insensitive.
    Returns the number of characters placed in the buffer
    */
size_t PersistentVar::GetIniFileStr(const char *section, const char *key,
        const char *defval, char *buffer,
        size_t buflen, const char *inifile)
{
    using std::string;
    std::ifstream fin(inifile);
    if(fin.bad())
        return 0;
    bool foundSection = false;
    do
    {
        // read a line
        string line;
        bool noEOF=true;
        while(('\n' != fin.peek())&&(noEOF==true))
        {
            if(fin.eof()!=0)
              {
              noEOF=false;
              break;
              }

            int c = fin.get();
            if(c!='\r')
              {
  	          line += c;
              }
		}
		// discard trailing 'newline'
        fin.get();
        if(fin.peek()=='\r')
          {
          fin.get();
          }
        

       // white line ?
        if(line.length() == line.find_first_not_of(' '))
            continue;

        // section ?
        if('[' == line[0])
        {
            // just a formal way, first should always be 1
            string::size_type first = line.find('[')+1;
            string::size_type last = line.rfind(']')-1;
            string sect = line.substr(first,last);
            // this will not only set 'foundSection' to true but also
            // reset it when entering a different section
            foundSection=(sect.compare(section)==0);
            continue;
        }

        // key
        if(foundSection==true)
        {
            // see if we found the requested key
            if(0 == line.find(key))
            {
                string rval = line.substr(strlen(key)+1);
                // make sure the given buffer is big enough
                // TODO: add suppport for resizable strings eg std::string or AString
                if(rval.length() <= buflen)
                {
                    strncpy(buffer,rval.c_str(),rval.length());
                    int tmp=rval.length();
                    buffer[tmp]=0;
                    return rval.length();
                }
                else
                {
                    return 0;
                }
            }
        }
    }while(fin.good());

    return 0;
}

    /*  Store a character string in the ini file.
    If value==NULL it deletes the entire key.
    If key==NULL it deletes the entire section.
    Returns TRUE is successfull.
    */
BOOL PersistentVar::PutIniFileStr(const char *section, const char *key,
        const char *value,	const char *inifile)
        {return TRUE;}

PersistentString::PersistentString(const char *fname, const char *block,
    const char *key, const char *defvalue)
{
    filename=fname;
    blockname=block;
    keyname=key;
    char *buf = (char *)malloc(LOAD_STR_SIZE);
    GetIniFileStr(blockname,keyname,defvalue ? defvalue : ""
	,buf,LOAD_STR_SIZE-1,filename);
    buf[LOAD_STR_SIZE-1]=NUL;
    value=buf;
    free(buf);
}

void PersistentString::Flush(void)
{
    const char *v = ((const char *)value) ? (const char *)value : "";
    PutIniFileStr(blockname,keyname,v,filename);
    dirty=FALSE;
}

const char * PersistentString::operator= (const char * newvalue)
{
    if (newvalue!=value || newvalue==NULL
	|| value==NULL || strcmp(newvalue,value)) {
	value=newvalue;
	dirty=TRUE;
    }
    return value;
}

PersistentInt::PersistentInt(const char *fname, const char *block,
    const char *key, int defvalue)
{
    filename=fname;
    blockname=block;
    keyname=key;
    char buf[LOAD_INT_SIZE];
    GetIniFileStr(blockname,keyname,"",buf,LOAD_INT_SIZE-1,filename);
    buf[LOAD_INT_SIZE-1]=NUL;
    value=defvalue;
    if (*buf) sscanf(buf,"%d",&value);
}

void PersistentInt::Flush(void)
{
    char buf[LOAD_INT_SIZE];
    sprintf(buf,"%d",value);
    PutIniFileStr(blockname,keyname,buf,filename);
    dirty=FALSE;
}

int PersistentInt::operator= (int newvalue)
{
    dirty|=(newvalue != value);
    value=newvalue;
    return value;
}
