/* common/settings.h

Project : Arianne RPG
Created : 27 Jun 2000

C++ wrapper for Windows-style profile files.

*/

#ifndef __ARIANNE_SETTINGS_H
#define __ARIANNE_SETTINGS_H
#ifndef __cplusplus
#error C++ compiler required.
#endif

#include "general.h"
//#include "inifile.h"
#include "astring.h"

// Abstract persistent variable class
class PersistentVar {
protected:
    AString filename, blockname, keyname;
    BOOL dirty;

    /*  Retrive a character string from the ini file in the buffer.
    If key==NULL it returns a double NUL terminated list of keys.
    The default value must never be NULL.
    Section and key names are case insensitive.
    Returns the number of characters placed in the buffer
    */
    size_t GetIniFileStr(const char *section, const char *key,
        const char *defval, char *buffer,
        size_t buflen, const char *inifile);

    /*  Store a character string in the ini file.
    If value==NULL it deletes the entire key.
    If key==NULL it deletes the entire section.
    Returns TRUE is successfull.
    */
    BOOL PutIniFileStr(const char *section, const char *key,
        const char *value,	const char *inifile);
public:
    PersistentVar(void) { dirty=FALSE; };
    virtual ~PersistentVar(){};

    const char *FileName() { return filename; };
    const char *Block() { return blockname; };
    const char *Key() { return keyname; };

    void virtual Flush(void) = 0;


    void Erase(void) {
        PutIniFileStr(blockname,keyname,NULL,filename);
        dirty=FALSE;
    };
};

// A persistent string
class PersistentString : public PersistentVar {
protected:
    AString value;
public:
    PersistentString(const char *fname, const char *block,
        const char *key, const char *defvalue = "");
    virtual ~PersistentString(void) { if (dirty) Flush(); };
    void virtual Flush(void);
    operator const char *(void) { return value; };
    const char * operator= (const char *newvalue);
};

// A persistent integer
class PersistentInt : public PersistentVar {
protected:
    int value;
public:
    PersistentInt(const char *fname, const char *block,
	const char *key, int defvalue = 0);
    virtual ~PersistentInt(void) { if (dirty) Flush(); };
    void virtual Flush(void);
    operator int(void) { return value; };
    int operator= (int newvalue);
};

#endif // __ARIANNE_SETTINGS_H
