/***************************************************************************
                          thread.cpp  -  description
                             -------------------
    begin                : Tue Jul 4 2000
    copyright            : (C) 2000 by Ulrich Eckhardt
    email                : Doomster@knuut.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    These are a thread abstraction class and a Loop thread abstraction 
    class.
***************************************************************************/

#include "thread.h"
#include "templock.h"
#include "Log.h"
#include <iostream>

#ifdef macintosh
#include "../MAC_FILES/thread_mac.h"
#endif
// end of addition

// Minami's addition
//	Entry function for ThreadManager thread
#ifdef macintosh
pascal void* macThreadFunction( void* threadPtr )
{
	Log::Debug("macThreadFunction() is called");		// Minami's debug
	if( Thread* thread = reinterpret_cast<Thread*>( threadPtr ))
	{
		thread->threadFunction_Mac();
	}
	
	return nil;					// Returns nil; is this right thing?
}
#endif
// end of addition

/** This is the global thread-function that is entered by every SDL_thread. It is always
given the address of a Thread-object as argument and simply calls its threadFunction().*/
extern "C"{
int /*Thread::*/commonThreadFunction(void* data)
{
	if(Thread* thread = reinterpret_cast<Thread*>(data))
	{
		// set the threadID
		thread->m_ID = SDL_ThreadID();
        //cerr << "commonFn: ID=" << thread->m_ID << endl;
		// start the action
		return thread->threadFunction();
	}
	else
	{
		throw "Thread::commonThreadFunction(void* data): called with data = 0";
	}

return -1;
}
           }
Thread::Thread():
	m_SDLThread(0),
	m_ID(0),
	m_autoDelete(false)
{
	Log::Debug("ctor Thread");
}

Thread::~Thread()
{
	Log::Debug("dtor: ID=%i",m_ID);
	if(m_SDLThread != 0)
		killThread();
}

void Thread::start()
  {
  #ifndef macintosh
  m_SDLThread = SDL_CreateThread(&commonThreadFunction, reinterpret_cast<void*>(this));
  #else
  OSErr	theErr = NewThread( kCooperativeThread, macThreadFunction, reinterpret_cast<void*>(this),
  							0, 0, nil, &m_threadID_Mac );
  #endif

  //m_SDLThread = SDL_CreateThread(&commonThreadFunction, reinterpret_cast<void*>(this));
  }

/** Get the 32-bit thread identifier for the thread */
Uint32 Thread::threadID() const
{
	return m_ID;
}
/** By setting the auto-delete flag you can create threads with new that will
automagically delete themselves after they finished their tasks. */
void Thread::setAutoDelete(bool flag)
{
	m_autoDelete = flag;
}
/** returns the status of the auto-delete flag */
bool Thread::autoDelete() const
{	
	return m_autoDelete;
}
/** Wait for a thread to finish. The return code for the thread function
is placed in the area pointed to by 'status', if 'status' is not NULL.*/
void Thread::waitThread(int *status/*= 0*/)
{
//	TempLock lock(m_mutex);
//	cerr << "waitThread: ID=" << threadID() << endl;
	
	SDL_WaitThread(m_SDLThread, status);
	m_SDLThread = 0;
	if(m_autoDelete)
		delete this;
}
/** Forcefully kill a thread without worrying about its state */
void Thread::killThread()
{
//	TempLock lock(m_mutex);
//	cerr << "killThread: ID=" << threadID() << endl;
	
	SDL_KillThread(m_SDLThread);
	m_SDLThread = 0;
	#ifdef macintosh
	::DisposeThread( m_threadID_Mac, nil, true );
	#endif
	if(m_autoDelete)
		delete this;
}
	                           	
LoopThread::LoopThread():
	m_suspend(true),
	m_sync(false),
	m_suspended(true),
	m_terminate(false),
	m_running(false)
{
    Thread::start();
}
LoopThread::~LoopThread()
{
}

/** the thread is created in suspended state, this method starts it*/
void LoopThread::start()
{
	TempLock lock(m_sync);
	if(m_suspended)
	{
		m_suspended = false;
		m_suspend.unlock();
	}
	if(!m_running)
	{
		m_running = true;
		m_suspend.unlock();
	}
}
/** this method suspends the thread after the current loop()-call
returns */
void LoopThread::suspend()
{
	TempLock lock(m_sync);
	if(! m_suspended)
	{
		m_suspended = true;
		m_suspend.lock();
	}
	m_running=false;
}
/** this method terminates the thread after the current loop()-call
returns */
int LoopThread::stop()
{
	m_terminate = true;
	m_running=false;
	// start the thread if not already running
	start();
	waitThread();
	 
	return 0;
}
int LoopThread::threadFunction()
{
	// wait until the mutex is unlocked and then start the loop
	m_suspend.lock();
	m_suspend.unlock();

	while(! m_terminate)
	{
		loop();
		// Minami's change
		// On MacOS, we are not using Mutex architecture;
		// Instead, we use ThreadManager functions
		#ifdef macintosh
		::YieldToAnyThread();
		#else	// Below are the original lines
		// lock the mutex temporarily. This will wait here if someone called
		// suspend() in the meantime
		m_suspend.lock();
		m_suspend.unlock();
		#endif
	}
	return 0;
}

// Minami's addition
// In order to call threadFunction() from outside of Thread/LoopThread function,
// I've made this wrapper function.
#ifdef macintosh
int LoopThread::threadFunction_Mac()
{
	threadFunction();
	return 0;
}
#endif
// end of addition


/** returns true if the Thread is running */
bool LoopThread::isRunning()
{
    return m_running;
}

