#include <stdio.h>

#include <string.h>



#define TRUE  1

#define FALSE 0



/**

These are the two kind of types we store on the symbol table.

**/

typedef enum EntryST_Type

  {

  message,mestype,messubtype,mespriority,var

  }

  EntryST_Type;



typedef struct Descriptor

  {

  int value;

  char *name;

  char *type;

  }

  Descriptor;



/**

A cell of the symbol table.

**/

typedef struct EntryST

  {

  EntryST_Type type;

  union

    {

    Descriptor Message;

    Descriptor Type;

    Descriptor Subtype;

    Descriptor Priority;

    Descriptor Var;

    }

    des;

  }

  EntryST;



/**

We redefine the standart type of Lex.

**/

#define YYSTYPE Descriptor



/**

We now should define the symbol table where symbols are stored.

**/



#define ST_SIZE 4096

EntryST Table[ST_SIZE+1];

int TableIndex,tmpIndex;



/**

Initialize the symbol table.

**/

void Init()

  {

  TableIndex=0;

  }



/**

Start a new Message.

**/

void NewMessage(char *Name)

  {

  if(TableIndex<ST_SIZE)

    {

    Table[TableIndex].type=message;

    Table[TableIndex].des.Message.name=strdup(Name);

    Table[TableIndex].des.Message.type=NULL;

    TableIndex++;

    }

  else

    {

    fprintf(stderr,"COMPILER INTERNAL ERROR: Table overflow.\nLINE: %i\tFILE: %s \n",__LINE__,__FILE__);

    exit(1);

    }

  }



/**

Finish a Message declaration

**/

void FinishMessage()

  {

  TableIndex--;

  while(Table[TableIndex].type!=message)

    {

    TableIndex--;



    if(TableIndex<0)

      {

      fprintf(stderr,"COMPILER INTERNAL ERROR: Bad Grammar\nLINE: %i\tFILE: %s \n",__LINE__,__FILE__);

      exit(1);

      }

    }

  }



/**

Add a variable to the symbol table.

**/

void Add(EntryST_Type Type,char *name,char *type)

  {

  if(TableIndex<ST_SIZE)

    {

    Table[TableIndex].type=Type;

	switch(Type)

	  {

	  case mestype:

        Table[TableIndex].des.Type.name=strdup(name);

        Table[TableIndex].des.Type.type=strdup(type);

		break;

	  case messubtype:

        Table[TableIndex].des.Subtype.name=strdup(name);

        Table[TableIndex].des.Subtype.type=strdup(type);

		break;

	  case mespriority:

        Table[TableIndex].des.Priority.name=strdup(name);

        Table[TableIndex].des.Priority.type=strdup(type);

		break;

	  case var:

        Table[TableIndex].des.Var.name=strdup(name);

        Table[TableIndex].des.Var.type=strdup(type);

		break;

	  default:

	    fprintf(stderr,"COMPILER INTERNAL ERROR: Invalid symbol table type\n");

		exit(1);

	  }

    TableIndex++;

    }

  else

    {

    fprintf(stderr,"COMPILER INTERNAL ERROR: Table overflow.\nLINE: %i\tFILE: %s \n",__LINE__,__FILE__);

    exit(1);

    }

  }





/**

Look for a symbol with the name name

**/

EntryST *Find(char *name)

  {

  int i=TableIndex-1;



  while(i>0)

    {

    if(Table[i].type==message)

      {

      if(strcmp(Table[i].des.Message.name,name)==0)

        {

        return &(Table[i]);

        }

	  else

		{

        return NULL;

        }

      }



    if(Table[i].type==var)

      {

      if(strcmp(Table[i].des.Var.name,name)==0)

        {

        return &(Table[i]);

        }

      }



    if(Table[i].type==mespriority)

      {

      if(strcmp(Table[i].des.Priority.name,name)==0)

        {

        return &(Table[i]);

        }

      }



    if(Table[i].type==mestype)

      {

      if(strcmp(Table[i].des.Type.name,name)==0)

        {

        return &(Table[i]);

        }

      }



    if(Table[i].type==messubtype)

      {

      if(strcmp(Table[i].des.Subtype.name,name)==0)

        {

        return &(Table[i]);

        }

      }



    i--;

    }



  return NULL;

  }



char *FindMessageName()

  {

  int i;

  for(i=0;i<TableIndex;i++)

    {

	if(Table[i].type==message)

	  {

	  return Table[i].des.Message.name;

	  }

    }



  return NULL;

  }



char *FindTypeName()

  {

  int i;

  for(i=0;i<TableIndex;i++)

    {

	if(Table[i].type==mestype)

	  {

	  return Table[i].des.Type.name;

	  }

    }



  return NULL;

  }



char *FindSubtypeName()

  {

  int i;

  for(i=0;i<TableIndex;i++)

    {

	if(Table[i].type==messubtype)

	  {

	  return Table[i].des.Subtype.name;

	  }

    }



  return NULL;

  }



char *FindPriorityName()

  {

  int i;

  for(i=0;i<TableIndex;i++)

    {

	if(Table[i].type==mespriority)

	  {

	  return Table[i].des.Priority.name;

	  }

    }



  return NULL;

  }



Descriptor *FindVariable()

  {

  int i;

  for(i=0;i<TableIndex;i++)

    {

	if(Table[i].type==var)

	  {

	  tmpIndex=i+1;

	  return &(Table[i].des.Var);

	  }

    }



  return NULL;

  }



Descriptor *FindNextVariable()

  {

  int i;

  for(i=tmpIndex;i<TableIndex;i++)

    {

	if(Table[i].type==var)

	  {

	  tmpIndex=i+1;

	  return &(Table[i].des.Var);

	  }

    }



  return NULL;

  }



