/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This class generate a report on the server.
***************************************************************************/
#include "ReportGenerator.h"
#include "../common/Log.h"
#include <time.h>


ReportGenerator::ReportGenerator()
  {
  SecondsSinceStart=0;
  NumPlayersSinceStart=0;
  world=NULL;
  netMan=NULL;
  DataSent=DataRecv=0;
  
  ErrorsOnZombieCharacters=0;
  NumPlayers=0;

  OS_Linux=0;
  OS_Windows=0;
  OS_BeOS=0;
  OS_MacOS=0;
  OS_Unknown=0;

  ServerDescription="Miguel's Arianne Server";
  
  time_t long_time;
  time(&long_time);
  struct tm *Localtime=localtime(&long_time);
  StartUpTime=asctime(Localtime);
  }

ReportGenerator::~ReportGenerator()
  {
  }

void ReportGenerator::Show()
  {
  Log::Debug(" -------------- REPORT ---------------");
  Log::Debug("|  Started  : %s",(const char *)(StartUpTime));
  Log::Debug("|  UpTime   : %f seconds",SecondsSinceStart);
  Log::Debug("|  Players Info:");
  Log::Debug("|    Players: %i ",NumPlayers);
  Log::Debug("|    AI     : %i (DISABLED)",0);    
  Log::Debug("|    Total  : %i",world->GetNumObjects());
  Log::Debug("|  Data since begin of the service: ");
  Log::Debug("|    Players  : %i",NumPlayersSinceStart);          

  for(int i=0;i<MAX_CONNECTIONS_PER_SERVER;i++)
    {
    Socket *socket=netMan->GetSocket(i);
    if(socket!=NULL)
      {
      DataSent+=socket->ResetBytesSent();
      DataRecv+=socket->ResetBytesRecv();
      }
    }

  Log::Debug("|    Data Sent: %i    \t(%f Kb/s)",DataSent,((float)DataSent/1024.0)/(float)SecondsSinceStart);          
  Log::Debug("|    Data Recv: %i    \t(%f Kb/s)",DataRecv,((float)DataRecv/1024.0)/(float)SecondsSinceStart);          
  Log::Debug("|  Errors: ");          
  Log::Debug("|    Errors   : %i (on Sockets)",netMan->GetErrors());          
  Log::Debug("|    Errors   : %i (on Zombies)",ErrorsOnZombieCharacters);          
  Log::Debug("|  Systems stats:");          
  Log::Debug("|    Linux    : %i",OS_Linux);          
  Log::Debug("|    Windows  : %i",OS_Windows);          
  Log::Debug("|    BeOS     : %i",OS_BeOS);          
  Log::Debug("|    MacOS    : %i",OS_MacOS);          
  Log::Debug("|    Unknown  : %i",OS_Unknown);          

  Log::Debug(" -------------- REPORT ---------------");

  FILE *Handle=fopen(DataLoader::GetReportFile(),"wb");
  if(Handle!=NULL)
    {
    fprintf(Handle," -------------- REPORT ---------------\n");
    fprintf(Handle,"|  Started  : %s",(const char *)(StartUpTime));
    fprintf(Handle,"|  UpTime: %f seconds\n",SecondsSinceStart);
    fprintf(Handle,"|  Players Info:\n");
    fprintf(Handle,"|    Players: %i  \n",NumPlayers);
    fprintf(Handle,"|    AI     : %i  (DISABLED)\n",0);    
    fprintf(Handle,"|    Total  : %i\n",world->GetNumObjects());
    fprintf(Handle,"|  Data since begin of the service: \n");
    fprintf(Handle,"|    Players  : %i\n",NumPlayersSinceStart);          
    fprintf(Handle,"|    Data Sent: %i    \t(%f Kb/s)\n",DataSent,((float)DataSent/1024.0)/(float)SecondsSinceStart);          
    fprintf(Handle,"|    Data Recv: %i    \t(%f Kb/s)\n",DataRecv,((float)DataRecv/1024.0)/(float)SecondsSinceStart);          
    fprintf(Handle,"|  Errors: \n");          
    fprintf(Handle,"|    Errors   : %i (on Sockets)\n",netMan->GetErrors());          
    fprintf(Handle,"|    Errors   : %i (on Zombies)\n",ErrorsOnZombieCharacters);          
    fprintf(Handle,"|  Systems stats:\n");          
    fprintf(Handle,"|    Linux    : %i\n",OS_Linux);          
    fprintf(Handle,"|    Windows  : %i\n",OS_Windows);          
    fprintf(Handle,"|    BeOS     : %i\n",OS_BeOS);          
    fprintf(Handle,"|    MacOS    : %i\n",OS_MacOS);          
    fprintf(Handle,"|    Unknown  : %i\n",OS_Unknown);          
    fprintf(Handle," -------------- REPORT ---------------\n");

    fclose(Handle);
    }   

  GenerateHTML();
  }

void ReportGenerator::IncTime(float time)
  {
  SecondsSinceStart+=time;
  }

void ReportGenerator::IncPlayersSinceStart()
  {
  NumPlayersSinceStart++;
  }

void ReportGenerator::SetWorld(World *world)
  {
  this->world=world;
  }

void ReportGenerator::SetNetworkServerManager(NetworkServerManager *netMan)
  {
  this->netMan=netMan;

  #ifndef macintosh
    ServerIPAddress=netMan->GetSocket()->ResolveIP();
  #else
    ServerIPAddress="(TODO in MacOS.\n)";
  #endif
  }

void ReportGenerator::SetErrorsOnZombieCharacters(Uint32 i)
  {
  ErrorsOnZombieCharacters=i;
  }

void ReportGenerator::SetNumPlayers(Uint32 i)
  {
  NumPlayers=i;
  }

void ReportGenerator::AddSystemInfoStats(AString OS,AString CPU,AString Mem)
  {
  if(OS=="Linux") 
    {
    OS_Linux++;    
    goto gotoCPU;
    }

  if(OS=="Windows") 
    {
    OS_Windows++;    
    goto gotoCPU;
    }

  if(OS=="BeOS") 
    {
    OS_BeOS++;    
    goto gotoCPU;
    }
  
  if(OS=="MacOS") 
    {
    OS_MacOS++;    
    goto gotoCPU;
    }

  OS_Unknown++;

  gotoCPU:;
  }
