/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
     This program should convert from a height map in ascii format to the map
     format that we are going to use on the game.
***************************************************************************/
#include "../common/general.h"
#include "../common/terrain.h"
#include "../common/MapTerrain.h"
#include "../common/Log.h"
#include "../common/FilePath.h"

enum TYPE_OF_TERRAIN {
  BLUE=0,
  DARK=1,
  WHITE=2
  }TYPE_OF_TERRAIN;

Uint32 GetTypeOfTerrain(Uint32 x,Uint32 y)
  {
  return DARK;
  }

int main(int Num,char **arg)
  {
#ifdef macintosh
	freopen("stderr.txt","w",stderr);
	freopen("stdout.txt","w",stdout);
#endif
  if(SDL_Init(SDL_INIT_NOPARACHUTE)<0)
    {
    Log::Error("Can't init SDL library.\n");
    }
  atexit(SDL_Quit);

  if(Num<3)
    {
    Log::Error("USAGE: ascii2map SideSizeInTiles NameAsciiMap MetersBetweenCells\n");
    return 0;
    }
  
  Uint16 Side=atoi(arg[1]);
  Uint16 PieceSide=atoi(arg[2]);

  Uint32 HeigthMap;
  Uint32 UnitsInCell=atoi(arg[4])*10; /* We use as basic unit the 10 cm. */

  AString FileName=arg[3];
  FileName=FileName+".Config";

  AString DataWorldFile=FileName;
  FilePath path( DataWorldFile );
  path.getValidFilePath( DataWorldFile );

  FILE *Handle=fopen((const char *)FileName,"wb");
  if(Handle==NULL)
    {
    Log::Error("ERROR: Unable to open file %s",arg[3]);
    return 0;
    }

  fprintf(Handle,"%i\n",(Side/PieceSide)*(Side/PieceSide));

  MapTerrains *mapTerrain=new MapTerrains();
  Uint32 i=0;

  for(Uint32 j=0;j<(Uint32)(Side/PieceSide);j++)
    {
    for(Uint32 k=0;k<(Uint32)(Side/PieceSide);k++)
      {
      char Buffer[256];

      sprintf(Buffer,"%s_%05i",arg[3],i);i++;

      mapTerrain->CreateNewTerrainPiece(j*PieceSide*UnitsInCell,k*PieceSide*UnitsInCell,
                                (j+1)*PieceSide*UnitsInCell,(k+1)*PieceSide*UnitsInCell);
    
      fprintf(Handle,"%i %i %i %i  %s\n",j*PieceSide*UnitsInCell,k*PieceSide*UnitsInCell,
                                (j+1)*PieceSide*UnitsInCell,(k+1)*PieceSide*UnitsInCell,Buffer);
      }
    }

  fclose(Handle);

  /* Now we try to open the heigth map file */
  Handle=fopen(arg[3],"rb");
  if(Handle==NULL)
    {
    Log::Error("ERROR: Unable to open file %s",arg[3]);
    return 0;
    }

  Log::Debug("Creating new terrain: ");
  Uint32 lasttime=SDL_GetTicks();

  for(i=0;i<(Uint32)(Side*Side);i++)
    {
    fscanf(Handle,"%i",&HeigthMap);

    Terrain *Piece=new Terrain();

    Piece->x1=(i%Side)*UnitsInCell;
    Piece->y1=(i/Side)*UnitsInCell;
    Piece->z1=HeigthMap;
    Piece->sizex=Piece->sizey=UnitsInCell;
    Piece->sizez=0;
    Piece->Flags=0;
    Piece->TextureIndex=GetTypeOfTerrain(i%Side,i/Side);

    mapTerrain->AddTerrain(Piece);
    if(i%256==0) putc('.',stdout);
    }
  fclose(Handle);

  Log::Debug("\n-DONE-   TIME: %f",(float)(SDL_GetTicks()-lasttime)/1000.0);
  lasttime=SDL_GetTicks();

  Log::Debug("Saving the terrain: ");

  Handle=fopen((const char *)(DataWorldFile),"rb");
  if(Handle==NULL)
    {
    Log::Error("ERROR: Unable to open file %s",(const char *)(DataWorldFile));
    }

  Uint32 NumPieces;
  char *Buffer=new char[80];

  fgets(Buffer,80,Handle);
  sscanf(Buffer,"%i",&NumPieces);
  
  for(i=0;i<NumPieces;i++)
    {
    Uint32 x_init,y_init,x_end,y_end;
  
    fgets(Buffer,80,Handle);
    char *NewFileName=new char[256];
    sscanf(Buffer,"%i %i %i %i %s",&x_init,&y_init,&x_end,&y_end,NewFileName);

    mapTerrain->SavePiece(NewFileName,x_init,y_init,x_end,y_end);
    delete NewFileName;
    }

  delete Buffer;

  Log::Debug("-DONE-   TIME: %f",(float)(SDL_GetTicks()-lasttime)/1000.0);
  lasttime=SDL_GetTicks();

  return 0;
  }

